@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -63,7 +63,7 @@ bool CChan::WriteConfig(CFile& File) {
 	if (!GetDefaultModes().empty())
 		m_pUser->PrintLine(File, "\tModes", GetDefaultModes());
 
-	MODULECALL(OnWriteChanConfig(File, *this), m_pUser, NULL,);
+	MODULECALL(OnWriteChanConfig(File, *this), m_pUser, NULL, NOTHING);
 
 	File.Write("\t</Chan>\n");
 	return true;
@@ -71,7 +71,7 @@ bool CChan::WriteConfig(CFile& File) {
 
 void CChan::Clone(CChan& chan) {
 	// We assume that m_sName and m_pUser are equal
-	SetBufferCount(chan.GetBufferCount());
+	SetBufferCount(chan.GetBufferCount(), true);
 	SetKeepBuffer(chan.KeepBuffer());
 	SetKey(chan.GetKey());
 	SetDefaultModes(chan.GetDefaultModes());
@@ -91,6 +91,13 @@ void CChan::Clone(CChan& chan) {
 	}
 }
 
+bool CChan::SetBufferCount(unsigned int u, bool bForce) {
+	if (!bForce && u > CZNC::Get().GetMaxBufferSize())
+		return false;
+	m_uBufferCount = u;
+	return true;
+}
+
 void CChan::Cycle() const {
 	m_pUser->PutIRC("PART " + GetName() + "\r\nJOIN " + GetName() + " " + GetKey());
 }
@@ -98,6 +105,7 @@ void CChan::Cycle() const {
 void CChan::JoinUser(bool bForce, const CString& sKey, CClient* pClient) {
 	if (!bForce && (!IsOn() || !IsDetached())) {
 		m_pUser->PutIRC("JOIN " + GetName() + " " + ((sKey.empty()) ? GetKey() : sKey));
+		SetDetached(false);
 		return;
 	}
 
@@ -120,18 +128,18 @@ void CChan::JoinUser(bool bForce, const CString& sKey, CClient* pClient) {
 		else
 			pThisClient = pClient;
 
-		for (map<CString,CNick*>::iterator a = m_msNicks.begin(); a != m_msNicks.end(); ++a) {
+		for (map<CString,CNick>::iterator a = m_msNicks.begin(); a != m_msNicks.end(); ++a) {
 			if (pThisClient->HasNamesx()) {
-				sPerm = a->second->GetPermStr();
+				sPerm = a->second.GetPermStr();
 			} else {
-				char c = a->second->GetPermChar();
+				char c = a->second.GetPermChar();
 				sPerm = "";
 				if (c != '\0') {
 					sPerm += c;
 				}
 			}
-			if (pThisClient->HasUHNames() && !a->second->GetIdent().empty() && !a->second->GetHost().empty()) {
-				sNick = a->first + "!" + a->second->GetIdent() + "@" + a->second->GetHost();
+			if (pThisClient->HasUHNames() && !a->second.GetIdent().empty() && !a->second.GetHost().empty()) {
+				sNick = a->first + "!" + a->second.GetIdent() + "@" + a->second.GetHost();
 			} else {
 				sNick = a->first;
 			}
@@ -212,15 +220,23 @@ void CChan::OnWho(const CString& sNick, const CString& sIdent, const CString& sH
 	}
 }
 
-void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
+void CChan::ModeChange(const CString& sModes, const CNick* pOpNick) {
 	CString sModeArg = sModes.Token(0);
 	CString sArgs = sModes.Token(1, true);
 	bool bAdd = true;
 
-	CNick* pOpNick = FindNick(sOpNick);
+	/* Try to find a CNick* from this channel so that pOpNick->HasPerm()
+	 * works as expected. */
+	if (pOpNick) {
+		CNick* OpNick = FindNick(pOpNick->GetNick());
+		/* If nothing was found, use the original pOpNick, else use the
+		 * CNick* from FindNick() */
+		if (OpNick)
+			pOpNick = OpNick;
+	}
 
 	if (pOpNick) {
-		MODULECALL(OnRawMode(*pOpNick, *this, sModeArg, sArgs), m_pUser, NULL, );
+		MODULECALL(OnRawMode(*pOpNick, *this, sModeArg, sArgs), m_pUser, NULL, NOTHING);
 	}
 
 	for (unsigned int a = 0; a < sModeArg.size(); a++) {
@@ -253,19 +269,19 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 					bool bNoChange = (pNick->HasPerm(uPerm) == bAdd);
 
 					if (uMode && pOpNick) {
-						MODULECALL(OnChanPermission(*pOpNick, *pNick, *this, uMode, bAdd, bNoChange), m_pUser, NULL, );
+						MODULECALL(OnChanPermission(*pOpNick, *pNick, *this, uMode, bAdd, bNoChange), m_pUser, NULL, NOTHING);
 
 						if (uMode == CChan::M_Op) {
 							if (bAdd) {
-								MODULECALL(OnOp(*pOpNick, *pNick, *this, bNoChange), m_pUser, NULL, );
+								MODULECALL(OnOp(*pOpNick, *pNick, *this, bNoChange), m_pUser, NULL, NOTHING);
 							} else {
-								MODULECALL(OnDeop(*pOpNick, *pNick, *this, bNoChange), m_pUser, NULL, );
+								MODULECALL(OnDeop(*pOpNick, *pNick, *this, bNoChange), m_pUser, NULL, NOTHING);
 							}
 						} else if (uMode == CChan::M_Voice) {
 							if (bAdd) {
-								MODULECALL(OnVoice(*pOpNick, *pNick, *this, bNoChange), m_pUser, NULL, );
+								MODULECALL(OnVoice(*pOpNick, *pNick, *this, bNoChange), m_pUser, NULL, NOTHING);
 							} else {
-								MODULECALL(OnDevoice(*pOpNick, *pNick, *this, bNoChange), m_pUser, NULL, );
+								MODULECALL(OnDevoice(*pOpNick, *pNick, *this, bNoChange), m_pUser, NULL, NOTHING);
 							}
 						}
 					}
@@ -301,7 +317,7 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 			} else {
 				bNoChange = !HasMode(uMode);
 			}
-			MODULECALL(OnMode(*pOpNick, *this, uMode, sArg, bAdd, bNoChange), m_pUser, NULL, );
+			MODULECALL(OnMode(*pOpNick, *this, uMode, sArg, bAdd, bNoChange), m_pUser, NULL, NOTHING);
 
 			if (!bList) {
 				(bAdd) ? AddMode(uMode, sArg) : RemMode(uMode);
@@ -361,10 +377,6 @@ CString CChan::GetModeArg(CString& sArgs) const {
 }
 
 void CChan::ClearNicks() {
-	for (map<CString,CNick*>::iterator a = m_msNicks.begin(); a != m_msNicks.end(); ++a) {
-		delete a->second;
-	}
-
 	m_msNicks.clear();
 }
 
@@ -426,7 +438,7 @@ bool CChan::AddNick(const CString& sNick) {
 		}
 	}
 
-	m_msNicks[pNick->GetNick()] = pNick;
+	m_msNicks[pNick->GetNick()] = *pNick;
 
 	return true;
 }
@@ -434,9 +446,9 @@ bool CChan::AddNick(const CString& sNick) {
 map<char, unsigned int> CChan::GetPermCounts() const {
 	map<char, unsigned int> mRet;
 
-	map<CString,CNick*>::const_iterator it;
+	map<CString,CNick>::const_iterator it;
 	for (it = m_msNicks.begin(); it != m_msNicks.end(); ++it) {
-		CString sPerms = it->second->GetPermStr();
+		CString sPerms = it->second.GetPermStr();
 
 		for (unsigned int p = 0; p < sPerms.size(); p++) {
 			mRet[sPerms[p]]++;
@@ -447,7 +459,7 @@ map<char, unsigned int> CChan::GetPermCounts() const {
 }
 
 bool CChan::RemNick(const CString& sNick) {
-	map<CString,CNick*>::iterator it;
+	map<CString,CNick>::iterator it;
 	set<unsigned char>::iterator it2;
 
 	it = m_msNicks.find(sNick);
@@ -455,21 +467,20 @@ bool CChan::RemNick(const CString& sNick) {
 		return false;
 	}
 
-	delete it->second;
 	m_msNicks.erase(it);
 
 	return true;
 }
 
 bool CChan::ChangeNick(const CString& sOldNick, const CString& sNewNick) {
-	map<CString,CNick*>::iterator it = m_msNicks.find(sOldNick);
+	map<CString,CNick>::iterator it = m_msNicks.find(sOldNick);
 
 	if (it == m_msNicks.end()) {
 		return false;
 	}
 
 	// Rename this nick
-	it->second->SetNick(sNewNick);
+	it->second.SetNick(sNewNick);
 
 	// Insert a new element into the map then erase the old one, do this to change the key to the new nick
 	m_msNicks[sNewNick] = it->second;
@@ -478,9 +489,14 @@ bool CChan::ChangeNick(const CString& sOldNick, const CString& sNewNick) {
 	return true;
 }
 
-CNick* CChan::FindNick(const CString& sNick) const {
-	map<CString,CNick*>::const_iterator it = m_msNicks.find(sNick);
-	return (it != m_msNicks.end()) ? it->second : NULL;
+const CNick* CChan::FindNick(const CString& sNick) const {
+	map<CString,CNick>::const_iterator it = m_msNicks.find(sNick);
+	return (it != m_msNicks.end()) ? &it->second : NULL;
+}
+
+CNick* CChan::FindNick(const CString& sNick) {
+	map<CString,CNick>::iterator it = m_msNicks.find(sNick);
+	return (it != m_msNicks.end()) ? &it->second : NULL;
 }
 
 int CChan::AddBuffer(const CString& sLine) {
