@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -67,9 +67,7 @@ void CClient::ReadLine(const CString& sData) {
 	if (IsAttached()) {
 		MODULECALL(OnUserRaw(sLine), m_pUser, this, return);
 	} else {
-		if (CZNC::Get().GetModules().OnUnknownUserRaw(this, sLine)) {
-			return;
-		}
+		GLOBALMODULECALL(OnUnknownUserRaw(sLine), m_pUser, this, return);
 	}
 
 	CString sCommand = sLine.Token(0);
@@ -92,10 +90,7 @@ void CClient::ReadLine(const CString& sData) {
 				m_sPass = m_sPass.Token(1, true, ":");
 			}
 
-			if ((m_bGotNick) && (m_bGotUser)) {
-				AuthUser();
-			}
-
+			AuthUser();
 			return;  // Don't forward this msg.  ZNC has already registered us.
 		}
 	} else if (sCommand.Equals("NICK")) {
@@ -108,9 +103,7 @@ void CClient::ReadLine(const CString& sData) {
 			m_sNick = sNick;
 			m_bGotNick = true;
 
-			if ((m_bGotPass) && (m_bGotUser)) {
-				AuthUser();
-			}
+			AuthUser();
 			return;  // Don't forward this msg.  ZNC will handle nick changes until auth is complete
 		}
 	} else if (sCommand.Equals("USER")) {
@@ -121,9 +114,9 @@ void CClient::ReadLine(const CString& sData) {
 
 			m_bGotUser = true;
 
-			if ((m_bGotPass) && (m_bGotNick)) {
+			if (m_bGotPass) {
 				AuthUser();
-			} else if (!m_bGotPass) {
+			} else {
 				PutClient(":irc.znc.in NOTICE AUTH :*** "
 					"You need to send your password. "
 					"Try /quote PASS <username>:<password>");
@@ -131,10 +124,16 @@ void CClient::ReadLine(const CString& sData) {
 
 			return;  // Don't forward this msg.  ZNC has already registered us.
 		}
+	} else if (sCommand.Equals("CAP")) {
+		HandleCap(sLine);
+
+		// Don't let the client talk to the server directly about CAP,
+		// we don't want anything enabled that ZNC does not support.
+		return;
 	}
 
 	if (!m_pUser) {
-		// Only NICK, USER and PASS are allowed before login
+		// Only CAP, NICK, USER and PASS are allowed before login
 		return;
 	}
 
@@ -179,17 +178,15 @@ void CClient::ReadLine(const CString& sData) {
 		PutStatusNotice("Detached from [" + sChan + "]");
 		return;
 	} else if (sCommand.Equals("PING")) {
-		CString sTarget = sLine.Token(1);
-
-		// If the client meant to ping us or we can be sure the server
-		// won't answer the ping (=no server connected) -> PONG back.
-		// else: It's the server's job to send a PONG.
-		if (sTarget.Equals("irc.znc.in") || !GetIRCSock()) {
-			PutClient("PONG " + sLine.substr(5));
-			return;
-		}
+		// All PONGs are generated by znc. We will still forward this to
+		// the ircd, but all PONGs from irc will be blocked.
+		if (sLine.length() >= 5)
+			PutClient(":irc.znc.in PONG irc.znc.in " + sLine.substr(5));
+		else
+			PutClient(":irc.znc.in PONG irc.znc.in");
 	} else if (sCommand.Equals("PONG")) {
-		return;  // Block pong replies, we already responded to the pings
+		// Block PONGs, we already responded to the pings
+		return;
 	} else if (sCommand.Equals("JOIN")) {
 		CString sChans = sLine.Token(1);
 		CString sKey = sLine.Token(2);
@@ -593,6 +590,9 @@ bool CClient::SendMotd() {
 }
 
 void CClient::AuthUser() {
+	if (!m_bGotNick || !m_bGotUser || !m_bGotPass || m_bInCap || IsAttached())
+		return;
+
 	m_spAuth = new CClientAuth(this, m_sUser, m_sPass);
 
 	CZNC::Get().AuthUser(m_spAuth);
@@ -627,11 +627,11 @@ void CAuthBase::AcceptLogin(CUser& User) {
 }
 
 void CAuthBase::RefuseLogin(const CString& sReason) {
-	CUser* pUser = CZNC::Get().GetUser(GetUsername());
-
 	if (!m_pSock)
 		return;
 
+	CUser* pUser = CZNC::Get().FindUser(GetUsername());
+
 	// If the username is valid, notify that user that someone tried to
 	// login. Use sReason because there are other reasons than "wrong
 	// password" for a login to be rejected (e.g. fail2ban).
@@ -640,7 +640,7 @@ void CAuthBase::RefuseLogin(const CString& sReason) {
 				"to login as you, but was rejected [" + sReason + "].");
 	}
 
-	CZNC::Get().GetModules().OnFailedLogin(GetUsername(), GetRemoteIP());
+	GLOBALMODULECALL(OnFailedLogin(GetUsername(), GetRemoteIP()), NULL, NULL, NOTHING);
 	RefusedLogin(sReason);
 	Invalidate();
 }
@@ -671,7 +671,7 @@ void CClient::AcceptLogin(CUser& User) {
 
 	SendMotd();
 
-	MODULECALL(OnClientLogin(), m_pUser, this, );
+	MODULECALL(OnClientLogin(), m_pUser, this, NOTHING);
 }
 
 void CClient::Timeout() {
@@ -692,7 +692,7 @@ void CClient::Disconnected() {
 		m_pUser->UserDisconnected(this);
 	}
 
-	MODULECALL(OnClientDisconnect(), m_pUser, this, );
+	MODULECALL(OnClientDisconnect(), m_pUser, this, NOTHING);
 }
 
 void CClient::ReachedMaxBuffer() {
@@ -766,10 +766,106 @@ CString CClient::GetNickMask() const {
 		return GetIRCSock()->GetNickMask();
 	}
 
-	CString sHost = m_pUser->GetVHost();
+	CString sHost = m_pUser->GetBindHost();
 	if (sHost.empty()) {
 		sHost = "irc.znc.in";
 	}
 
 	return GetNick() + "!" + m_pUser->GetIdent() + "@" + sHost;
 }
+
+void CClient::RespondCap(const CString& sResponse)
+{
+	PutClient(":irc.znc.in CAP " + GetNick() + " " + sResponse);
+}
+
+void CClient::HandleCap(const CString& sLine)
+{
+	CString sSubCmd = sLine.Token(1);
+
+	if (sSubCmd.Equals("LS")) {
+		SCString ssOfferCaps;
+		GLOBALMODULECALL(OnClientCapLs(ssOfferCaps), m_pUser, this, NOTHING);
+		CString sRes;
+		for (SCString::iterator i = ssOfferCaps.begin(); i != ssOfferCaps.end(); ++i) {
+			sRes += *i + " ";
+		}
+		RespondCap("LS :" + sRes + "userhost-in-names multi-prefix");
+		m_bInCap = true;
+	} else if (sSubCmd.Equals("END")) {
+		m_bInCap = false;
+		AuthUser();
+	} else if (sSubCmd.Equals("REQ")) {
+		VCString vsTokens;
+		VCString::iterator it;
+		sLine.Token(2, true).TrimPrefix_n(":").Split(" ", vsTokens, false);
+
+		for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
+			bool bVal = true;
+			CString sCap = *it;
+			if (sCap.TrimPrefix("-"))
+				bVal = false;
+
+			bool bAccepted = ("multi-prefix" == sCap) || ("userhost-in-names" == sCap);
+			GLOBALMODULECALL(IsClientCapSupported(sCap, bVal), m_pUser, this, bAccepted = true);
+
+			if (!bAccepted) {
+				// Some unsupported capability is requested
+				RespondCap("NAK :" + sLine.Token(2, true).TrimPrefix_n(":"));
+				return;
+			}
+		}
+
+		// All is fine, we support what was requested
+		for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
+			bool bVal = true;
+			if (it->TrimPrefix("-"))
+				bVal = false;
+
+			if ("multi-prefix" == *it) {
+				m_bNamesx = bVal;
+			} else if ("userhost-in-names" == *it) {
+				m_bUHNames = bVal;
+			}
+			GLOBALMODULECALL(OnClientCapRequest(*it, bVal), m_pUser, this, NOTHING);
+
+			if (bVal) {
+				m_ssAcceptedCaps.insert(*it);
+			} else {
+				m_ssAcceptedCaps.erase(*it);
+			}
+		}
+
+		RespondCap("ACK :" + sLine.Token(2, true).TrimPrefix_n(":"));
+	} else if (sSubCmd.Equals("LIST")) {
+		CString sList = "";
+		for (SCString::iterator i = m_ssAcceptedCaps.begin(); i != m_ssAcceptedCaps.end(); ++i) {
+			sList += *i + " ";
+		}
+		RespondCap("LIST :" + sList.TrimSuffix_n(" "));
+	} else if (sSubCmd.Equals("CLEAR")) {
+		SCString ssRemoved;
+		for (SCString::iterator i = m_ssAcceptedCaps.begin(); i != m_ssAcceptedCaps.end(); ++i) {
+			bool bRemoving = false;
+			GLOBALMODULECALL(IsClientCapSupported(*i, false), m_pUser, this, bRemoving = true);
+			if (bRemoving) {
+				GLOBALMODULECALL(OnClientCapRequest(*i, false), m_pUser, this, NOTHING);
+				ssRemoved.insert(*i);
+			}
+		}
+		if (m_bNamesx) {
+			m_bNamesx = false;
+			ssRemoved.insert("multi-prefix");
+		}
+		if (m_bUHNames) {
+			m_bUHNames = false;
+			ssRemoved.insert("userhost-in-names");
+		}
+		CString sList = "";
+		for (SCString::iterator i = ssRemoved.begin(); i != ssRemoved.end(); ++i) {
+			m_ssAcceptedCaps.erase(*i);
+			sList += "-" + *i + " ";
+		}
+		RespondCap("ACK :" + sList.TrimSuffix_n(" "));
+	}
+}
