@@ -28,7 +28,6 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.138 $
 */
 
 #include "Csocket.h"
@@ -479,7 +478,7 @@ void SSLErrors( const char *filename, u_int iLineNum )
 		char szError[512];
 		memset( (char *) szError, '\0', 512 );
 		ERR_error_string_n( iSSLError, szError, 511 );
-		if ( strlen( szError ) > 0 )
+		if ( *szError )
 			CS_DEBUG( szError );
 	}
 }
@@ -588,7 +587,7 @@ u_int CCron::GetCyclesLeft() const { return( ( m_iMaxCycles > m_iCycles ? ( m_iM
 bool CCron::isValid() { return( m_bActive ); }
 const CS_STRING & CCron::GetName() const { return( m_sName ); }
 void CCron::SetName( const CS_STRING & sName ) { m_sName = sName; }
-void CCron::RunJob() { CS_DEBUG( "This should be overriden" ); }
+void CCron::RunJob() { CS_DEBUG( "This should be overridden" ); }
 
 Csock::Csock( int itimeout )
 {
@@ -682,7 +681,6 @@ void Csock::Copy( const Csock & cCopy )
 	m_bssl			= cCopy.m_bssl;
 	m_bIsConnected	= cCopy.m_bIsConnected;
 	m_bBLOCK		= cCopy.m_bBLOCK;
-	m_bFullsslAccept	= cCopy.m_bFullsslAccept;
 	m_bsslEstablished	= cCopy.m_bsslEstablished;
 	m_bEnableReadLine	= cCopy.m_bEnableReadLine;
 	m_bPauseRead		= cCopy.m_bPauseRead;
@@ -1078,12 +1076,9 @@ bool Csock::AcceptSSL()
 
 	if ( err == 1 )
 	{
-		m_bFullsslAccept = true;
 		return( true );
 	}
 
-	m_bFullsslAccept = false;
-
 	int sslErr = SSL_get_error( m_ssl, err );
 
 	if ( ( sslErr == SSL_ERROR_WANT_READ ) || ( sslErr == SSL_ERROR_WANT_WRITE ) )
@@ -1599,10 +1594,14 @@ CS_STRING Csock::GetLocalIP()
 
 	if( !GetIPv6() )
 	{
+		char straddr[INET_ADDRSTRLEN];
 		struct sockaddr_in mLocalAddr;
 		socklen_t mLocalLen = sizeof( mLocalAddr );
-		if ( getsockname( iSock, (struct sockaddr *) &mLocalAddr, &mLocalLen ) == 0 )
-			m_sLocalIP = inet_ntoa( mLocalAddr.sin_addr );
+		if ( ( getsockname( iSock, (struct sockaddr *) &mLocalAddr, &mLocalLen ) == 0 )
+			&& ( inet_ntop( AF_INET, &mLocalAddr.sin_addr, straddr, sizeof(straddr) ) ) )
+		{
+			m_sLocalIP = straddr;
+		}
 	}
 #ifdef HAVE_IPV6
 	else
@@ -1670,7 +1669,7 @@ CS_STRING Csock::ConvertAddress( void *addr, bool bIPv6 )
 	return( sRet );
 }
 
-bool Csock::IsConnected() { return( m_bIsConnected ); }
+bool Csock::IsConnected() const { return( m_bIsConnected ); }
 void Csock::SetIsConnected( bool b ) { m_bIsConnected = b; }
 
 cs_sock_t & Csock::GetRSock() { return( m_iReadSock ); }
@@ -1930,7 +1929,6 @@ void Csock::SetSSLMethod( int iMethod ) { m_iMethod = iMethod; }
 int Csock::GetSSLMethod() { return( m_iMethod ); }
 void Csock::SetSSLObject( SSL *ssl ) { m_ssl = ssl; }
 void Csock::SetCTXObject( SSL_CTX *sslCtx ) { m_ssl_ctx = sslCtx; }
-void Csock::SetFullSSLAccept() { m_bFullsslAccept = true; }
 
 SSL_SESSION * Csock::GetSSLSession()
 {
@@ -1943,7 +1941,6 @@ SSL_SESSION * Csock::GetSSLSession()
 
 const CS_STRING & Csock::GetWriteBuffer() { return( m_sSend ); }
 void Csock::ClearWriteBuffer() { m_sSend.clear(); }
-bool Csock::FullSSLAccept() { return ( m_bFullsslAccept ); }
 bool Csock::SslIsEstablished() { return ( m_bsslEstablished ); }
 
 bool Csock::ConnectInetd( bool bIsSSL, const CS_STRING & sHostname )
@@ -2153,7 +2150,10 @@ void Csock::DelCronByAddr( CCron *pcCron )
 }
 
 void Csock::EnableReadLine() { m_bEnableReadLine = true; }
-void Csock::DisableReadLine() { m_bEnableReadLine = false; }
+void Csock::DisableReadLine() {
+	m_bEnableReadLine = false;
+	m_sbuffer.clear();
+}
 
 void Csock::ReachedMaxBuffer()
 {
@@ -2433,7 +2433,6 @@ void Csock::Init( const CS_STRING & sHostname, u_short uPort, int itimeout )
 	m_iMaxMilliSeconds = 0;
 	m_iLastSendTime = 0;
 	m_iLastSend = 0;
-	m_bFullsslAccept = false;
 	m_bsslEstablished = false;
 	m_bEnableReadLine = false;
 	m_iMaxStoredBufferLength = 1024;
