@@ -28,13 +28,14 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.211 $
 */
 
 // note to compile with win32 need to link to winsock2, using gcc its -lws2_32
 
 #ifndef _HAS_CSOCKET_
 #define _HAS_CSOCKET_
+#include "defines.h" // require this as a general rule, most projects have a defines.h or the like
+
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/time.h>
@@ -58,9 +59,14 @@
 #define ETIMEDOUT WSAETIMEDOUT
 #define EADDRNOTAVAIL WSAEADDRNOTAVAIL
 #define ECONNABORTED WSAECONNABORTED
+#define ENETUNREACH WSAENETUNREACH
 
 #endif /* _WIN32 */
 
+#ifdef HAVE_C_ARES
+#include <ares.h>
+#endif /* HAVE_C_ARES */
+
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
@@ -79,14 +85,13 @@
 #endif /* __sun */
 
 #include <vector>
+#include <list>
 #include <iostream>
 #include <sstream>
 #include <string>
 #include <set>
 #include <map>
 
-#include "defines.h" // require this as a general rule, most projects have a defines.h or the like
-
 #ifndef CS_STRING
 #	ifdef _HAS_CSTRING_
 #		define CS_STRING Cstring
@@ -111,6 +116,24 @@
 #endif /* __DEBUG__ */
 #endif /* PERROR */
 
+#ifdef _WIN32
+typedef SOCKET cs_sock_t;
+#ifdef _WIN64
+typedef signed __int64 cs_ssize_t;
+#else
+typedef signed int cs_ssize_t;
+#endif  /* _WIN64 */
+#define CS_INVALID_SOCK	INVALID_SOCKET
+#else
+typedef int cs_sock_t;
+typedef ssize_t cs_ssize_t;
+#define CS_INVALID_SOCK	-1
+#endif /* _WIN32 */
+
+#ifdef CSOCK_USE_POLL
+#include <poll.h>
+#endif /* CSOCK_USE_POLL */
+
 #ifndef _NO_CSOCKET_NS // some people may not want to use a namespace
 namespace Csocket
 {
@@ -300,12 +323,12 @@ inline void TFD_ZERO( fd_set *set )
 	FD_ZERO( set );
 }
 
-inline void TFD_SET( u_int iSock, fd_set *set )
+inline void TFD_SET( cs_sock_t iSock, fd_set *set )
 {
 	FD_SET( iSock, set );
 }
 
-inline bool TFD_ISSET( u_int iSock, fd_set *set )
+inline bool TFD_ISSET( cs_sock_t iSock, fd_set *set )
 {
 	if ( FD_ISSET( iSock, set ) )
 		return( true );
@@ -313,7 +336,7 @@ inline bool TFD_ISSET( u_int iSock, fd_set *set )
 	return( false );
 }
 
-inline void TFD_CLR( u_int iSock, fd_set *set )
+inline void TFD_CLR( cs_sock_t iSock, fd_set *set )
 {
 	FD_CLR( iSock, set );
 }
@@ -408,10 +431,10 @@ public:
 	* Advanced constructor, for creating a simple connection
 	*
 	* @param sHostname the hostname your are connecting to
-	* @param iport the port you are connectint to
+	* @param uPort the port you are connectint to
 	* @param itimeout how long to wait before ditching the connection, default is 60 seconds
 	*/
-	Csock( const CS_STRING & sHostname, u_short iport, int itimeout = 60 );
+	Csock( const CS_STRING & sHostname, u_short uPort, int itimeout = 60 );
 
 	//! override this for accept sockets
 	virtual Csock *GetSockObj( const CS_STRING & sHostname, u_short iPort );
@@ -521,7 +544,7 @@ public:
 	virtual bool Listen( u_short iPort, int iMaxConns = SOMAXCONN, const CS_STRING & sBindHost = "", u_int iTimeout = 0 );
 
 	//! Accept an inbound connection, this is used internally
-	virtual int Accept( CS_STRING & sHost, u_short & iRPort );
+	virtual cs_sock_t Accept( CS_STRING & sHost, u_short & iRPort );
 
 	//! Accept an inbound SSL connection, this is used internally and called after Accept
 	virtual bool AcceptSSL();
@@ -551,7 +574,7 @@ public:
 	* @param len the length of data
 	*
 	*/
-	virtual bool Write( const char *data, int len );
+	virtual bool Write( const char *data, size_t len );
 
 	/**
 	* convience function
@@ -574,23 +597,25 @@ public:
 	* Returns READ_TIMEDOUT for a connection that timed out at the TCP level
 	* Otherwise returns the bytes read into data
 	*/
-	virtual int Read( char *data, int len );
+	virtual cs_ssize_t Read( char *data, size_t len );
 	CS_STRING GetLocalIP();
 	CS_STRING GetRemoteIP();
 
+	virtual CS_STRING ConvertAddress( void *addr, bool bIPv6 = false );
+
 	//! Tells you if the socket is connected
-	virtual bool IsConnected();
+	virtual bool IsConnected() const;
 	//! Sets the sock, telling it its connected (internal use only)
 	virtual void SetIsConnected( bool b );
 
 	//! returns a reference to the sock
-	int & GetRSock();
-	void SetRSock( int iSock );
-	int & GetWSock();
-	void SetWSock( int iSock );
+	cs_sock_t & GetRSock();
+	void SetRSock( cs_sock_t iSock );
+	cs_sock_t & GetWSock();
+	void SetWSock( cs_sock_t iSock );
 
-	void SetSock( int iSock );
-	int & GetSock();
+	void SetSock( cs_sock_t iSock );
+	cs_sock_t & GetSock();
 
 	//! resets the time counter, this is virtual in the event you need an event on the timer being Reset
 	virtual void ResetTimer();
@@ -627,7 +652,7 @@ public:
 	* pushes data up on the buffer, if a line is ready
 	* it calls the ReadLine event
 	*/
-	virtual void PushBuff( const char *data, int len, bool bStartAtZero = false );
+	virtual void PushBuff( const char *data, size_t len, bool bStartAtZero = false );
 
 	//! This gives access to the internal read buffer, if your
 	//! not going to use ReadLine(), then you may want to clear this out
@@ -719,7 +744,6 @@ public:
 
 	void SetSSLObject( SSL *ssl );
 	void SetCTXObject( SSL_CTX *sslCtx );
-	void SetFullSSLAccept();
 	SSL_SESSION * GetSSLSession();
 
 	void SetCertVerifyCB( FPCertVerifyCB pFP ) { m_pCerVerifyCB = pFP; }
@@ -730,7 +754,6 @@ public:
 	void ClearWriteBuffer();
 
 	//! is SSL_accept finished ?
-	bool FullSSLAccept();
 	//! is the ssl properly finished (from write no error)
 	bool SslIsEstablished();
 
@@ -744,8 +767,11 @@ public:
 #ifdef HAVE_LIBSSL
 	X509 *getX509();
 
-	//! Returns The Peers Public Key
+	//! Returns the peer's public key
 	CS_STRING GetPeerPubKey();
+	//! Returns the peer's certificate finger print
+	int GetPeerFingerprint( CS_STRING & sFP);
+
 	unsigned int GetRequireClientCertFlags();
 	//! legacy, deprecated @see SetRequireClientCertFlags
 	void SetRequiresClientCert( bool bRequiresCert );
@@ -815,7 +841,7 @@ public:
 	*
 	* Ready to read data event
 	*/
-	virtual void ReadData( const char *data, int len ) {}
+	virtual void ReadData( const char *data, size_t len ) {}
 	/**
 	* Override these functions for an easy interface when using the Socket Manager
 	* Don't bother using these callbacks if you are using this class directly (without Socket Manager)
@@ -884,7 +910,7 @@ public:
 
 
 	//! return how long it has been (in seconds) since the last read or successful write
-	int GetTimeSinceLastDataTransaction( time_t iNow = 0 )
+	time_t GetTimeSinceLastDataTransaction( time_t iNow = 0 )
 	{
 		if( m_iLastCheckTimeoutTime == 0 )
 			return( 0 );
@@ -905,30 +931,33 @@ public:
 	//! grabs fd's for the sockets
 	bool CreateSocksFD()
 	{
-		if( m_iReadSock != -1 )
+		if( m_iReadSock != CS_INVALID_SOCK )
 			return( true );
 
-		m_iReadSock = m_iWriteSock = SOCKET();
-		if ( m_iReadSock == -1 )
+		m_iReadSock = m_iWriteSock = CreateSocket();
+		if ( m_iReadSock == CS_INVALID_SOCK )
 			return( false );
 
 		m_address.SinFamily();
-		m_address.SinPort( m_iport );
+		m_address.SinPort( m_uPort );
 
 		return( true );
 	}
 
+	//! puts the socks back to the state they were prior to calling CreateSocksFD
+	void CloseSocksFD();
+
 	const CS_STRING & GetBindHost() const { return( m_sBindHost ); }
 	void SetBindHost( const CS_STRING & sBindHost ) { m_sBindHost = sBindHost; }
 
 	enum EDNSLType
 	{
-		DNS_VHOST,
-		DNS_DEST
+		DNS_VHOST, //!< this lookup is for the vhost bind
+		DNS_DEST //!< this lookup is for the destination address
 	};
 
 	/**
-	 * nonblocking dns lookup (when -pthread is set to compile)
+	 * dns lookup @see EDNSLType
 	 * @return 0 for success, EAGAIN to check back again (same arguments as before), ETIMEDOUT on failure
 	 */
 	int DNSLookup( EDNSLType eDNSLType );
@@ -966,21 +995,29 @@ public:
 	 */
 	virtual int GetAddrInfo( const CS_STRING & sHostname, CSSockAddr & csSockAddr );
 
+#ifdef HAVE_C_ARES
+	CSSockAddr * GetCurrentAddr() const { return( m_pCurrAddr ); }
+	void SetAresFinished( int status ) { m_pCurrAddr = NULL; m_iARESStatus = status; }
+	ares_channel GetAresChannel() { return( m_pARESChannel ); }
+#endif /* HAVE_C_ARES */
+
 private:
 	//! making private for safety
 	Csock( const Csock & cCopy ) {}
 
 	// NOTE! if you add any new members, be sure to add them to Copy()
-	u_short		m_iport, m_iRemotePort, m_iLocalPort;
-	int			m_iReadSock, m_iWriteSock, m_itimeout, m_iConnType, m_iMethod, m_iTcount;
-	bool		m_bssl, m_bIsConnected, m_bBLOCK, m_bFullsslAccept;
+	u_short		m_uPort, m_iRemotePort, m_iLocalPort;
+	cs_sock_t	m_iReadSock, m_iWriteSock;
+	int m_itimeout, m_iConnType, m_iMethod, m_iTcount;
+	bool		m_bssl, m_bIsConnected, m_bBLOCK;
 	bool		m_bsslEstablished, m_bEnableReadLine, m_bPauseRead;
 	CS_STRING	m_shostname, m_sbuffer, m_sSockName, m_sPemFile, m_sCipherType, m_sParentName;
 	CS_STRING	m_sSend, m_sPemPass, m_sLocalIP, m_sRemoteIP;
 	ECloseType	m_eCloseType;
 
 	unsigned long long	m_iMaxMilliSeconds, m_iLastSendTime, m_iBytesRead, m_iBytesWritten, m_iStartTime;
-	unsigned int		m_iMaxBytes, m_iLastSend, m_iMaxStoredBufferLength, m_iTimeoutType;
+	unsigned int		m_iMaxBytes, m_iMaxStoredBufferLength, m_iTimeoutType;
+	size_t				m_iLastSend;
 
 	CSSockAddr 		m_address, m_bindhost;
 	bool			m_bIsIPv6, m_bSkipConnect;
@@ -1002,14 +1039,20 @@ private:
 	std::vector<CCron *>		m_vcCrons;
 
 	//! Create the socket
-	int SOCKET( bool bListen = false );
-	void Init( const CS_STRING & sHostname, u_short iport, int itimeout = 60 );
+	cs_sock_t CreateSocket( bool bListen = false );
+	void Init( const CS_STRING & sHostname, u_short uPort, int itimeout = 60 );
 
 
 	// Connection State Info
 	ECONState		m_eConState;
 	CS_STRING		m_sBindHost;
 	u_int			m_iCurBindCount, m_iDNSTryCount;
+#ifdef HAVE_C_ARES
+	void FreeAres();
+	ares_channel	m_pARESChannel;
+	CSSockAddr		*m_pCurrAddr;
+	int				m_iARESStatus;
+#endif /* HAVE_C_ARES */
 
 };
 
@@ -1223,6 +1266,8 @@ public:
 		m_errno = SUCCESS;
 		m_iCallTimeouts = millitime();
 		m_iSelectWait = 100000; // Default of 100 milliseconds
+		m_iBytesRead = 0;
+		m_iBytesWritten = 0;
 	}
 
 	virtual ~TSocketManager()
@@ -1316,6 +1361,12 @@ public:
 				pcSock->SetIPv6( true );
 #endif /* HAVE_IPV6 */
 		}
+#ifdef HAVE_IPV6
+		else
+		{
+				pcSock->SetIPv6( true );
+		}
+#endif /* HAVE_IPV6 */
 #ifdef HAVE_LIBSSL
 		pcSock->SetSSL( cListen.GetIsSSL() );
 		if( ( cListen.GetIsSSL() ) && ( !cListen.GetPemLocation().empty() ) )
@@ -1335,9 +1386,9 @@ public:
 			AddSock( pcSock, cListen.GetSockName() );
 			if( ( piRandPort ) && ( cListen.GetPort() == 0 ) )
 			{
-				int iSock = pcSock->GetSock();
+				cs_sock_t iSock = pcSock->GetSock();
 
-				if ( iSock < 0 )
+				if ( iSock == CS_INVALID_SOCK )
 				{
 					CS_DEBUG( "Failed to attain a valid file descriptor" );
 					pcSock->Close();
@@ -1369,7 +1420,6 @@ public:
 
 			if ( ( pcSock->GetType() != T::OUTBOUND ) || ( pcSock->GetConState() == T::CST_OK ) )
 				continue;
-
 			if ( pcSock->GetConState() == T::CST_DNS )
 			{
 				if ( pcSock->DNSLookup( T::DNS_VHOST ) == ETIMEDOUT )
@@ -1384,7 +1434,7 @@ public:
 			{
 				if ( !pcSock->SetupVHost() )
 				{
-					pcSock->SockError( errno );
+					pcSock->SockError( GetSockError() );
 					DelSock( a-- );
 					continue;
 				}
@@ -1458,10 +1508,9 @@ public:
 
 						CSCharBuffer cBuff( iLen );
 
-						int bytes = pcSock->Read( cBuff(), iLen );
+						cs_ssize_t bytes = pcSock->Read( cBuff(), iLen );
 
-						if ( ( bytes != T::READ_TIMEDOUT ) && ( bytes != T::READ_CONNREFUSED )
-							&& ( !pcSock->IsConnected() ) )
+						if ( bytes != T::READ_TIMEDOUT && bytes != T::READ_CONNREFUSED && bytes != T::READ_ERR && !pcSock->IsConnected() )
 						{
 							pcSock->SetIsConnected( true );
 							pcSock->Connected();
@@ -1517,6 +1566,7 @@ public:
 			}
 
 			case SELECT_TIMEOUT:
+			case SELECT_TRYAGAIN:
 			case SELECT_ERROR:
 			default	:
 				break;
@@ -1578,7 +1628,7 @@ public:
 	virtual void AddSock( T *pcSock, const CS_STRING & sSockName )
 	{
 		pcSock->SetSockName( sSockName );
-		push_back( pcSock );
+		this->push_back( pcSock );
 	}
 
 	//! returns a pointer to the FIRST sock found by port or NULL on no match
@@ -1616,7 +1666,7 @@ public:
 	}
 
 	//! returns a pointer to the FIRST sock found by filedescriptor or NULL on no match
-	virtual T * FindSockByFD( int iFD )
+	virtual T * FindSockByFD( cs_sock_t iFD )
 	{
 		for( unsigned int i = 0; i < this->size(); i++ )
 			if ( ( (*this)[i]->GetRSock() == iFD ) || ( (*this)[i]->GetWSock() == iFD ) )
@@ -1706,7 +1756,7 @@ public:
 
 	//! Get the Select Timeout in MICROSECONDS ( 1000 == 1 millisecond )
 	u_long GetSelectTimeout() { return( m_iSelectWait ); }
-	//! Set the Select Timeout in MICROSECODS ( 1000 == 1 millisecond )
+	//! Set the Select Timeout in MICROSECONDS ( 1000 == 1 millisecond )
 	//! Setting this to 0 will cause no timeout to happen, Select() will return instantly
 	void  SetSelectTimeout( u_long iTimeout ) { m_iSelectWait = iTimeout; }
 
@@ -1821,11 +1871,103 @@ public:
 
 protected:
 	//! this is a strict wrapper around C-api select(). Added in the event you need to do special work here
-	virtual int Select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout)
+	enum ECheckType
 	{
-		return( select( nfds, readfds, writefds, exceptfds, timeout ) );
+		eCheckRead = 1,
+		eCheckWrite = 2
+	};
+
+	void FDSetCheck( int iFd, std::map< int, short > & miiReadyFds, ECheckType eType )
+	{
+		std::map< int, short >::iterator it = miiReadyFds.find( iFd );
+		if( it != miiReadyFds.end() )
+			it->second |= eType;
+		else
+			miiReadyFds[iFd] = eType;
 	}
-protected:
+	bool FDHasCheck( int iFd, std::map< int, short > & miiReadyFds, ECheckType eType )
+	{
+		std::map< int, short >::iterator it = miiReadyFds.find( iFd );
+		if( it != miiReadyFds.end() )
+			return( (it->second & eType) );
+		return( false );
+	}
+	virtual int Select( std::map< int, short > & miiReadyFds, struct timeval *tvtimeout)
+	{
+#ifdef CSOCK_USE_POLL
+		if( miiReadyFds.empty() )
+			return( select( 0, NULL, NULL, NULL, tvtimeout ) );
+
+		struct pollfd * pFDs = (struct pollfd *)malloc( sizeof( struct pollfd ) * miiReadyFds.size() );
+		size_t uCurrPoll = 0;
+		for( std::map< int, short >::iterator it = miiReadyFds.begin(); it != miiReadyFds.end(); ++it, ++uCurrPoll )
+		{
+			short iEvents = 0;
+			if( it->second & eCheckRead )
+				iEvents |= POLLIN;
+			if( it->second & eCheckWrite )
+				iEvents |= POLLOUT;
+			pFDs[uCurrPoll].fd = it->first;
+			pFDs[uCurrPoll].events = iEvents;
+			pFDs[uCurrPoll].revents = 0;
+		}
+		int iTimeout = (int)(tvtimeout->tv_usec / 1000);
+		iTimeout += (int)(tvtimeout->tv_sec * 1000);
+		size_t uMaxFD = miiReadyFds.size();
+		int iRet = poll( pFDs, uMaxFD, iTimeout );
+		miiReadyFds.clear();
+		for( uCurrPoll = 0; uCurrPoll < uMaxFD; ++uCurrPoll )
+		{
+			short iEvents = 0;
+			if( (pFDs[uCurrPoll].revents & (POLLIN|POLLERR|POLLHUP|POLLNVAL) ) )
+				iEvents |= eCheckRead;
+			if( (pFDs[uCurrPoll].revents & POLLOUT ) )
+				iEvents |= eCheckWrite;
+			std::map< int, short >::iterator it = miiReadyFds.find( pFDs[uCurrPoll].fd );
+			if( it != miiReadyFds.end() )
+				it->second |= iEvents;
+			else
+				miiReadyFds[pFDs[uCurrPoll].fd] = iEvents;
+		}
+		free( pFDs );
+#else
+		fd_set rfds, wfds;
+		TFD_ZERO( &rfds );
+		TFD_ZERO( &wfds );
+		bool bHasWrite = false;
+		int iHighestFD = 0;
+		for( std::map< int, short >::iterator it = miiReadyFds.begin(); it != miiReadyFds.end(); ++it )
+		{
+			iHighestFD = std::max( it->first, iHighestFD );
+			if( it->second & eCheckRead )
+			{
+				TFD_SET( it->first, &rfds );
+			}
+			if( it->second & eCheckWrite )
+			{
+				bHasWrite = true;
+				TFD_SET( it->first, &wfds );
+			}
+		}
+
+		int iRet = select( iHighestFD + 1, &rfds, ( bHasWrite ? &wfds : NULL ), NULL, tvtimeout );
+		if( iRet <= 0 )
+			miiReadyFds.clear();
+		else
+		{
+			for( std::map< int, short >::iterator it = miiReadyFds.begin(); it != miiReadyFds.end(); ++it )
+			{
+				if( (it->second & eCheckRead) && !TFD_ISSET( it->first, &rfds ) )
+					it->second &= ~eCheckRead;
+				if( (it->second & eCheckWrite) && !TFD_ISSET( it->first, &wfds ) )
+					it->second &= ~eCheckWrite;
+			}
+		}
+#endif /* CSOCK_USE_POLL */
+
+		return( iRet );
+	}
+private:
 	/**
 	* fills a map of socks to a message for check
 	* map is empty if none are ready, check GetErrno() for the error, if not SUCCESS Select() failed
@@ -1836,49 +1978,69 @@ protected:
 	{
 		mpeSocks.clear();
 		struct timeval tv;
-		fd_set rfds, wfds;
 
+		std::map< int, short > miiReadyFds;
 		tv.tv_sec = m_iSelectWait / 1000000;
 		tv.tv_usec = m_iSelectWait % 1000000;
 		u_int iQuickReset = 100;
 		if ( m_iSelectWait == 0 )
 			iQuickReset = 0;
 
-		TFD_ZERO( &rfds );
-		TFD_ZERO( &wfds );
-
-		// before we go any further, Process work needing to be done on the job
-		for( unsigned int i = 0; i < this->size(); i++ )
-		{
-			Csock::ECloseType eCloseType = (*this)[i]->GetCloseType();
-			if( eCloseType == T::CLT_NOW || eCloseType == T::CLT_DEREFERENCE || ( eCloseType == T::CLT_AFTERWRITE && (*this)[i]->GetWriteBuffer().empty() ) )
-				DelSock( i-- ); // close any socks that have requested it
-			else
-				(*this)[i]->Cron(); // call the Cron handler here
-		}
-
-		bool bHasWriteable = false;
 		bool bHasAvailSocks = false;
 		unsigned long long iNOW = 0;
-
 		for( unsigned int i = 0; i < this->size(); i++ )
 		{
 			T *pcSock = (*this)[i];
 
+			Csock::ECloseType eCloseType = pcSock->GetCloseType();
+
+			if( eCloseType == T::CLT_NOW || eCloseType == T::CLT_DEREFERENCE || ( eCloseType == T::CLT_AFTERWRITE && pcSock->GetWriteBuffer().empty() ) )
+			{
+				DelSock( i-- ); // close any socks that have requested it
+				continue;
+			}
+			else
+				pcSock->Cron(); // call the Cron handler here
+
+			cs_sock_t & iRSock = pcSock->GetRSock();
+			cs_sock_t & iWSock = pcSock->GetWSock();
+#ifndef CSOCK_USE_POLL
+			if( iRSock > FD_SETSIZE || iWSock > FD_SETSIZE )
+			{
+				CS_DEBUG( "FD is larger than select() can handle" );
+				DelSock( i-- );
+				continue;
+			}
+#endif /* CSOCK_USE_POLL */
+		
+#ifdef HAVE_C_ARES
+			ares_channel pChannel = pcSock->GetAresChannel();
+			if( pChannel )
+			{
+				ares_socket_t aiAresSocks[1];
+				aiAresSocks[0] = ARES_SOCKET_BAD;
+				int iSockMask = ares_getsock( pChannel, aiAresSocks, 1 );
+				if( ARES_GETSOCK_READABLE( iSockMask, 0 ) )
+					FDSetCheck( aiAresSocks[0], miiReadyFds, eCheckRead );
+				if( ARES_GETSOCK_WRITABLE( iSockMask, 0 ) )
+					FDSetCheck( aiAresSocks[0], miiReadyFds, eCheckWrite );
+				// let ares drop the timeout if it has something timing out sooner then whats in tv currently
+				ares_timeout( pChannel, &tv, &tv );
+			}
+#endif /* HAVE_C_ARES */
+
 			if ( pcSock->GetConState() != T::CST_OK )
 				continue;
 
 			bHasAvailSocks = true;
 
-			int & iRSock = pcSock->GetRSock();
-			int & iWSock = pcSock->GetWSock();
 			bool bIsReadPaused = pcSock->IsReadPaused();
 			if ( bIsReadPaused )
 			{
 				pcSock->ReadPaused();
 				bIsReadPaused = pcSock->IsReadPaused(); // re-read it again, incase it changed status)
 			}
-			if ( ( iRSock < 0 ) || ( iWSock < 0 ) )
+			if ( iRSock == CS_INVALID_SOCK || iWSock == CS_INVALID_SOCK )
 			{
 				SelectSock( mpeSocks, SUCCESS, pcSock );
 				continue;	// invalid sock fd
@@ -1886,61 +2048,44 @@ protected:
 
 			if ( pcSock->GetType() != T::LISTENER )
 			{
-				if ( ( pcSock->GetSSL() ) && ( pcSock->GetType() == T::INBOUND ) && ( !pcSock->FullSSLAccept() ) )
-				{
-					tv.tv_usec = iQuickReset;	// just make sure this returns quick incase we still need pending
-					tv.tv_sec = 0;
-					// try accept on this socket again
-					if ( !pcSock->AcceptSSL() )
-						pcSock->Close();
-
-				} else if ( ( pcSock->IsConnected() ) && ( pcSock->GetWriteBuffer().empty() ) )
+				if ( ( pcSock->IsConnected() ) && ( pcSock->GetWriteBuffer().empty() ) )
 				{
 					if ( !bIsReadPaused )
-						TFD_SET( iRSock, &rfds );
+						FDSetCheck( iRSock, miiReadyFds, eCheckRead );
 
 				} else if ( ( pcSock->GetSSL() ) && ( !pcSock->SslIsEstablished() ) && ( !pcSock->GetWriteBuffer().empty() ) )
 				{
 					// do this here, cause otherwise ssl will cause a small
 					// cpu spike waiting for the handshake to finish
-					TFD_SET( iRSock, &rfds );
+					FDSetCheck( iRSock, miiReadyFds, eCheckRead );
 					// resend this data
 					if ( !pcSock->Write( "" ) )
 					{
 						pcSock->Close();
 					}
-					if( pcSock->GetWriteBuffer().size() )
+					if( !pcSock->GetWriteBuffer().empty() )
 					{ // this means we need to write again, not everything got knocked out
-						TFD_SET( iWSock, &wfds );
-						bHasWriteable = true;
+						FDSetCheck( iWSock, miiReadyFds, eCheckWrite );
 					}
 
 				} else
 				{
 					if ( !bIsReadPaused )
-						TFD_SET( iRSock, &rfds );
+						FDSetCheck( iRSock, miiReadyFds, eCheckRead );
 
 					if( pcSock->AllowWrite( iNOW ) )
 					{
-						TFD_SET( iWSock, &wfds );
-						bHasWriteable = true;
+						FDSetCheck( iWSock, miiReadyFds, eCheckWrite );
 					}
 				}
 
-			} else
-				TFD_SET( iRSock, &rfds );
-		}
-
-		// first check to see if any ssl sockets are ready for immediate read
-		// a mini select() type deal for ssl
-		for( unsigned int i = 0; i < this->size(); i++ )
-		{
-			T *pcSock = (*this)[i];
-
-			if ( pcSock->GetConState() != T::CST_OK )
-				continue;
-
-			if ( ( pcSock->GetSSL() ) && ( pcSock->GetType() != Csock::LISTENER ) )
+			} 
+			else
+			{
+				FDSetCheck( iRSock, miiReadyFds, eCheckRead );
+			}
+			
+			if( pcSock->GetSSL() && pcSock->GetType() != Csock::LISTENER )
 			{
 				if ( ( pcSock->GetPending() > 0 ) && ( !pcSock->IsReadPaused() ) )
 					SelectSock( mpeSocks, SUCCESS, pcSock );
@@ -1961,17 +2106,23 @@ protected:
 			tv.tv_sec = 0;
 		}
 
-
-		if ( bHasWriteable )
-			iSel = Select(FD_SETSIZE, &rfds, &wfds, NULL, &tv);
-		else
-			iSel = Select(FD_SETSIZE, &rfds, NULL, NULL, &tv);
+		iSel = Select( miiReadyFds, &tv );
 		if ( iSel == 0 )
 		{
 			if ( mpeSocks.empty() )
 				m_errno = SELECT_TIMEOUT;
 			else
 				m_errno = SUCCESS;
+#ifdef HAVE_C_ARES
+			// run through ares channels and process timeouts
+			for( u_long uSock = 0; uSock < this->size(); ++uSock )
+			{
+				T *pcSock = this->at( uSock );
+				ares_channel pChannel = pcSock->GetAresChannel();
+				if( pChannel )
+					ares_process_fd( pChannel, ARES_SOCKET_BAD, ARES_SOCKET_BAD );
+			}
+#endif /* HAVE_C_ARES */
 
 			return;
 		}
@@ -1984,7 +2135,8 @@ protected:
 				m_errno = SUCCESS;
 
 			return;
-		} else if ( iSel == -1 )
+		} 
+		else if ( iSel == -1 )
 		{
 			if ( mpeSocks.empty() )
 				m_errno = SELECT_ERROR;
@@ -1992,7 +2144,8 @@ protected:
 				m_errno = SUCCESS;
 
 			return;
-		} else
+		} 
+		else
 		{
 			m_errno = SUCCESS;
 		}
@@ -2002,14 +2155,26 @@ protected:
 		{
 			T *pcSock = (*this)[i];
 
+#ifdef HAVE_C_ARES
+			ares_channel pChannel = pcSock->GetAresChannel();
+			if( pChannel )
+			{
+				ares_socket_t aiAresSocks[1];
+				aiAresSocks[0] = ARES_SOCKET_BAD;
+				ares_getsock( pChannel, aiAresSocks, 1 );
+				if( FDHasCheck( aiAresSocks[0], miiReadyFds, eCheckRead ) || FDHasCheck( aiAresSocks[0], miiReadyFds, eCheckWrite ) )
+					ares_process_fd( pChannel, aiAresSocks[0], aiAresSocks[0] );
+			}
+#endif /* HAVE_C_ARES */
+
 			if ( pcSock->GetConState() != T::CST_OK )
 				continue;
 
-			int & iRSock = pcSock->GetRSock();
-			int & iWSock = pcSock->GetWSock();
+			cs_sock_t & iRSock = pcSock->GetRSock();
+			cs_sock_t & iWSock = pcSock->GetWSock();
 			EMessages iErrno = SUCCESS;
 
-			if ( ( iRSock < 0 ) || ( iWSock < 0 ) )
+			if ( iRSock == CS_INVALID_SOCK || iWSock == CS_INVALID_SOCK )
 			{
 				// trigger a success so it goes through the normal motions
 				// and an error is produced
@@ -2017,7 +2182,7 @@ protected:
 				continue; // watch for invalid socks
 			}
 
-			if ( TFD_ISSET( iWSock, &wfds ) )
+			if ( FDHasCheck( iWSock, miiReadyFds, eCheckWrite ) )
 			{
 				if ( iSel > 0 )
 				{
@@ -2035,7 +2200,8 @@ protected:
 
 				SelectSock( mpeSocks, iErrno, pcSock );
 
-			} else if ( TFD_ISSET( iRSock, &rfds ) )
+			} 
+			else if ( FDHasCheck( iRSock, miiReadyFds, eCheckRead ) )
 			{
 				if ( iSel > 0 )
 					iErrno = SUCCESS;
@@ -2048,9 +2214,9 @@ protected:
 				{
 					CS_STRING sHost;
 					u_short port;
-					int inSock = pcSock->Accept( sHost, port );
+					cs_sock_t inSock = pcSock->Accept( sHost, port );
 
-					if ( inSock != -1 )
+					if ( inSock != CS_INVALID_SOCK )
 					{
 						if ( T::TMO_ACCEPT & pcSock->GetTimeoutType() )
 							pcSock->ResetTimer();	// let them now it got dinged
