@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -12,6 +12,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <fcntl.h>
 
 #ifndef HAVE_LSTAT
 #  define lstat(a, b)	stat(a, b)
@@ -36,7 +37,10 @@ CFile::~CFile() {
 }
 
 void CFile::SetFileName(const CString& sLongName) {
-	m_sLongName = sLongName;
+	if (sLongName.Left(2) == "~/") {
+		m_sLongName = CZNC::Get().GetHomePath() + sLongName.substr(1);
+	} else
+		m_sLongName = sLongName;
 
 	m_sShortName = sLongName;
 	m_sShortName.TrimRight("/");
@@ -388,23 +392,33 @@ bool CFile::TryExLock(const CString& sLockFile, int iFlags) {
 }
 
 bool CFile::TryExLock() {
-	return Lock(LOCK_EX|LOCK_NB);
+	return Lock(F_WRLCK, false);
+}
+
+bool CFile::ExLock() {
+	return Lock(F_WRLCK, true);
 }
 
 bool CFile::UnLock() {
-	return Lock(LOCK_UN);
+	return Lock(F_UNLCK, true);
 }
 
-bool CFile::Lock(int iOperation) {
+bool CFile::Lock(int iType, bool bBlocking) {
+	struct flock fl;
+
 	if (m_iFD == -1) {
 		return false;
 	}
 
-	if (flock(m_iFD, iOperation) != 0) {
+	fl.l_type   = iType;
+	fl.l_whence = SEEK_SET;
+	fl.l_start  = 0;
+	fl.l_len    = 0;
+	if (fcntl(m_iFD, (bBlocking ? F_SETLKW : F_SETLK), &fl) == -1) {
 		return false;
+	} else {
+		return true;
 	}
-
-	return true;
 }
 
 bool CFile::IsOpen() const { return (m_iFD != -1); }
