@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -10,6 +10,9 @@
 #include "HTTPSock.h"
 #include "znc.h"
 
+#include <sstream>
+#include <iomanip>
+
 #define MAX_POST_SIZE	1024 * 1024
 
 CHTTPSock::CHTTPSock(CModule *pMod) : CSocket(pMod) {
@@ -46,6 +49,7 @@ bool CHTTPSock::SendCookie(const CString& sKey, const CString& sValue) {
 		if (m_msRequestCookies.find(sKey) == m_msRequestCookies.end() ||
 			m_msRequestCookies[sKey].StrCmp(sValue) != 0)
 		{
+			// only queue a Set-Cookie to be sent if the client didn't send a Cookie header of the same name+value.
 			m_msResponseCookies[sKey] = sValue;
 		}
 		return true;
@@ -54,10 +58,6 @@ bool CHTTPSock::SendCookie(const CString& sKey, const CString& sValue) {
 	return false;
 }
 
-const MCString& CHTTPSock::GetRequestCookies() const {
-	return m_msRequestCookies;
-}
-
 CString CHTTPSock::GetRequestCookie(const CString& sKey) const {
 	MCString::const_iterator it = m_msRequestCookies.find(sKey);
 
@@ -118,7 +118,6 @@ void CHTTPSock::ReadLine(const CString& sData) {
 		m_sIfNoneMatch = sLine.Token(1, true);
 	} else if (sLine.empty()) {
 		m_bGotHeader = true;
-		DisableReadLine();
 
 		if (m_bPost) {
 			m_sPostData = GetInternalReadBuffer();
@@ -126,9 +125,33 @@ void CHTTPSock::ReadLine(const CString& sData) {
 		} else {
 			GetPage();
 		}
+
+		DisableReadLine();
 	}
 }
 
+CString CHTTPSock::GetDate(time_t stamp) {
+	struct tm tm;
+	std::stringstream stream;
+	const char *wkday[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
+	const char *month[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
+
+	if (stamp == 0)
+		time(&stamp);
+	gmtime_r(&stamp, &tm);
+
+	stream << wkday[tm.tm_wday] << ", ";
+	stream << std::setfill('0') << std::setw(2) << tm.tm_mday << " ";
+	stream << month[tm.tm_mon] << " ";
+	stream << std::setfill('0') << std::setw(4) << tm.tm_year + 1900 << " ";
+	stream << std::setfill('0') << std::setw(2) << tm.tm_hour << ":";
+	stream << std::setfill('0') << std::setw(2) << tm.tm_min << ":";
+	stream << std::setfill('0') << std::setw(2) << tm.tm_sec << " GMT";
+
+	return stream.str();
+}
+
 void CHTTPSock::GetPage() {
 	DEBUG("Page Request [" << m_sURI << "] ");
 
@@ -198,6 +221,7 @@ bool CHTTPSock::PrintFile(const CString& sFileName, CString sContentType) {
 	if (iMTime > 0 && !m_bHTTP10Client) {
 		sETag = "-" + CString(iMTime); // lighttpd style ETag
 
+		AddHeader("Last-Modified", GetDate(iMTime));
 		AddHeader("ETag", "\"" + sETag + "\"");
 		AddHeader("Cache-Control", "public");
 
@@ -406,21 +430,19 @@ bool CHTTPSock::PrintErrorPage(unsigned int uStatusId, const CString& sStatusMsg
 		return false;
 	}
 
-	CString sPage = GetErrorPage(uStatusId, sStatusMsg, sMessage);
-	PrintHeader(sPage.length(), "text/html", uStatusId, sStatusMsg);
-	Write(sPage);
-	Close(Csock::CLT_AFTERWRITE);
-
-	return true;
-}
-
-CString CHTTPSock::GetErrorPage(unsigned int uStatusId, const CString& sStatusMsg, const CString& sMessage) {
-	return "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\r\n"
+	CString sPage =
+		"<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\r\n"
 		"<html><head>\r\n<title>" + CString(uStatusId) + " " + sStatusMsg.Escape_n(CString::EHTML) + "</title>\r\n"
 		"</head><body>\r\n<h1>" + sStatusMsg.Escape_n(CString::EHTML) + "</h1>\r\n"
 		"<p>" + sMessage.Escape_n(CString::EHTML) + "</p>\r\n"
 		"<hr />\r\n<address>" + CZNC::GetTag(false).Escape_n(CString::EHTML) + " at " + GetLocalIP().Escape_n(CString::EHTML) + " Port " + CString(GetLocalPort()) + "</address>\r\n"
 		"</body></html>\r\n";
+
+	PrintHeader(sPage.length(), "text/html", uStatusId, sStatusMsg);
+	Write(sPage);
+	Close(Csock::CLT_AFTERWRITE);
+
+	return true;
 }
 
 bool CHTTPSock::ForceLogin() {
@@ -433,11 +455,8 @@ bool CHTTPSock::ForceLogin() {
 		return false;
 	}
 
-	CString sPage = GetErrorPage(401, "Unauthorized", "You need to login to view this page.");
 	AddHeader("WWW-Authenticate", "Basic realm=\"" + CZNC::GetTag(false) + "\"");
-	PrintHeader(sPage.length(), "text/html", 401, "Unauthorized");
-	Write(sPage);
-	Close(Csock::CLT_AFTERWRITE);
+	PrintErrorPage(401, "Unauthorized", "You need to login to view this page.");
 
 	return false;
 }
@@ -467,7 +486,7 @@ bool CHTTPSock::PrintHeader(off_t uContentLength, const CString& sContentType, u
 	DEBUG("- " << uStatusId << " (" << sStatusMsg << ") [" << m_sContentType << "]");
 
 	Write("HTTP/" + CString(m_bHTTP10Client ? "1.0 " : "1.1 ") + CString(uStatusId) + " " + sStatusMsg + "\r\n");
-	//Write("Date: Tue, 28 Jun 2005 20:45:36 GMT\r\n");
+	Write("Date: " + GetDate() + "\r\n");
 	Write("Server: " + CZNC::GetTag(false) + "\r\n");
 	if (uContentLength > 0) {
 		Write("Content-Length: " + CString(uContentLength) + "\r\n");
@@ -507,11 +526,8 @@ bool CHTTPSock::Redirect(const CString& sURL) {
 	}
 
 	DEBUG("- Redirect to [" << sURL << "]");
-	CString sPage = GetErrorPage(302, "Found", "The document has moved <a href=\"" + sURL.Escape_n(CString::EHTML) + "\">here</a>.");
 	AddHeader("Location", sURL);
-	PrintHeader(sPage.length(), "text/html", 302, "Found");
-	Write(sPage);
-	Close(Csock::CLT_AFTERWRITE);
+	PrintErrorPage(302, "Found", "The document has moved <a href=\"" + sURL.Escape_n(CString::EHTML) + "\">here</a>.");
 
 	return true;
 }
