@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -13,6 +13,10 @@
 #include "User.h"
 #include "znc.h"
 
+// These are used in OnGeneralCTCP()
+const time_t CIRCSock::m_uCTCPFloodTime = 5;
+const unsigned int CIRCSock::m_uCTCPFloodCount = 5;
+
 CIRCSock::CIRCSock(CUser* pUser) : CZNCSock() {
 	m_pUser = pUser;
 	m_bISpoofReleased = false;
@@ -21,9 +25,12 @@ CIRCSock::CIRCSock(CUser* pUser) : CZNCSock() {
 	m_bUHNames = false;
 	EnableReadLine();
 	m_Nick.SetIdent(pUser->GetIdent());
-	m_Nick.SetHost(pUser->GetVHost());
+	m_Nick.SetHost(pUser->GetBindHost());
 
 	m_uMaxNickLen = 9;
+	m_uCapPaused = 0;
+	m_lastCTCP = 0;
+	m_uNumCTCP = 0;
 	m_sPerms = "*!@%+";
 	m_sPermModes = "qaohv";
 	m_mueChanModes['b'] = ListArg;
@@ -37,6 +44,8 @@ CIRCSock::CIRCSock(CUser* pUser) : CZNCSock() {
 	m_mueChanModes['i'] = NoArg;
 	m_mueChanModes['n'] = NoArg;
 
+	pUser->SetIRCSocket(this);
+
 	// RFC says a line can have 512 chars max, but we don't care ;)
 	SetMaxBufferThreshold(1024);
 }
@@ -64,6 +73,10 @@ CIRCSock::~CIRCSock() {
 }
 
 void CIRCSock::Quit(const CString& sQuitMsg) {
+	if (!m_bAuthed) {
+		Close(CLT_NOW);
+		return;
+	}
 	CString sMsg = (!sQuitMsg.empty()) ? sQuitMsg : m_pUser->GetQuitMsg();
 	PutIRC("QUIT :" + sMsg);
 	Close(CLT_AFTERWRITE);
@@ -79,15 +92,16 @@ void CIRCSock::ReadLine(const CString& sData) {
 	MODULECALL(OnRaw(sLine), m_pUser, NULL, return);
 
 	if (sLine.Equals("PING ", false, 5)) {
+		// Generate a reply and don't forward this to any user,
+		// we don't want any PING forwarded
 		PutIRC("PONG " + sLine.substr(5));
-		m_pUser->PutUser(sLine);
 		return;
-	} else if (sLine.Token(1).Equals("PONG") && sLine.Token(3).Equals(":ZNC")) {
-		// We asked for this so don't forward the reply to clients.
+	} else if (sLine.Token(1).Equals("PONG")) {
+		// Block PONGs, we already responded to the pings
 		return;
 	} else if (sLine.Equals("ERROR ", false, 6)) {
 		//ERROR :Closing Link: nick[24.24.24.24] (Excess Flood)
-		CString sError(sLine.substr(7));
+		CString sError(sLine.substr(6));
 
 		if (sError.Left(1) == ":") {
 			sError.LeftChomp();
@@ -135,7 +149,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				SetNick(sNick);
 
-				MODULECALL(OnIRCConnected(), m_pUser, NULL, );
+				MODULECALL(OnIRCConnected(), m_pUser, NULL, NOTHING);
 
 				m_pUser->ClearRawBuffer();
 				m_pUser->AddRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
@@ -315,6 +329,11 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 				break;
 			}
+			case 451:
+				// :irc.server.com 451 CAP :You have not registered
+				// Servers that dont support CAP will give us this error, dont send it to the client
+				if (sNick.Equals("CAP"))
+					return;
 			case 470: {
 				// :irc.unreal.net 470 mynick [Link] #chan1 has become full, so you are automatically being transferred to the linked channel #chan2
 				// :mccaffrey.freenode.net 470 mynick #electronics ##electronics :Forwarding to another channel
@@ -368,7 +387,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				SetNick(sNewNick);
 			}
 
-			MODULECALL(OnNick(Nick, sNewNick, vFoundChans), m_pUser, NULL, );
+			MODULECALL(OnNick(Nick, sNewNick, vFoundChans), m_pUser, NULL, NOTHING);
 
 			if (!bIsVisible) {
 				return;
@@ -406,7 +425,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 			}
 
-			MODULECALL(OnQuit(Nick, sMessage, vFoundChans), m_pUser, NULL, );
+			MODULECALL(OnQuit(Nick, sMessage, vFoundChans), m_pUser, NULL, NOTHING);
 
 			if (!bIsVisible) {
 				return;
@@ -436,7 +455,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 			if (pChan) {
 				pChan->AddNick(Nick.GetNickMask());
-				MODULECALL(OnJoin(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
+				MODULECALL(OnJoin(Nick.GetNickMask(), *pChan), m_pUser, NULL, NOTHING);
 
 				if (pChan->IsDetached()) {
 					return;
@@ -447,12 +466,13 @@ void CIRCSock::ReadLine(const CString& sData) {
 			if (sChan.Left(1) == ":") {
 				sChan.LeftChomp();
 			}
+			CString sMsg = sRest.Token(1, true).TrimPrefix_n(":");
 
 			CChan* pChan = m_pUser->FindChan(sChan);
 			bool bDetached = false;
 			if (pChan) {
 				pChan->RemNick(Nick.GetNick());
-				MODULECALL(OnPart(Nick.GetNickMask(), *pChan), m_pUser, NULL, );
+				MODULECALL(OnPart(Nick.GetNickMask(), *pChan, sMsg), m_pUser, NULL, NOTHING);
 
 				if (pChan->IsDetached())
 					bDetached = true;
@@ -480,7 +500,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 			CChan* pChan = m_pUser->FindChan(sTarget);
 			if (pChan) {
-				pChan->ModeChange(sModes, Nick.GetNick());
+				pChan->ModeChange(sModes, &Nick);
 
 				if (pChan->IsDetached()) {
 					return;
@@ -517,8 +537,10 @@ void CIRCSock::ReadLine(const CString& sData) {
 			CChan* pChan = m_pUser->FindChan(sChan);
 
 			if (pChan) {
+				MODULECALL(OnKick(Nick, sKickedNick, *pChan, sMsg), m_pUser, NULL, NOTHING);
+				// do not remove the nick till after the OnKick call, so modules
+				// can do Chan.FindNick or something to get more info.
 				pChan->RemNick(sKickedNick);
-				MODULECALL(OnKick(Nick, sKickedNick, *pChan, sMsg), m_pUser, NULL, );
 			}
 
 			if (GetNick().Equals(sKickedNick) && pChan) {
@@ -639,44 +661,94 @@ void CIRCSock::ReadLine(const CString& sData) {
 				m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " WALLOPS ", ":" + m_pUser->AddTimestamp(sMsg), false);
 			}
 		} else if (sCmd.Equals("CAP")) {
-			// sRest.Token(0) is most likely "*". No idea why, the
-			// CAP spec don't mention this, but all implementations
-			// I've seen add this extra asterisk
-			CString sSubCmd = sRest.Token(1);
-			CString sArgs = sRest.Token(2, true).TrimPrefix_n(":");
+			// CAPs are supported only before authorization.
+			if (!m_bAuthed) {
+				// sRest.Token(0) is most likely "*". No idea why, the
+				// CAP spec don't mention this, but all implementations
+				// I've seen add this extra asterisk
+				CString sSubCmd = sRest.Token(1);
 
-			if (sSubCmd == "LS" && !m_bAuthed) {
-				VCString vsTokens;
-				VCString::iterator it;
-				sArgs.Split(" ", vsTokens, false);
-
-				for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
-					if (*it == "multi-prefix" || *it == "userhost-in-names") {
-						PutIRC("CAP REQ " + *it);
-					}
+				// If the caplist of a reply is too long, it's split
+				// into multiple replies. A "*" is prepended to show
+				// that the list was split into multiple replies.
+				// This is useful mainly for LS. For ACK and NAK
+				// replies, there's no real need for this, because
+				// we request only 1 capability per line.
+				// If we will need to support broken servers or will
+				// send several requests per line, need to delay ACK
+				// actions until all ACK lines are received and
+				// to recognize past request of NAK by 100 chars
+				// of this reply.
+				CString sArgs;
+				if (sRest.Token(2) == "*") {
+					sArgs = sRest.Token(3, true).TrimPrefix_n(":");
+				} else {
+					sArgs = sRest.Token(2, true).TrimPrefix_n(":");
 				}
 
-				// Tell the IRC server we are done with CAP
-				PutIRC("CAP END");
-			} else if (sSubCmd == "ACK" && !m_bAuthed) {
-				VCString vsTokens;
-				VCString::iterator it;
-				sArgs.Split(" ", vsTokens, false);
+				if (sSubCmd == "LS") {
+					VCString vsTokens;
+					VCString::iterator it;
+					sArgs.Split(" ", vsTokens, false);
 
-				for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
-					if (*it == "multi-prefix") {
+					for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
+						if (OnServerCapAvailable(*it) || *it == "multi-prefix" || *it == "userhost-in-names") {
+							m_ssPendingCaps.insert(*it);
+						}
+					}
+				} else if (sSubCmd == "ACK") {
+					sArgs.Trim();
+					MODULECALL(OnServerCapResult(sArgs, true), m_pUser, NULL, NOTHING);
+					if ("multi-prefix" == sArgs) {
 						m_bNamesx = true;
-					} else if (*it == "userhost-in-names") {
+					} else if ("userhost-in-names" == sArgs) {
 						m_bUHNames = true;
 					}
+					m_ssAcceptedCaps.insert(sArgs);
+				} else if (sSubCmd == "NAK") {
+					// This should work because there's no [known]
+					// capability with length of name more than 100 characters.
+					sArgs.Trim();
+					MODULECALL(OnServerCapResult(sArgs, false), m_pUser, NULL, NOTHING);
 				}
+				
+				SendNextCap();
 			}
+			// Don't forward any CAP stuff to the client
+			return;
 		}
 	}
 
 	m_pUser->PutUser(sLine);
 }
 
+void CIRCSock::SendNextCap() {
+	if (!m_uCapPaused) {
+		if (m_ssPendingCaps.empty()) {
+			// We already got all needed ACK/NAK replies.
+			PutIRC("CAP END");
+		} else {
+			CString sCap = *m_ssPendingCaps.begin();
+			m_ssPendingCaps.erase(m_ssPendingCaps.begin());
+			PutIRC("CAP REQ :" + sCap);
+		}
+	}
+}
+
+void CIRCSock::PauseCap() {
+	++m_uCapPaused;
+}
+
+void CIRCSock::ResumeCap() {
+	--m_uCapPaused;
+	SendNextCap();
+}
+
+bool CIRCSock::OnServerCapAvailable(const CString& sCap) {
+	MODULECALL(OnServerCapAvailable(sCap), m_pUser, NULL, return true);
+	return false;
+}
+
 bool CIRCSock::OnCTCPReply(CNick& Nick, CString& sMessage) {
 	MODULECALL(OnCTCPReply(Nick, sMessage), m_pUser, NULL, return true);
 
@@ -750,15 +822,11 @@ bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
 
 bool CIRCSock::OnGeneralCTCP(CNick& Nick, CString& sMessage) {
 	const MCString& mssCTCPReplies = m_pUser->GetCTCPReplies();
-	MCString::const_iterator it = mssCTCPReplies.find(sMessage.AsUpper());
 	CString sQuery = sMessage.Token(0).AsUpper();
+	MCString::const_iterator it = mssCTCPReplies.find(sQuery);
 	bool bHaveReply = false;
 	CString sReply;
 
-	if (it == mssCTCPReplies.end()) {
-		it = mssCTCPReplies.find(sQuery);
-	}
-
 	if (it != mssCTCPReplies.end()) {
 		sReply = m_pUser->ExpandString(it->second);
 		bHaveReply = true;
@@ -773,6 +841,18 @@ bool CIRCSock::OnGeneralCTCP(CNick& Nick, CString& sMessage) {
 	}
 
 	if (!sReply.empty()) {
+		time_t now = time(NULL);
+		// If the last CTCP is older than m_uCTCPFloodTime, reset the counter
+		if (m_lastCTCP + m_uCTCPFloodTime < now)
+			m_uNumCTCP = 0;
+		m_lastCTCP = now;
+		// If we are over the limit, don't reply to this CTCP
+		if (m_uNumCTCP >= m_uCTCPFloodCount) {
+			DEBUG("CTCP flood detected - not replying to query");
+			return false;
+		}
+		m_uNumCTCP++;
+
 		PutIRC("NOTICE " + Nick.GetNick() + " :\001" + sQuery + " " + sReply + "\001");
 		return true;
 	}
@@ -861,7 +941,6 @@ void CIRCSock::SetNick(const CString& sNick) {
 
 void CIRCSock::Connected() {
 	DEBUG(GetSockName() << " == Connected()");
-	m_pUser->IRCConnected(this);
 
 	CString sPass = m_sPass;
 	CString sNick = m_pUser->GetNick();
@@ -884,7 +963,7 @@ void CIRCSock::Connected() {
 }
 
 void CIRCSock::Disconnected() {
-	MODULECALL(OnIRCDisconnected(), m_pUser, NULL, );
+	MODULECALL(OnIRCDisconnected(), m_pUser, NULL, NOTHING);
 
 	DEBUG(GetSockName() << " == Disconnected()");
 	if (!m_pUser->IsBeingDeleted() && m_pUser->GetIRCConnectEnabled() &&
@@ -915,7 +994,7 @@ void CIRCSock::SockError(int iErrno) {
 	CString sError;
 
 	if (iErrno == EDOM) {
-		sError = "Your VHost could not be resolved";
+		sError = "Your bind host could not be resolved";
 	} else if (iErrno == EADDRNOTAVAIL) {
 		// Csocket uses this if it can't resolve the dest host name
 		// ...but it also does generate this if bind() fails -.-
@@ -923,7 +1002,7 @@ void CIRCSock::SockError(int iErrno) {
 		if (GetBindHost().empty())
 			sError += " (Is your IRC server's host name valid?)";
 		else
-			sError += " (Is your IRC server's host name and vhost valid?)";
+			sError += " (Is your IRC server's host name and ZNC bind host valid?)";
 	} else {
 		sError = strerror(iErrno);
 	}
