@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,12 +9,14 @@
 #ifndef _IRCSOCK_H
 #define _IRCSOCK_H
 
+#include "zncconfig.h"
 #include "Socket.h"
 #include "Nick.h"
 
 // Forward Declarations
 class CChan;
 class CUser;
+class CClient;
 // !Forward Declarations
 
 class CIRCSock : public CZNCSock {
@@ -23,10 +25,11 @@ public:
 	virtual ~CIRCSock();
 
 	typedef enum {
-		ListArg		= 0,	// These values must line up with their position in the CHANMODE argument to raw 005
-		HasArg		= 1,
-		ArgWhenSet	= 2,
-		NoArg		= 3
+		// These values must line up with their position in the CHANMODE argument to raw 005
+		ListArg    = 0,
+		HasArg     = 1,
+		ArgWhenSet = 2,
+		NoArg      = 3
 	} EChanModeArgs;
 
 	// Message Handlers
@@ -38,6 +41,7 @@ public:
 	bool OnChanMsg(CNick& Nick, const CString& sChan, CString& sMessage);
 	bool OnPrivNotice(CNick& Nick, CString& sMessage);
 	bool OnChanNotice(CNick& Nick, const CString& sChan, CString& sMessage);
+	bool OnServerCapAvailable(const CString& sCap);
 	// !Message Handlers
 
 	virtual void ReadLine(const CString& sData);
@@ -50,7 +54,17 @@ public:
 
 	void PutIRC(const CString& sLine);
 	void ResetChans();
-	void Quit();
+	void Quit(const CString& sQuitMsg = "");
+
+	/** You can call this from CModule::OnServerCapResult to suspend
+	 *  sending other CAP requests and CAP END for a while. Each
+	 *  call to PauseCap should be balanced with a call to ResumeCap.
+	 */
+	void PauseCap();
+	/** If you used PauseCap, call this when CAP negotiation and logging in
+	 *  should be resumed again.
+	 */
+	void ResumeCap();
 
 	// Setters
 	void SetPass(const CString& s) { m_sPass = s; }
@@ -74,27 +88,39 @@ public:
 	const set<unsigned char>& GetUserModes() const { return m_scUserModes; }
 	// This is true if we are past raw 001
 	bool IsAuthed() const { return m_bAuthed; }
+	bool IsCapAccepted(const CString& sCap) { return 1 == m_ssAcceptedCaps.count(sCap); }
 	// !Getters
+
+	// This handles NAMESX and UHNAMES in a raw 353 reply
+	void ForwardRaw353(const CString& sLine) const;
+	void ForwardRaw353(const CString& sLine, CClient* pClient) const;
 private:
 	void SetNick(const CString& sNick);
 	void ParseISupport(const CString& sLine);
-	void ForwardRaw353(const CString& sLine) const;
 	// This is called when we connect and the nick we want is already taken
 	void SendAltNick(const CString& sBadNick);
+	void SendNextCap();
 protected:
-	bool						m_bISpoofReleased;
-	bool						m_bAuthed;
-	bool						m_bNamesx;
-	bool						m_bUHNames;
-	CString						m_sPerms;
-	CString						m_sPermModes;
-	set<unsigned char>			m_scUserModes;
-	map<unsigned char, EChanModeArgs>	m_mueChanModes;
-	CUser*						m_pUser;
-	CNick						m_Nick;
-	CString						m_sPass;
-	map<CString, CChan*>		m_msChans;
-	unsigned int				m_uMaxNickLen;
+	bool                                m_bISpoofReleased;
+	bool                                m_bAuthed;
+	bool                                m_bNamesx;
+	bool                                m_bUHNames;
+	CString                             m_sPerms;
+	CString                             m_sPermModes;
+	set<unsigned char>                  m_scUserModes;
+	map<unsigned char, EChanModeArgs>   m_mueChanModes;
+	CUser*                              m_pUser;
+	CNick                               m_Nick;
+	CString                             m_sPass;
+	map<CString, CChan*>                m_msChans;
+	unsigned int                        m_uMaxNickLen;
+	unsigned int                        m_uCapPaused;
+	SCString                            m_ssAcceptedCaps;
+	SCString                            m_ssPendingCaps;
+	time_t                              m_lastCTCP;
+	unsigned int                        m_uNumCTCP;
+	static const time_t                 m_uCTCPFloodTime;
+	static const unsigned int           m_uCTCPFloodCount;
 };
 
 #endif // !_IRCSOCK_H
