@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -104,7 +104,6 @@ const CString& CTimer::GetDescription() const { return m_sDescription; }
 
 
 CModule::CModule(ModHandle pDLL, CUser* pUser, const CString& sModName, const CString& sDataDir) {
-	m_bFake = false;
 	m_bGlobal = false;
 	m_pDLL = pDLL;
 	m_pManager = &(CZNC::Get().GetManager());;
@@ -120,7 +119,6 @@ CModule::CModule(ModHandle pDLL, CUser* pUser, const CString& sModName, const CS
 }
 
 CModule::CModule(ModHandle pDLL, const CString& sModName, const CString& sDataDir) {
-	m_bFake = false;
 	m_pDLL = pDLL;
 	m_pManager = &(CZNC::Get().GetManager());
 	m_pUser = NULL;
@@ -146,14 +144,13 @@ CModule::~CModule() {
 
 void CModule::SetUser(CUser* pUser) { m_pUser = pUser; }
 void CModule::SetClient(CClient* pClient) { m_pClient = pClient; }
-void CModule::Unload() { throw UNLOAD; }
 
 bool CModule::LoadRegistry() {
 	//CString sPrefix = (m_pUser) ? m_pUser->GetUserName() : ".global";
-	return (m_mssRegistry.ReadFromDisk(GetSavePath() + "/.registry", 0600) == MCString::MCS_SUCCESS);
+	return (m_mssRegistry.ReadFromDisk(GetSavePath() + "/.registry") == MCString::MCS_SUCCESS);
 }
 
-bool CModule::SaveRegistry() {
+bool CModule::SaveRegistry() const {
 	//CString sPrefix = (m_pUser) ? m_pUser->GetUserName() : ".global";
 	return (m_mssRegistry.WriteToDisk(GetSavePath() + "/.registry", 0600) == MCString::MCS_SUCCESS);
 }
@@ -167,8 +164,8 @@ bool CModule::SetNV(const CString & sName, const CString & sValue, bool bWriteTo
 	return true;
 }
 
-CString CModule::GetNV(const CString & sName) {
-	MCString::iterator it = m_mssRegistry.find(sName);
+CString CModule::GetNV(const CString & sName) const {
+	MCString::const_iterator it = m_mssRegistry.find(sName);
 
 	if (it != m_mssRegistry.end()) {
 		return it->second;
@@ -385,11 +382,76 @@ void CModule::ListSockets() {
 	PutModule(Table);
 }
 
+bool CModule::AddCommand(const CModCommand& Command)
+{
+	if (Command.GetFunction() == NULL)
+		return false;
+	if (Command.GetCommand().find(' ') != CString::npos)
+		return false;
+	if (FindCommand(Command.GetCommand()) != NULL)
+		return false;
+
+	m_mCommands[Command.GetCommand()] = Command;
+	return true;
+}
+
+bool CModule::AddCommand(const CString& sCmd, CModCommand::ModCmdFunc func, const CString& sArgs, const CString& sDesc)
+{
+	CModCommand cmd(sCmd, func, sArgs, sDesc);
+	return AddCommand(cmd);
+}
+
+void CModule::AddHelpCommand()
+{
+	AddCommand("Help", &CModule::HandleHelpCommand, "", "Generate this output");
+}
+
+bool CModule::RemCommand(const CString& sCmd)
+{
+	return m_mCommands.erase(sCmd) > 0;
+}
+
+const CModCommand* CModule::FindCommand(const CString& sCmd) const
+{
+	map<CString, CModCommand>::const_iterator it;
+	for (it = m_mCommands.begin(); it != m_mCommands.end(); ++it) {
+		if (!it->first.Equals(sCmd))
+			continue;
+		return &it->second;
+	}
+	return NULL;
+}
+
+bool CModule::HandleCommand(const CString& sLine) {
+	const CString& sCmd = sLine.Token(0);
+	const CModCommand* pCmd = FindCommand(sCmd);
+
+	if (pCmd) {
+		pCmd->Call(this, sLine);
+		return true;
+	}
+
+	OnUnknownModCommand(sLine);
+
+	return false;
+}
+
+void CModule::HandleHelpCommand(const CString& sLine) {
+	CTable Table;
+	map<CString, CModCommand>::const_iterator it;
+
+	CModCommand::InitHelp(Table);
+	for (it = m_mCommands.begin(); it != m_mCommands.end(); ++it)
+		it->second.AddHelp(Table);
+	PutModule(Table);
+}
+
 CString CModule::GetModNick() const { return ((m_pUser) ? m_pUser->GetStatusPrefix() : "*") + m_sModName; }
 
 // Webmods
 bool CModule::OnWebPreRequest(CWebSock& WebSock, const CString& sPageName) { return false; }
 bool CModule::OnWebRequest(CWebSock& WebSock, const CString& sPageName, CTemplate& Tmpl) { return false; }
+bool CModule::OnEmbeddedWebRequest(CWebSock& WebSock, const CString& sPageName, CTemplate& Tmpl) { return false; }
 // !Webmods
 
 bool CModule::OnLoad(const CString& sArgs, CString& sMessage) { sMessage = ""; return true; }
@@ -418,15 +480,22 @@ void CModule::OnMode(const CNick& OpNick, CChan& Channel, char uMode, const CStr
 CModule::EModRet CModule::OnRaw(CString& sLine) { return CONTINUE; }
 
 CModule::EModRet CModule::OnStatusCommand(CString& sCommand) { return CONTINUE; }
-void CModule::OnModCommand(const CString& sCommand) {}
 void CModule::OnModNotice(const CString& sMessage) {}
 void CModule::OnModCTCP(const CString& sMessage) {}
 
+void CModule::OnModCommand(const CString& sCommand) {
+	HandleCommand(sCommand);
+}
+void CModule::OnUnknownModCommand(const CString& sLine) {
+	if (!m_mCommands.empty())
+		PutModule("Unknown command!");
+}
+
 void CModule::OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans) {}
 void CModule::OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans) {}
 void CModule::OnKick(const CNick& Nick, const CString& sKickedNick, CChan& Channel, const CString& sMessage) {}
 void CModule::OnJoin(const CNick& Nick, CChan& Channel) {}
-void CModule::OnPart(const CNick& Nick, CChan& Channel) {}
+void CModule::OnPart(const CNick& Nick, CChan& Channel, const CString& sMessage) {}
 
 CModule::EModRet CModule::OnChanBufferStarting(CChan& Chan, CClient& Client) { return CONTINUE; }
 CModule::EModRet CModule::OnChanBufferEnding(CChan& Chan, CClient& Client) { return CONTINUE; }
@@ -458,7 +527,9 @@ CModule::EModRet CModule::OnChanNotice(CNick& Nick, CChan& Channel, CString& sMe
 CModule::EModRet CModule::OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) { return CONTINUE; }
 CModule::EModRet CModule::OnTimerAutoJoin(CChan& Channel) { return CONTINUE; }
 
-ModHandle CModule::GetDLL() { return m_pDLL; }
+bool CModule::OnServerCapAvailable(const CString& sCap) { return false; }
+void CModule::OnServerCapResult(const CString& sCap, bool bSuccess) {}
+
 bool CModule::PutIRC(const CString& sLine) {
 	return (m_pUser) ? m_pUser->PutIRC(sLine) : false;
 }
@@ -468,31 +539,25 @@ bool CModule::PutUser(const CString& sLine) {
 bool CModule::PutStatus(const CString& sLine) {
 	return (m_pUser) ? m_pUser->PutStatus(sLine, m_pClient) : false;
 }
-unsigned int CModule::PutModule(const CTable& table, const CString& sIdent, const CString& sHost) {
+unsigned int CModule::PutModule(const CTable& table) {
 	if (!m_pUser)
 		return 0;
 
 	unsigned int idx = 0;
 	CString sLine;
 	while (table.GetLine(idx++, sLine))
-		PutModule(sLine, sIdent, sHost);
+		PutModule(sLine);
 	return idx - 1;
 }
-bool CModule::PutModule(const CString& sLine, const CString& sIdent, const CString& sHost) {
+bool CModule::PutModule(const CString& sLine) {
 	if (!m_pUser)
 		return false;
-	return m_pUser->PutUser(":" + GetModNick() + "!" +
-		(sIdent.empty() ? GetModName() : sIdent) + "@" + sHost +
-		" PRIVMSG " + m_pUser->GetCurNick() + " :" + sLine,
-		m_pClient);
+	return m_pUser->PutModule(GetModName(), sLine, m_pClient);
 }
-bool CModule::PutModNotice(const CString& sLine, const CString& sIdent, const CString& sHost) {
+bool CModule::PutModNotice(const CString& sLine) {
 	if (!m_pUser)
 		return false;
-	return m_pUser->PutUser(":" + GetModNick() + "!" +
-		(sIdent.empty() ? GetModName() : sIdent) + "@" + sHost +
-		" NOTICE " + m_pUser->GetCurNick() + " :" + sLine,
-		m_pClient);
+	return m_pUser->PutModNotice(GetModName(), sLine, m_pClient);
 }
 
 ///////////////////
@@ -504,7 +569,18 @@ CModule::EModRet CGlobalModule::OnDeleteUser(CUser& User) { return CONTINUE; }
 void CGlobalModule::OnClientConnect(CZNCSock* pClient, const CString& sHost, unsigned short uPort) {}
 CModule::EModRet CGlobalModule::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) { return CONTINUE; }
 void CGlobalModule::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {}
-CModule::EModRet CGlobalModule::OnUnknownUserRaw(CClient* pClient, CString& sLine) { return CONTINUE; }
+CModule::EModRet CGlobalModule::OnUnknownUserRaw(CString& sLine) { return CONTINUE; }
+void CGlobalModule::OnClientCapLs(SCString& ssCaps) {}
+bool CGlobalModule::IsClientCapSupported(const CString& sCap, bool bState) { return false; }
+void CGlobalModule::OnClientCapRequest(const CString& sCap, bool bState) {}
+CModule::EModRet CGlobalModule::OnModuleLoading(const CString& sModName, const CString& sArgs,
+		bool& bSuccess, CString& sRetMsg) { return CONTINUE; }
+CModule::EModRet CGlobalModule::OnModuleUnloading(CModule* pModule, bool& bSuccess, CString& sRetMsg) {
+	return CONTINUE;
+}
+CModule::EModRet CGlobalModule::OnGetModInfo(CModInfo& ModInfo, const CString& sModule,
+		bool& bSuccess, CString& sRetMsg) { return CONTINUE; }
+void CGlobalModule::OnGetAvailableMods(set<CModInfo>& ssMods, bool bGlobal) {}
 
 
 CModules::CModules() {
@@ -528,7 +604,7 @@ bool CModules::OnBoot() {
 	for (unsigned int a = 0; a < size(); a++) {
 		try {
 			if (!(*this)[a]->OnBoot()) {
-				return false;
+				return true;
 			}
 		} catch (CModule::EModException e) {
 			if (e == CModule::UNLOAD) {
@@ -537,7 +613,7 @@ bool CModules::OnBoot() {
 		}
 	}
 
-	return true;
+	return false;
 }
 
 bool CModules::OnPreRehash() { MODUNLOADCHK(OnPreRehash()); return false; }
@@ -577,7 +653,7 @@ bool CModules::OnQuit(const CNick& Nick, const CString& sMessage, const vector<C
 bool CModules::OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans) { MODUNLOADCHK(OnNick(Nick, sNewNick, vChans)); return false; }
 bool CModules::OnKick(const CNick& Nick, const CString& sKickedNick, CChan& Channel, const CString& sMessage) { MODUNLOADCHK(OnKick(Nick, sKickedNick, Channel, sMessage)); return false; }
 bool CModules::OnJoin(const CNick& Nick, CChan& Channel) { MODUNLOADCHK(OnJoin(Nick, Channel)); return false; }
-bool CModules::OnPart(const CNick& Nick, CChan& Channel) { MODUNLOADCHK(OnPart(Nick, Channel)); return false; }
+bool CModules::OnPart(const CNick& Nick, CChan& Channel, const CString& sMessage) { MODUNLOADCHK(OnPart(Nick, Channel, sMessage)); return false; }
 bool CModules::OnChanBufferStarting(CChan& Chan, CClient& Client) { MODHALTCHK(OnChanBufferStarting(Chan, Client)); }
 bool CModules::OnChanBufferEnding(CChan& Chan, CClient& Client) { MODHALTCHK(OnChanBufferEnding(Chan, Client)); }
 bool CModules::OnChanBufferPlayLine(CChan& Chan, CClient& Client, CString& sLine) { MODHALTCHK(OnChanBufferPlayLine(Chan, Client, sLine)); }
@@ -598,6 +674,35 @@ bool CModules::OnModCommand(const CString& sCommand) { MODUNLOADCHK(OnModCommand
 bool CModules::OnModNotice(const CString& sMessage) { MODUNLOADCHK(OnModNotice(sMessage)); return false; }
 bool CModules::OnModCTCP(const CString& sMessage) { MODUNLOADCHK(OnModCTCP(sMessage)); return false; }
 
+// Why MODHALTCHK works only with functions returning EModRet ? :(
+bool CModules::OnServerCapAvailable(const CString& sCap) {
+	bool bResult = false;
+	for (unsigned int a = 0; a < size(); ++a) {
+		try {
+			CModule* pMod = (*this)[a];
+			CClient* pOldClient = pMod->GetClient();
+			pMod->SetClient(m_pClient);
+			if (m_pUser) {
+				CUser* pOldUser = pMod->GetUser();
+				pMod->SetUser(m_pUser);
+				bResult |= pMod->OnServerCapAvailable(sCap);
+				pMod->SetUser(pOldUser);
+			} else {
+				// WTF? Is that possible?
+				bResult |= pMod->OnServerCapAvailable(sCap);
+			}
+			pMod->SetClient(pOldClient);
+		} catch (CModule::EModException e) {
+			if (CModule::UNLOAD == e) {
+				UnloadModule((*this)[a]->GetModName());
+			}
+		}
+	}
+	return bResult;
+}
+
+bool CModules::OnServerCapResult(const CString& sCap, bool bSuccess) { MODUNLOADCHK(OnServerCapResult(sCap, bSuccess)); return false; }
+
 ////////////////////
 // CGlobalModules //
 ////////////////////
@@ -613,22 +718,81 @@ bool CGlobalModules::OnDeleteUser(CUser& User) {
 	GLOBALMODHALTCHK(OnDeleteUser(User));
 }
 
-void CGlobalModules::OnClientConnect(CZNCSock* pClient, const CString& sHost, unsigned short uPort) {
+bool CGlobalModules::OnClientConnect(CZNCSock* pClient, const CString& sHost, unsigned short uPort) {
 	GLOBALMODCALL(OnClientConnect(pClient, sHost, uPort));
+	return false;
 }
 
 bool CGlobalModules::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) {
 	GLOBALMODHALTCHK(OnLoginAttempt(Auth));
 }
 
-void CGlobalModules::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {
+bool CGlobalModules::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {
 	GLOBALMODCALL(OnFailedLogin(sUsername, sRemoteIP));
+	return false;
 }
 
-bool CGlobalModules::OnUnknownUserRaw(CClient* pClient, CString& sLine) {
-	GLOBALMODHALTCHK(OnUnknownUserRaw(pClient, sLine));
+bool CGlobalModules::OnUnknownUserRaw(CString& sLine) {
+	GLOBALMODHALTCHK(OnUnknownUserRaw(sLine));
 }
 
+bool CGlobalModules::OnClientCapLs(SCString& ssCaps) {
+	GLOBALMODCALL(OnClientCapLs(ssCaps));
+	return false;
+}
+
+// Maybe create new macro for this?
+bool CGlobalModules::IsClientCapSupported(const CString& sCap, bool bState) {
+	bool bResult = false;
+	for (unsigned int a = 0; a < size(); ++a) {
+		try {
+			CGlobalModule* pMod = (CGlobalModule*) (*this)[a];
+			CClient* pOldClient = pMod->GetClient();
+			pMod->SetClient(m_pClient);
+			if (m_pUser) {
+				CUser* pOldUser = pMod->GetUser();
+				pMod->SetUser(m_pUser);
+				bResult |= pMod->IsClientCapSupported(sCap, bState);
+				pMod->SetUser(pOldUser);
+			} else {
+				// WTF? Is that possible?
+				bResult |= pMod->IsClientCapSupported(sCap, bState);
+			}
+			pMod->SetClient(pOldClient);
+		} catch (CModule::EModException e) {
+			if (CModule::UNLOAD == e) {
+				UnloadModule((*this)[a]->GetModName());
+			}
+		}
+	}
+	return bResult;
+}
+
+bool CGlobalModules::OnClientCapRequest(const CString& sCap, bool bState) {
+	GLOBALMODCALL(OnClientCapRequest(sCap, bState));
+	return false;
+}
+
+bool CGlobalModules::OnModuleLoading(const CString& sModName, const CString& sArgs,
+		bool& bSuccess, CString& sRetMsg) {
+	GLOBALMODHALTCHK(OnModuleLoading(sModName, sArgs, bSuccess, sRetMsg));
+}
+
+bool CGlobalModules::OnModuleUnloading(CModule* pModule, bool& bSuccess, CString& sRetMsg) {
+	GLOBALMODHALTCHK(OnModuleUnloading(pModule, bSuccess, sRetMsg));
+}
+
+bool CGlobalModules::OnGetModInfo(CModInfo& ModInfo, const CString& sModule,
+		bool& bSuccess, CString& sRetMsg) {
+	GLOBALMODHALTCHK(OnGetModInfo(ModInfo, sModule, bSuccess, sRetMsg));
+}
+
+bool CGlobalModules::OnGetAvailableMods(set<CModInfo>& ssMods, bool bGlobal) {
+	GLOBALMODCALL(OnGetAvailableMods(ssMods, bGlobal));
+	return false;
+}
+
+
 CModule* CModules::FindModule(const CString& sModule) const {
 	for (unsigned int a = 0; a < size(); a++) {
 		if (sModule.Equals((*this)[a]->GetModName())) {
@@ -639,7 +803,7 @@ CModule* CModules::FindModule(const CString& sModule) const {
 	return NULL;
 }
 
-bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg, bool bFake) {
+bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg) {
 	sRetMsg = "";
 
 	if (FindModule(sModule) != NULL) {
@@ -647,15 +811,8 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 		return false;
 	}
 
-	if (bFake) {
-		CModule* pModule = new CModule(NULL, sModule, "");
-		pModule->SetArgs(sArgs);
-		pModule->SetDescription("<<Fake Module>>");
-		pModule->SetFake(true);
-		push_back(pModule);
-		sRetMsg = "Loaded fake module [" + sModule + "]";
-		return true;
-	}
+	bool bSuccess;
+	GLOBALMODULECALL(OnModuleLoading(sModule, sArgs, bSuccess, sRetMsg), pUser, NULL, return bSuccess);
 
 	CString sModPath, sDataPath;
 	CString sDesc;
@@ -760,18 +917,8 @@ bool CModules::UnloadModule(const CString& sModule, CString& sRetMsg) {
 		return false;
 	}
 
-	if (pModule->IsFake()) {
-		for (iterator it = begin(); it != end(); ++it) {
-			if (*it == pModule) {
-				erase(it);
-				sRetMsg = "Fake module [" + sMod + "] unloaded";
-				return true;
-			}
-		}
-
-		sRetMsg = "Fake module [" + sMod + "] not loaded.";
-		return false;
-	}
+	bool bSuccess;
+	GLOBALMODULECALL(OnModuleUnloading(pModule, bSuccess, sRetMsg), pModule->GetUser(), NULL, return bSuccess);
 
 	ModHandle p = pModule->GetDLL();
 
@@ -821,6 +968,9 @@ bool CModules::ReloadModule(const CString& sModule, const CString& sArgs, CUser*
 bool CModules::GetModInfo(CModInfo& ModInfo, const CString& sModule, CString& sRetMsg) {
 	CString sModPath, sTmp;
 
+	bool bSuccess;
+	GLOBALMODULECALL(OnGetModInfo(ModInfo, sModule, bSuccess, sRetMsg), NULL, NULL, return bSuccess);
+
 	if (!FindModPath(sModule, sModPath, sTmp)) {
 		sRetMsg = "Unable to find module [" + sModule + "]";
 		return false;
@@ -880,6 +1030,8 @@ void CModules::GetAvailableMods(set<CModInfo>& ssMods, bool bGlobal) {
 			}
 		}
 	}
+
+	GLOBALMODULECALL(OnGetAvailableMods(ssMods, bGlobal), NULL, NULL, NOTHING);
 }
 
 bool CModules::FindModPath(const CString& sModule, CString& sModPath,
@@ -987,7 +1139,7 @@ ModHandle CModules::OpenModule(const CString& sModule, const CString& sModPath,
 	if (!GetDesc) {
 		dlclose(p);
 		sRetMsg = "Could not find ZNCModDescription() in module [" + sModule + "]";
-		return false;
+		return NULL;
 	}
 
 	if (CModule::GetCoreVersion() != Version()) {
@@ -1002,3 +1154,40 @@ ModHandle CModules::OpenModule(const CString& sModule, const CString& sModPath,
 
 	return p;
 }
+
+CModCommand::CModCommand()
+	: m_sCmd(), m_pFunc(NULL), m_sArgs(), m_sDesc()
+{
+}
+
+CModCommand::CModCommand(const CString& sCmd, ModCmdFunc func, const CString& sArgs, const CString& sDesc)
+	: m_sCmd(sCmd), m_pFunc(func), m_sArgs(sArgs), m_sDesc(sDesc)
+{
+}
+
+CModCommand::CModCommand(const CModCommand& other)
+	: m_sCmd(other.m_sCmd), m_pFunc(other.m_pFunc), m_sArgs(other.m_sArgs), m_sDesc(other.m_sDesc)
+{
+}
+
+CModCommand& CModCommand::operator=(const CModCommand& other)
+{
+	m_sCmd = other.m_sCmd;
+	m_pFunc = other.m_pFunc;
+	m_sArgs = other.m_sArgs;
+	m_sDesc = other.m_sDesc;
+	return *this;
+}
+
+void CModCommand::InitHelp(CTable& Table) {
+	Table.AddColumn("Command");
+	Table.AddColumn("Arguments");
+	Table.AddColumn("Description");
+}
+
+void CModCommand::AddHelp(CTable& Table) const {
+	Table.AddRow();
+	Table.SetCell("Command", GetCommand());
+	Table.SetCell("Arguments", GetArgs());
+	Table.SetCell("Description", GetDescription());
+}
