@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -48,11 +48,15 @@ protected:
 	CUser* m_pUser;
 };
 
-CUser::CUser(const CString& sUserName) {
+CUser::CUser(const CString& sUserName)
+		: m_sUserName(sUserName), m_sCleanUserName(MakeCleanUserName(sUserName))
+{
+	// set paths that depend on the user name:
+	m_sUserPath = CZNC::Get().GetUserPath() + "/" + m_sUserName;
+	m_sDLPath = m_sUserPath + "/downloads";
+
 	m_pIRCSock = NULL;
 	m_fTimezoneOffset = 0;
-	m_uConnectTime = 0;
-	SetUserName(sUserName);
 	m_sNick = m_sCleanUserName;
 	m_sIdent = m_sCleanUserName;
 	m_sRealName = sUserName;
@@ -62,6 +66,7 @@ CUser::CUser(const CString& sUserName) {
 	m_pModules = new CModules;
 	m_RawBuffer.SetLineCount(100);   // This should be more than enough raws, especially since we are buffering the MOTD separately
 	m_MotdBuffer.SetLineCount(200);  // This should be more than enough motd lines
+	m_QueryBuffer.SetLineCount(250);
 	m_bMultiClients = true;
 	m_bBounceDCCs = true;
 	m_eHashType = HASH_NONE;
@@ -69,7 +74,7 @@ CUser::CUser(const CString& sUserName) {
 	m_bDenyLoadMod = false;
 	m_bAdmin= false;
 	m_bIRCAway = false;
-	m_bDenySetVHost= false;
+	m_bDenySetBindHost= false;
 	m_sStatusPrefix = "*";
 	m_sChanPrefixes = "";
 	m_uBufferCount = 50;
@@ -107,10 +112,8 @@ CUser::~CUser() {
 }
 
 void CUser::DelModules() {
-	if (m_pModules) {
-		delete m_pModules;
-		m_pModules = NULL;
-	}
+	delete m_pModules;
+	m_pModules = NULL;
 }
 
 bool CUser::UpdateModule(const CString &sModule) {
@@ -158,10 +161,23 @@ void CUser::DelServers()
 	m_vServers.clear();
 }
 
-void CUser::IRCConnected(CIRCSock* pIRCSock) {
+void CUser::SetIRCSocket(CIRCSock* pIRCSock) {
 	m_pIRCSock = pIRCSock;
 }
 
+bool CUser::IsIRCConnected() const
+{
+	const CIRCSock* pSock = GetIRCSock();
+
+	if (!pSock)
+		return false;
+
+	if (!pSock->IsConnected())
+		return false;
+
+	return true;
+}
+
 void CUser::IRCDisconnected() {
 	m_pIRCSock = NULL;
 
@@ -196,7 +212,8 @@ CString& CUser::ExpandString(const CString& sStr, CString& sRet) const {
 	sRet.Replace("%altnick%", GetAltNick());
 	sRet.Replace("%ident%", GetIdent());
 	sRet.Replace("%realname%", GetRealName());
-	sRet.Replace("%vhost%", GetVHost());
+	sRet.Replace("%vhost%", GetBindHost());
+	sRet.Replace("%bindhost%", GetBindHost());
 	sRet.Replace("%version%", CZNC::GetVersion());
 	sRet.Replace("%time%", sTime);
 	sRet.Replace("%uptime%", CZNC::Get().GetUptime());
@@ -331,13 +348,11 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 		return false;
 	}
 
+	// user names can only specified for the constructor, changing it later
+	// on breaks too much stuff (e.g. lots of paths depend on the user name)
 	if (GetUserName() != User.GetUserName()) {
-		if (CZNC::Get().FindUser(User.GetUserName())) {
-			sErrorRet = "New username already exists";
-			return false;
-		}
-
-		SetUserName(User.GetUserName());
+		DEBUG("Ignoring username in CUser::Clone(), old username [" << GetUserName()
+				<< "]; New username [" << User.GetUserName() << "]");
 	}
 
 	if (!User.GetPass().empty()) {
@@ -349,12 +364,12 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	SetIdent(User.GetIdent(false));
 	SetRealName(User.GetRealName());
 	SetStatusPrefix(User.GetStatusPrefix());
-	SetVHost(User.GetVHost());
-	SetDCCVHost(User.GetDCCVHost());
+	SetBindHost(User.GetBindHost());
+	SetDCCBindHost(User.GetDCCBindHost());
 	SetQuitMsg(User.GetQuitMsg());
 	SetSkinName(User.GetSkinName());
 	SetDefaultChanModes(User.GetDefaultChanModes());
-	SetBufferCount(User.GetBufferCount());
+	SetBufferCount(User.GetBufferCount(), true);
 	SetJoinTries(User.JoinTries());
 	SetMaxJoins(User.MaxJoins());
 
@@ -452,7 +467,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	SetUseClientIP(User.UseClientIP());
 	SetDenyLoadMod(User.DenyLoadMod());
 	SetAdmin(User.IsAdmin());
-	SetDenySetVHost(User.DenySetVHost());
+	SetDenySetBindHost(User.DenySetBindHost());
 	SetTimestampAppend(User.GetTimestampAppend());
 	SetTimestampPrepend(User.GetTimestampPrepend());
 	SetTimestampFormat(User.GetTimestampFormat());
@@ -602,7 +617,7 @@ bool CUser::DelChan(const CString& sName) {
 	return false;
 }
 
-bool CUser::PrintLine(CFile& File, CString sName, CString sValue) {
+bool CUser::PrintLine(CFile& File, CString sName, CString sValue) const {
 	sName.Trim();
 	sValue.Trim();
 
@@ -639,8 +654,8 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "AltNick", GetAltNick());
 	PrintLine(File, "Ident", GetIdent());
 	PrintLine(File, "RealName", GetRealName());
-	PrintLine(File, "VHost", GetVHost());
-	PrintLine(File, "DCCVHost", GetDCCVHost());
+	PrintLine(File, "BindHost", GetBindHost());
+	PrintLine(File, "DCCBindHost", GetDCCBindHost());
 	PrintLine(File, "QuitMsg", GetQuitMsg());
 	if (CZNC::Get().GetStatusPrefix() != GetStatusPrefix())
 		PrintLine(File, "StatusPrefix", GetStatusPrefix());
@@ -652,7 +667,7 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "BounceDCCs", CString(BounceDCCs()));
 	PrintLine(File, "DenyLoadMod", CString(DenyLoadMod()));
 	PrintLine(File, "Admin", CString(IsAdmin()));
-	PrintLine(File, "DenySetVHost", CString(DenySetVHost()));
+	PrintLine(File, "DenySetBindHost", CString(DenySetBindHost()));
 	PrintLine(File, "DCCLookupMethod", CString((UseClientIP()) ? "client" : "default"));
 	PrintLine(File, "TimestampFormat", GetTimestampFormat());
 	PrintLine(File, "AppendTimestamp", CString(GetTimestampAppend()));
@@ -674,7 +689,7 @@ bool CUser::WriteConfig(CFile& File) {
 
 	// CTCP Replies
 	if (!m_mssCTCPReplies.empty()) {
-		for (MCString::iterator itb = m_mssCTCPReplies.begin(); itb != m_mssCTCPReplies.end(); ++itb) {
+		for (MCString::const_iterator itb = m_mssCTCPReplies.begin(); itb != m_mssCTCPReplies.end(); ++itb) {
 			PrintLine(File, "CTCPReply", itb->first.AsUpper() + " " + itb->second);
 		}
 
@@ -714,7 +729,7 @@ bool CUser::WriteConfig(CFile& File) {
 		}
 	}
 
-	MODULECALL(OnWriteUserConfig(File), this, NULL,);
+	MODULECALL(OnWriteUserConfig(File), this, NULL, NOTHING);
 
 	File.Write("</User>\n");
 
@@ -742,6 +757,7 @@ void CUser::JoinChans() {
 	// still be able to join the rest of your channels.
 	unsigned int start = rand() % m_vChans.size();
 	unsigned int uJoins = m_uMaxJoins;
+	set<CChan*> sChans;
 	for (unsigned int a = 0; a < m_vChans.size(); a++) {
 		unsigned int idx = (start + a) % m_vChans.size();
 		CChan* pChan = m_vChans[idx];
@@ -749,11 +765,50 @@ void CUser::JoinChans() {
 			if (!JoinChan(pChan))
 				continue;
 
+			sChans.insert(pChan);
+
 			// Limit the number of joins
 			if (uJoins != 0 && --uJoins == 0)
-				return;
+				break;
 		}
 	}
+
+	while (!sChans.empty())
+		JoinChans(sChans);
+}
+
+void CUser::JoinChans(set<CChan*>& sChans) {
+	CString sKeys, sJoin;
+	bool bHaveKey = false;
+	size_t uiJoinLength = strlen("JOIN ");
+
+	while (!sChans.empty()) {
+		set<CChan*>::iterator it = sChans.begin();
+		const CString& sName = (*it)->GetName();
+		const CString& sKey = (*it)->GetKey();
+		size_t len = sName.length() + sKey.length();
+		len += 2; // two comma
+
+		if (!sKeys.empty() && uiJoinLength + len >= 512)
+			break;
+
+		if (!sJoin.empty()) {
+			sJoin += ",";
+			sKeys += ",";
+		}
+		uiJoinLength += len;
+		sJoin += sName;
+		if (!sKey.empty()) {
+			sKeys += sKey;
+			bHaveKey = true;
+		}
+		sChans.erase(it);
+	}
+
+	if (bHaveKey)
+		PutIRC("JOIN " + sJoin + " " + sKeys);
+	else
+		PutIRC("JOIN " + sJoin);
 }
 
 bool CUser::JoinChan(CChan* pChan) {
@@ -763,8 +818,6 @@ bool CUser::JoinChan(CChan* pChan) {
 	} else {
 		pChan->IncJoinTries();
 		MODULECALL(OnTimerAutoJoin(*pChan), this, NULL, return false);
-
-		PutIRC("JOIN " + pChan->GetName() + " " + pChan->GetKey());
 		return true;
 	}
 	return false;
@@ -926,6 +979,17 @@ CServer* CUser::GetCurrentServer() const {
 	return m_vServers[uIdx];
 }
 
+bool CUser::SetNextServer(const CServer* pServer) {
+	for (unsigned int a = 0; a < m_vServers.size(); a++) {
+		if (m_vServers[a] == pServer) {
+			m_uServerIdx = a;
+			return true;
+		}
+	}
+
+	return false;
+}
+
 bool CUser::CheckPass(const CString& sPass) const {
 	switch (m_eHashType)
 	{
@@ -971,8 +1035,8 @@ CString CUser::GetLocalIP() {
 }
 
 CString CUser::GetLocalDCCIP() {
-	if (!GetDCCVHost().empty())
-		return GetDCCVHost();
+	if (!GetDCCBindHost().empty())
+		return GetDCCBindHost();
 	return GetLocalIP();
 }
 
@@ -1043,6 +1107,20 @@ bool CUser::PutModule(const CString& sModule, const CString& sLine, CClient* pCl
 	return (pClient == NULL);
 }
 
+bool CUser::PutModNotice(const CString& sModule, const CString& sLine, CClient* pClient, CClient* pSkipClient) {
+	for (unsigned int a = 0; a < m_vClients.size(); a++) {
+		if ((!pClient || pClient == m_vClients[a]) && pSkipClient != m_vClients[a]) {
+			m_vClients[a]->PutModNotice(sModule, sLine);
+
+			if (pClient) {
+				return true;
+			}
+		}
+	}
+
+	return (pClient == NULL);
+}
+
 bool CUser::ResumeFile(unsigned short uPort, unsigned long uFileSize) {
 	CSockManager& Manager = CZNC::Get().GetManager();
 
@@ -1130,15 +1208,6 @@ CString CUser::MakeCleanUserName(const CString& sUserName) {
 }
 
 // Setters
-void CUser::SetUserName(const CString& s) {
-	m_sCleanUserName = CUser::MakeCleanUserName(s);
-	m_sUserName = s;
-
-	// set paths that depend on the user name:
-	m_sUserPath = CZNC::Get().GetUserPath() + "/" + m_sUserName;
-	m_sDLPath = GetUserPath() + "/downloads";
-}
-
 bool CUser::IsChan(const CString& sChan) const {
 	if (sChan.empty())
 		return false; // There is no way this is a chan
@@ -1152,8 +1221,8 @@ void CUser::SetNick(const CString& s) { m_sNick = s; }
 void CUser::SetAltNick(const CString& s) { m_sAltNick = s; }
 void CUser::SetIdent(const CString& s) { m_sIdent = s; }
 void CUser::SetRealName(const CString& s) { m_sRealName = s; }
-void CUser::SetVHost(const CString& s) { m_sVHost = s; }
-void CUser::SetDCCVHost(const CString& s) { m_sDCCVHost = s; }
+void CUser::SetBindHost(const CString& s) { m_sBindHost = s; }
+void CUser::SetDCCBindHost(const CString& s) { m_sDCCBindHost = s; }
 void CUser::SetPass(const CString& s, eHashType eHash, const CString& sSalt) {
 	m_sPass = s;
 	m_eHashType = eHash;
@@ -1164,15 +1233,20 @@ void CUser::SetBounceDCCs(bool b) { m_bBounceDCCs = b; }
 void CUser::SetUseClientIP(bool b) { m_bUseClientIP = b; }
 void CUser::SetDenyLoadMod(bool b) { m_bDenyLoadMod = b; }
 void CUser::SetAdmin(bool b) { m_bAdmin = b; }
-void CUser::SetDenySetVHost(bool b) { m_bDenySetVHost = b; }
+void CUser::SetDenySetBindHost(bool b) { m_bDenySetBindHost = b; }
 void CUser::SetDefaultChanModes(const CString& s) { m_sDefaultChanModes = s; }
 void CUser::SetIRCServer(const CString& s) { m_sIRCServer = s; }
 void CUser::SetQuitMsg(const CString& s) { m_sQuitMsg = s; }
-void CUser::SetBufferCount(unsigned int u) { m_uBufferCount = u; }
 void CUser::SetKeepBuffer(bool b) { m_bKeepBuffer = b; }
 
-void CUser::CheckIRCConnect()
-{
+bool CUser::SetBufferCount(unsigned int u, bool bForce) {
+	if (!bForce && u > CZNC::Get().GetMaxBufferSize())
+		return false;
+	m_uBufferCount = u;
+	return true;
+}
+
+void CUser::CheckIRCConnect() {
 	// Do we want to connect?
 	if (m_bIRCConnectEnabled && GetIRCSock() == NULL)
 		CZNC::Get().EnableConnectUser();
@@ -1187,14 +1261,22 @@ void CUser::SetIRCNick(const CNick& n) {
 }
 
 bool CUser::AddCTCPReply(const CString& sCTCP, const CString& sReply) {
+	// Reject CTCP requests containing spaces
+	if (sCTCP.find_first_of(' ') != CString::npos) {
+		return false;
+	}
+	// Reject empty CTCP requests
 	if (sCTCP.empty()) {
 		return false;
 	}
-
 	m_mssCTCPReplies[sCTCP.AsUpper()] = sReply;
 	return true;
 }
 
+bool CUser::DelCTCPReply(const CString& sCTCP) {
+	return m_mssCTCPReplies.erase(sCTCP) > 0;
+}
+
 bool CUser::SetStatusPrefix(const CString& s) {
 	if ((!s.empty()) && (s.length() < 6) && (s.find(' ') == CString::npos)) {
 		m_sStatusPrefix = (s.empty()) ? "*" : s;
@@ -1212,30 +1294,16 @@ const CString& CUser::GetNick(bool bAllowDefault) const { return (bAllowDefault
 const CString& CUser::GetAltNick(bool bAllowDefault) const { return (bAllowDefault && m_sAltNick.empty()) ? GetCleanUserName() : m_sAltNick; }
 const CString& CUser::GetIdent(bool bAllowDefault) const { return (bAllowDefault && m_sIdent.empty()) ? GetCleanUserName() : m_sIdent; }
 const CString& CUser::GetRealName() const { return m_sRealName.empty() ? m_sUserName : m_sRealName; }
-const CString& CUser::GetVHost() const { return m_sVHost; }
-const CString& CUser::GetDCCVHost() const { return m_sDCCVHost; }
+const CString& CUser::GetBindHost() const { return m_sBindHost; }
+const CString& CUser::GetDCCBindHost() const { return m_sDCCBindHost; }
 const CString& CUser::GetPass() const { return m_sPass; }
 CUser::eHashType CUser::GetPassHashType() const { return m_eHashType; }
 const CString& CUser::GetPassSalt() const { return m_sPassSalt; }
 
-bool CUser::ConnectPaused() {
-	if (!m_uConnectTime) {
-		m_uConnectTime = time(NULL);
-		return false;
-	}
-
-	if (time(NULL) - m_uConnectTime >= 5) {
-		m_uConnectTime = time(NULL);
-		return false;
-	}
-
-	return true;
-}
-
 bool CUser::UseClientIP() const { return m_bUseClientIP; }
 bool CUser::DenyLoadMod() const { return m_bDenyLoadMod; }
 bool CUser::IsAdmin() const { return m_bAdmin; }
-bool CUser::DenySetVHost() const { return m_bDenySetVHost; }
+bool CUser::DenySetBindHost() const { return m_bDenySetBindHost; }
 bool CUser::MultiClients() const { return m_bMultiClients; }
 bool CUser::BounceDCCs() const { return m_bBounceDCCs; }
 const CString& CUser::GetStatusPrefix() const { return m_sStatusPrefix; }
