@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -355,9 +355,14 @@ bool CTable::AddColumn(const CString& sName) {
 }
 
 unsigned int CTable::AddRow() {
+	// Don't add a row if no headers are defined
+	if (m_vsHeaders.empty()) {
+		return (unsigned int) -1;
+	}
+
 	// Add a vector with enough space for each column
 	push_back(vector<CString>(m_vsHeaders.size()));
-	return size() -1;
+	return size() - 1;
 }
 
 bool CTable::SetCell(const CString& sColumn, const CString& sValue, unsigned int uRowIdx) {
@@ -369,7 +374,12 @@ bool CTable::SetCell(const CString& sColumn, const CString& sValue, unsigned int
 		uRowIdx = size() -1;
 	}
 
-	(*this)[uRowIdx][GetColumnIndex(sColumn)] = sValue;
+	unsigned int uColIdx = GetColumnIndex(sColumn);
+
+	if (uColIdx == (unsigned int) -1)
+		return false;
+
+	(*this)[uRowIdx][uColIdx] = sValue;
 
 	if (m_msuWidths[sColumn] < sValue.size())
 		m_msuWidths[sColumn] = sValue.size();
@@ -380,7 +390,7 @@ bool CTable::SetCell(const CString& sColumn, const CString& sValue, unsigned int
 bool CTable::GetLine(unsigned int uIdx, CString& sLine) const {
 	stringstream ssRet;
 
-	if (!size()) {
+	if (empty()) {
 		return false;
 	}
 
@@ -430,33 +440,6 @@ bool CTable::GetLine(unsigned int uIdx, CString& sLine) const {
 	return false;
 }
 
-/*
-bool CTable::Output(std::ostream oOut) {
-	stringstream ssSep;
-
-	ssSep << "-+-";
-
-	oOut << endl << ssSep.str() << endl;
-
-	for (unsigned int b = 0; b < size(); b++) {
-		map<CString, CString>* pRow = (*this)[b];
-
-		oOut << " | ";
-
-		for (unsigned int c = 0; c < m_vsHeaders.size(); c++) {
-			oOut.width(GetColumnWidth(c));
-			oOut << (*pRow)[m_vsHeaders[c]];
-			oOut << " | ";
-		}
-
-		oOut << endl;
-	}
-
-	oOut << ssSep.str() << endl;
-	return true;
-}
-*/
-
 unsigned int CTable::GetColumnIndex(const CString& sName) const {
 	for (unsigned int i = 0; i < m_vsHeaders.size(); i++) {
 		if (m_vsHeaders[i] == sName)
@@ -465,7 +448,7 @@ unsigned int CTable::GetColumnIndex(const CString& sName) const {
 
 	DEBUG("CTable::GetColumnIndex(" + sName + ") failed");
 
-	return 0;
+	return (unsigned int) -1;
 }
 
 unsigned int CTable::GetColumnWidth(unsigned int uIdx) const {
