@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -28,7 +28,7 @@ CString::CString(unsigned long long i) : string() { stringstream s; s << i; *thi
 CString::CString(double i, int precision) : string() { stringstream s; s.precision(precision); s << std::fixed << i; *this = s.str(); }
 CString::CString(float i, int precision) : string() { stringstream s; s.precision(precision); s << std::fixed << i; *this = s.str(); }
 
-inline unsigned char* CString::strnchr(const unsigned char* src, unsigned char c, unsigned int iMaxBytes, unsigned char* pFill, unsigned int* piCount) const {
+unsigned char* CString::strnchr(const unsigned char* src, unsigned char c, unsigned int iMaxBytes, unsigned char* pFill, unsigned int* piCount) const {
 	for (unsigned int a = 0; a < iMaxBytes && *src; a++, src++) {
 		if (pFill) {
 			pFill[a] = *src;
@@ -805,13 +805,13 @@ CString CString::SHA256() const {
 }
 
 #ifdef HAVE_LIBSSL
-CString CString::Encrypt_n(const CString& sPass, const CString& sIvec) {
+CString CString::Encrypt_n(const CString& sPass, const CString& sIvec) const {
 	CString sRet;
 	sRet.Encrypt(sPass, sIvec);
 	return sRet;
 }
 
-CString CString::Decrypt_n(const CString& sPass, const CString& sIvec) {
+CString CString::Decrypt_n(const CString& sPass, const CString& sIvec) const {
 	CString sRet;
 	sRet.Decrypt(sPass, sIvec);
 	return sRet;
@@ -1031,7 +1031,7 @@ bool CString::RightChomp(unsigned int uLen) {
 }
 
 //////////////// MCString ////////////////
-int MCString::WriteToDisk(const CString& sPath, mode_t iMode) {
+MCString::status_t MCString::WriteToDisk(const CString& sPath, mode_t iMode) const {
 	CFile cFile(sPath);
 
 	if (this->empty()) {
@@ -1045,7 +1045,7 @@ int MCString::WriteToDisk(const CString& sPath, mode_t iMode) {
 		return MCS_EOPEN;
 	}
 
-	for (MCString::iterator it = this->begin(); it != this->end(); ++it) {
+	for (MCString::const_iterator it = this->begin(); it != this->end(); ++it) {
 		CString sKey = it->first;
 		CString sValue = it->second;
 		if (!WriteFilter(sKey, sValue)) {
@@ -1066,10 +1066,10 @@ int MCString::WriteToDisk(const CString& sPath, mode_t iMode) {
 	return MCS_SUCCESS;
 }
 
-int MCString::ReadFromDisk(const CString& sPath, mode_t iMode) {
+MCString::status_t MCString::ReadFromDisk(const CString& sPath) {
 	clear();
 	CFile cFile(sPath);
-	if (!cFile.Open(O_RDONLY, iMode)) {
+	if (!cFile.Open(O_RDONLY)) {
 		return MCS_EOPEN;
 	}
 
@@ -1095,7 +1095,7 @@ int MCString::ReadFromDisk(const CString& sPath, mode_t iMode) {
 
 static const char hexdigits[] = "0123456789abcdef";
 
-CString& MCString::Encode(CString& sValue) {
+CString& MCString::Encode(CString& sValue) const {
 	CString sTmp;
 	for (CString::iterator it = sValue.begin(); it != sValue.end(); ++it) {
 		if (isalnum(*it)) {
@@ -1111,7 +1111,7 @@ CString& MCString::Encode(CString& sValue) {
 	return sValue;
 }
 
-CString& MCString::Decode(CString& sValue) {
+CString& MCString::Decode(CString& sValue) const {
 	const char *pTmp = sValue.c_str();
 	char *endptr;
 	CString sTmp;
