@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -27,7 +27,7 @@ static const struct option g_LongOpts[] = {
 };
 
 static void GenerateHelp(const char *appname) {
-	CUtils::PrintMessage("USAGE: " + CString(appname) + " [options] [config]");
+	CUtils::PrintMessage("USAGE: " + CString(appname) + " [options]");
 	CUtils::PrintMessage("Options are:");
 	CUtils::PrintMessage("\t-h, --help         List available command line options (this page)");
 	CUtils::PrintMessage("\t-v, --version      Output version information and exit");
@@ -40,7 +40,7 @@ static void GenerateHelp(const char *appname) {
 #ifdef HAVE_LIBSSL
 	CUtils::PrintMessage("\t-p, --makepem      Generates a pemfile for use with SSL");
 #endif /* HAVE_LIBSSL */
-	CUtils::PrintMessage("\t-d, --datadir      Set a different znc repository (default is ~/.znc)");
+	CUtils::PrintMessage("\t-d, --datadir      Set a different ZNC repository (default is ~/.znc)");
 }
 
 static void die(int sig) {
@@ -108,7 +108,7 @@ int main(int argc, char** argv) {
 	bool bMakePass = false;
 	bool bAllowRoot = false;
 	bool bForeground = false;
-#ifdef _DEBUG
+#ifdef ALWAYS_RUN_IN_FOREGROUND
 	bForeground = true;
 #endif
 #ifdef HAVE_LIBSSL
@@ -160,7 +160,9 @@ int main(int argc, char** argv) {
 	}
 
 	if (optind < argc) {
-		sConfig = argv[optind];
+		CUtils::PrintError("Specifying a config file as an argument isn't supported anymore.");
+		CUtils::PrintError("Use --datadir instead.");
+		return 1;
 	}
 
 	CZNC* pZNC = &CZNC::Get();
@@ -187,7 +189,7 @@ int main(int argc, char** argv) {
 
 #ifndef RUN_FROM_SOURCE
 	if (CFile::Exists(pZNC->GetCurPath() + "/znc-uninstalled.pc")) {
-		CUtils::PrintError("It looks like you are running znc without installing it first.");
+		CUtils::PrintError("It looks like you are running ZNC without installing it first.");
 		CUtils::PrintError("Recompile with --enable-run-from-source if you intend to do that.");
 	}
 #endif
@@ -252,6 +254,16 @@ int main(int argc, char** argv) {
 			return 0;
 		}
 
+		/* fcntl() locks don't necessarily propagate to forked()
+		 *   children.  Reacquire the lock here.  Use the blocking
+		 *   call to avoid race condition with parent exiting.
+		 */
+		if (!pZNC->WaitForChildLock()) {
+			CUtils::PrintError("Child was unable to obtain lock on config file.");
+			delete pZNC;
+			return 1;
+		}
+
 		// Redirect std in/out/err to /dev/null
 		close(0); open("/dev/null", O_RDONLY);
 		close(1); open("/dev/null", O_WRONLY);
@@ -303,7 +315,6 @@ int main(int argc, char** argv) {
 					NULL,
 					NULL,
 					NULL,
-					NULL,
 					NULL
 				};
 				int pos = 3;
@@ -315,13 +326,12 @@ int main(int argc, char** argv) {
 					args[pos++] = strdup("--no-color");
 				if (bAllowRoot)
 					args[pos++] = strdup("--allow-root");
-				args[pos++] = strdup(pZNC->GetConfigFile().c_str());
-				// The above code adds 4 entries to args tops
+				// The above code adds 3 entries to args tops
 				// which means the array should be big enough
 
 				delete pZNC;
 				execvp(args[0], args);
-				CUtils::PrintError("Unable to restart znc [" + CString(strerror(errno)) + "]");
+				CUtils::PrintError("Unable to restart ZNC [" + CString(strerror(errno)) + "]");
 			} /* Fall through */
 			default:
 				iRet = 1;
