@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  * Copyright (C) 2008 by Stefan Rado
  * based on admin.cpp by Sebastian Ramacher
  * based on admin.cpp in crox branch
@@ -13,6 +13,7 @@
 #include "User.h"
 #include "Modules.h"
 #include "Chan.h"
+#include "IRCSock.h"
 
 template<std::size_t N>
 struct array_size_helper {
@@ -30,31 +31,7 @@ class CAdminMod : public CModule {
 	using CModule::PutModule;
 
 	void PrintHelp(const CString&) {
-		CTable CmdTable;
-		CmdTable.AddColumn("Command");
-		CmdTable.AddColumn("Arguments");
-		CmdTable.AddColumn("Description");
-		static const char* help[][3] = {
-			{"Get",          "variable [username]",           "Prints the variable's value for the given or current user"},
-			{"Set",          "variable username value",       "Sets the variable's value for the given user (use $me for the current user)"},
-			{"GetChan",      "variable [username] chan",      "Prints the variable's value for the given channel"},
-			{"SetChan",      "variable username chan value",  "Sets the variable's value for the given channel"},
-			{"ListUsers",    "",                              "Lists users"},
-			{"AddUser",      "username password [ircserver]", "Adds a new user"},
-			{"DelUser",      "username",                      "Deletes a user"},
-			{"CloneUser",    "oldusername newusername",       "Clones a user"},
-			{"AddServer",    "[username] server",             "Adds a new IRC server for the given or current user"},
-			{"LoadModule",   "username modulename",           "Loads a Module for a user"},
-			{"UnLoadModule", "username modulename",           "Removes a Module of a user"},
-			{"ListMods",     "username",                      "Get the list of modules for a user"}
-		};
-		for (unsigned int i = 0; i != ARRAY_SIZE(help); ++i) {
-			CmdTable.AddRow();
-			CmdTable.SetCell("Command",     help[i][0]);
-			CmdTable.SetCell("Arguments",   help[i][1]);
-			CmdTable.SetCell("Description", help[i][2]);
-		}
-		PutModule(CmdTable);
+		HandleHelpCommand();
 
 		PutModule("The following variables are available when using the Set/Get commands:");
 
@@ -70,12 +47,12 @@ class CAdminMod : public CModule {
 			{"Altnick",          string},
 			{"Ident",            string},
 			{"RealName",         string},
-			{"VHost",            string},
+			{"BindHost",         string},
 			{"MultiClients",     boolean},
 			{"BounceDCCs",       boolean},
 			{"UseClientIP",      boolean},
 			{"DenyLoadMod",      boolean},
-			{"DenySetVHost",     boolean},
+			{"DenySetBindHost",  boolean},
 			{"DefaultChanModes", string},
 			{"QuitMsg",          string},
 			{"BufferCount",      integer},
@@ -87,7 +64,9 @@ class CAdminMod : public CModule {
 			{"Admin",            boolean},
 			{"AppendTimestamp",  boolean},
 			{"PrependTimestamp", boolean},
-			{"DCCVHost",         boolean}
+			{"TimestampFormat",  string},
+			{"DCCBindHost",      boolean},
+			{"StatusPrefix",     string}
 		};
 		for (unsigned int i = 0; i != ARRAY_SIZE(vars); ++i) {
 			VarTable.AddRow();
@@ -160,8 +139,8 @@ class CAdminMod : public CModule {
 			PutModule("Ident = " + pUser->GetIdent());
 		else if (sVar == "realname")
 			PutModule("RealName = " + pUser->GetRealName());
-		else if (sVar == "vhost")
-			PutModule("VHost = " + pUser->GetVHost());
+		else if (sVar == "bindhost")
+			PutModule("BindHost = " + pUser->GetBindHost());
 		else if (sVar == "multiclients")
 			PutModule("MultiClients = " + CString(pUser->MultiClients()));
 		else if (sVar == "bouncedccs")
@@ -170,8 +149,8 @@ class CAdminMod : public CModule {
 			PutModule("UseClientIP = " + CString(pUser->UseClientIP()));
 		else if (sVar == "denyloadmod")
 			PutModule("DenyLoadMod = " + CString(pUser->DenyLoadMod()));
-		else if (sVar == "denysetvhost")
-			PutModule("DenySetVHost = " + CString(pUser->DenySetVHost()));
+		else if (sVar == "denysetbindhost")
+			PutModule("DenySetBindHost = " + CString(pUser->DenySetBindHost()));
 		else if (sVar == "defaultchanmodes")
 			PutModule("DefaultChanModes = " + pUser->GetDefaultChanModes());
 		else if (sVar == "quitmsg")
@@ -188,12 +167,16 @@ class CAdminMod : public CModule {
 			PutModule("TimezoneOffset = " + CString(pUser->GetTimezoneOffset()));
 		else if (sVar == "appendtimestamp")
 			PutModule("AppendTimestamp = " + CString(pUser->GetTimestampAppend()));
-		else if (sVar == "preprendtimestamp")
-			PutModule("PreprendTimestamp = " + CString(pUser->GetTimestampPrepend()));
-		else if (sVar == "dccvhost")
-			PutModule("DCCVHost = " + CString(pUser->GetDCCVHost()));
+		else if (sVar == "prependtimestamp")
+			PutModule("PrependTimestamp = " + CString(pUser->GetTimestampPrepend()));
+		else if (sVar == "timestampformat")
+			PutModule("TimestampFormat = " + pUser->GetTimestampFormat());
+		else if (sVar == "dccbindhost")
+			PutModule("DCCBindHost = " + CString(pUser->GetDCCBindHost()));
 		else if (sVar == "admin")
 			PutModule("Admin = " + CString(pUser->IsAdmin()));
+		else if (sVar == "statusprefix")
+			PutModule("StatuxPrefix = " + pUser->GetStatusPrefix());
 		else
 			PutModule("Error: Unknown variable");
 	}
@@ -228,10 +211,10 @@ class CAdminMod : public CModule {
 			pUser->SetRealName(sValue);
 			PutModule("RealName = " + sValue);
 		}
-		else if (sVar == "vhost") {
-			if(!pUser->DenySetVHost() || m_pUser->IsAdmin()) {
-				pUser->SetVHost(sValue);
-				PutModule("VHost = " + sValue);
+		else if (sVar == "bindhost") {
+			if(!pUser->DenySetBindHost() || m_pUser->IsAdmin()) {
+				pUser->SetBindHost(sValue);
+				PutModule("BindHost = " + sValue);
 			} else {
 				PutModule("Access denied!");
 			}
@@ -260,11 +243,11 @@ class CAdminMod : public CModule {
 				PutModule("Access denied!");
 			}
 		}
-		else if (sVar == "denysetvhost") {
+		else if (sVar == "denysetbindhost") {
 			if(m_pUser->IsAdmin()) {
 				bool b = sValue.ToBool();
-				pUser->SetDenySetVHost(b);
-				PutModule("DenySetVHost = " + CString(b));
+				pUser->SetDenySetBindHost(b);
+				PutModule("DenySetBindHost = " + CString(b));
 			} else {
 				PutModule("Access denied!");
 			}
@@ -279,8 +262,13 @@ class CAdminMod : public CModule {
 		}
 		else if (sVar == "buffercount") {
 			unsigned int i = sValue.ToUInt();
-			pUser->SetBufferCount(i);
-			PutModule("BufferCount = " + sValue);
+			// Admins don't have to honour the buffer limit
+			if (pUser->SetBufferCount(i, m_pUser->IsAdmin())) {
+				PutModule("BufferCount = " + sValue);
+			} else {
+				PutModule("Setting failed, limit is " +
+						CString(CZNC::Get().GetMaxBufferSize()));
+			}
 		}
 		else if (sVar == "keepbuffer") {
 			bool b = sValue.ToBool();
@@ -327,14 +315,26 @@ class CAdminMod : public CModule {
 			pUser->SetTimestampAppend(b);
 			PutModule("AppendTimestamp = " + CString(b));
 		}
-		else if (sVar == "dccvhost") {
-			if(!pUser->DenySetVHost() || m_pUser->IsAdmin()) {
-				pUser->SetDCCVHost(sValue);
-				PutModule("DCCVHost = " + sValue);
+		else if (sVar == "timestampformat") {
+			pUser->SetTimestampFormat(sValue);
+			PutModule("TimestampFormat = " + sValue);
+		}
+		else if (sVar == "dccbindhost") {
+			if(!pUser->DenySetBindHost() || m_pUser->IsAdmin()) {
+				pUser->SetDCCBindHost(sValue);
+				PutModule("DCCBindHost = " + sValue);
 			} else {
 				PutModule("Access denied!");
 			}
 		}
+		else if (sVar == "statusprefix") {
+			if (sVar.find_first_of(" \t\n") == CString::npos) {
+				pUser->SetStatusPrefix(sValue);
+				PutModule("StatusPrefix = " + sValue);
+			} else {
+				PutModule("That would be a bad idea!");
+			}
+		}
 		else
 			PutModule("Error: Unknown variable");
 	}
@@ -408,8 +408,13 @@ class CAdminMod : public CModule {
 			PutModule("DefModes = " + sValue);
 		} else if (sVar == "buffer") {
 			unsigned int i = sValue.ToUInt();
-			pChan->SetBufferCount(i);
-			PutModule("Buffer = " + CString(i));
+			// Admins don't have to honour the buffer limit
+			if (pChan->SetBufferCount(i, m_pUser->IsAdmin())) {
+				PutModule("Buffer = " + sValue);
+			} else {
+				PutModule("Setting failed, limit is " +
+						CString(CZNC::Get().GetMaxBufferSize()));
+			}
 		} else if (sVar == "inconfig") {
 			bool b = sValue.ToBool();
 			pChan->SetInConfig(b);
@@ -446,7 +451,7 @@ class CAdminMod : public CModule {
 		Table.AddColumn("Nick");
 		Table.AddColumn("AltNick");
 		Table.AddColumn("Ident");
-		Table.AddColumn("VHost");
+		Table.AddColumn("BindHost");
 
 		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); ++it) {
 			Table.AddRow();
@@ -459,7 +464,7 @@ class CAdminMod : public CModule {
 			Table.SetCell("Nick", it->second->GetNick());
 			Table.SetCell("AltNick", it->second->GetAltNick());
 			Table.SetCell("Ident", it->second->GetIdent());
-			Table.SetCell("VHost", it->second->GetVHost());
+			Table.SetCell("BindHost", it->second->GetBindHost());
 		}
 
 		PutModule(Table);
@@ -558,14 +563,13 @@ class CAdminMod : public CModule {
 			return;
 		}
 
-		CUser* pNewUser = new CUser(sOldUsername);
+		CUser* pNewUser = new CUser(sNewUsername);
 		CString sError;
 		if (!pNewUser->Clone(*pOldUser, sError)) {
 			delete pNewUser;
 			PutModule("Error: Cloning failed! [" + sError + "]");
 			return;
 		}
-		pNewUser->SetUserName(sNewUsername);
 		pNewUser->SetIRCConnectEnabled(false);
 
 		if (!CZNC::Get().AddUser(pNewUser, sError)) {
@@ -580,8 +584,12 @@ class CAdminMod : public CModule {
 
 	void AddServer(const CString& sLine) {
 		CString sUsername = sLine.Token(1);
-		const CString sServer = sLine.Token(2, true);
+		CString sServer = sLine.Token(2, true);
 
+		if (sServer.empty()) {
+			sServer = sUsername;
+			sUsername = m_pUser->GetUserName();
+		}
 		if (sServer.empty()) {
 			PutModule("Usage: addserver <username> <server>");
 			return;
@@ -591,8 +599,140 @@ class CAdminMod : public CModule {
 		if (!pUser)
 			return;
 
-		pUser->AddServer(sServer);
-		PutModule("Added IRC Server: " + sServer);
+		if (pUser->AddServer(sServer))
+			PutModule("Added IRC Server: " + sServer);
+		else
+			PutModule("Could not add IRC server");
+	}
+
+	void ReconnectUser(const CString& sLine) {
+		CString sUserName = sLine.Token(1, true);
+
+		if (sUserName.empty()) {
+			sUserName = m_pUser->GetUserName();
+		}
+		CUser* pUser = GetUser(sUserName);
+		if (!pUser) {
+			PutModule("User not found.");
+			return;
+		}
+
+		CIRCSock *pIRCSock = pUser->GetIRCSock();
+		// cancel connection attempt:
+		if (pIRCSock && !pIRCSock->IsConnected()) {
+			pIRCSock->Close();
+		}
+		// or close existing connection:
+		else if(pIRCSock) {
+			pIRCSock->Quit();
+		}
+
+		// then reconnect
+		pUser->SetIRCConnectEnabled(true);
+		pUser->CheckIRCConnect();
+
+		PutModule("Queued user for a reconnect.");
+	}
+
+	void DisconnectUser(const CString& sLine) {
+		CString sUserName = sLine.Token(1, true);
+
+		if (sUserName.empty()) {
+			sUserName = m_pUser->GetUserName();
+		}
+		CUser* pUser = GetUser(sUserName);
+		if (!pUser) {
+			PutModule("User not found.");
+			return;
+		}
+
+		CIRCSock *pIRCSock = pUser->GetIRCSock();
+		if (pIRCSock && !pIRCSock->IsConnected())
+			pIRCSock->Close();
+		else if(pIRCSock)
+			pIRCSock->Quit();
+
+		pUser->SetIRCConnectEnabled(false);
+
+		PutModule("Closed user's IRC connection.");
+	}
+
+	void ListCTCP(const CString& sLine) {
+		CString sUserName = sLine.Token(1, true);
+
+		if (sUserName.empty()) {
+			sUserName = m_pUser->GetUserName();
+		}
+		CUser* pUser = GetUser(sUserName);
+		if (!pUser)
+			return;
+
+		const MCString& msCTCPReplies = pUser->GetCTCPReplies();
+		CTable Table;
+		Table.AddColumn("Request");
+		Table.AddColumn("Reply");
+		for (MCString::const_iterator it = msCTCPReplies.begin(); it != msCTCPReplies.end(); it++) {
+			Table.AddRow();
+			Table.SetCell("Request", it->first);
+			Table.SetCell("Reply", it->second);
+		}
+
+		if (Table.empty()) {
+			PutModule("No CTCP replies for user [" + pUser->GetUserName() + "] configured!");
+		} else {
+			PutModule("CTCP replies for user [" + pUser->GetUserName() + "]:");
+			PutModule(Table);
+		}
+	}
+
+	void AddCTCP(const CString& sLine) {
+		CString sUserName    = sLine.Token(1);
+		CString sCTCPRequest = sLine.Token(2);
+		CString sCTCPReply   = sLine.Token(3, true);
+
+		if (sCTCPRequest.empty()) {
+			sCTCPRequest = sUserName;
+			sCTCPReply = sLine.Token(2, true);
+			sUserName = m_pUser->GetUserName();
+		}
+		if (sCTCPRequest.empty()) {
+			PutModule("Usage: AddCTCP [user] [request] [reply]");
+			PutModule("This will cause ZNC to reply to the CTCP instead of forwarding it to clients.");
+			PutModule("An empty reply will cause the CTCP request to be blocked.");
+			return;
+		}
+
+		CUser* pUser = GetUser(sUserName);
+		if (!pUser)
+			return;
+
+		if (pUser->AddCTCPReply(sCTCPRequest, sCTCPReply))
+			PutModule("Added!");
+		else
+			PutModule("Error!");
+	}
+
+	void DelCTCP(const CString& sLine) {
+		CString sUserName    = sLine.Token(1);
+		CString sCTCPRequest = sLine.Token(2, true);
+
+		if (sCTCPRequest.empty()) {
+			sCTCPRequest = sUserName;
+			sUserName = m_pUser->GetUserName();
+		}
+		CUser* pUser = GetUser(sUserName);
+		if (!pUser)
+			return;
+
+		if (sCTCPRequest.empty()) {
+			PutModule("Usage: DelCTCP [user] [request]");
+			return;
+		}
+
+		if (pUser->DelCTCPReply(sCTCPRequest))
+			PutModule("Successfully removed [" + sCTCPRequest + "]");
+		else
+			PutModule("Error: [" + sCTCPRequest + "] not found!");
 	}
 
 	void LoadModuleForUser(const CString& sLine) {
@@ -640,7 +780,7 @@ class CAdminMod : public CModule {
 		CString sModRet;
 
 		if (sModName.empty()) {
-			PutModule("Usage: loadmodule <username> <modulename>");
+			PutModule("Usage: unloadmodule <username> <modulename>");
 			return;
 		}
 
@@ -690,40 +830,47 @@ class CAdminMod : public CModule {
 
 	}
 
-	typedef void (CAdminMod::* fn)(const CString&);
-	typedef std::map<CString, fn> function_map;
-	function_map fnmap_;
-
 public:
 	MODCONSTRUCTOR(CAdminMod) {
-		fnmap_["help"]         = &CAdminMod::PrintHelp;
-		fnmap_["get"]          = &CAdminMod::Get;
-		fnmap_["set"]          = &CAdminMod::Set;
-		fnmap_["getchan"]      = &CAdminMod::GetChan;
-		fnmap_["setchan"]      = &CAdminMod::SetChan;
-		fnmap_["listusers"]    = &CAdminMod::ListUsers;
-		fnmap_["adduser"]      = &CAdminMod::AddUser;
-		fnmap_["deluser"]      = &CAdminMod::DelUser;
-		fnmap_["cloneuser"]    = &CAdminMod::CloneUser;
-		fnmap_["addserver"]    = &CAdminMod::AddServer;
-		fnmap_["loadmodule"]   = &CAdminMod::LoadModuleForUser;
-		fnmap_["unloadmodule"] = &CAdminMod::UnLoadModuleForUser;
-		fnmap_["listmods"]     = &CAdminMod::ListModuleForUser;
+		AddCommand("Help",         static_cast<CModCommand::ModCmdFunc>(&CAdminMod::PrintHelp),
+			"",                              "Generates this output");
+		AddCommand("Get",          static_cast<CModCommand::ModCmdFunc>(&CAdminMod::Get),
+			"variable [username]",           "Prints the variable's value for the given or current user");
+		AddCommand("Set",          static_cast<CModCommand::ModCmdFunc>(&CAdminMod::Set),
+			"variable username value",       "Sets the variable's value for the given user (use $me for the current user)");
+		AddCommand("GetChan",      static_cast<CModCommand::ModCmdFunc>(&CAdminMod::GetChan),
+			"variable [username] chan",      "Prints the variable's value for the given channel");
+		AddCommand("SetChan",      static_cast<CModCommand::ModCmdFunc>(&CAdminMod::SetChan),
+			"variable username chan value",  "Sets the variable's value for the given channel");
+		AddCommand("ListUsers",    static_cast<CModCommand::ModCmdFunc>(&CAdminMod::ListUsers),
+			"",                              "Lists users");
+		AddCommand("AddUser",      static_cast<CModCommand::ModCmdFunc>(&CAdminMod::AddUser),
+			"username password [ircserver]", "Adds a new user");
+		AddCommand("DelUser",      static_cast<CModCommand::ModCmdFunc>(&CAdminMod::DelUser),
+			"username",                      "Deletes a user");
+		AddCommand("CloneUser",    static_cast<CModCommand::ModCmdFunc>(&CAdminMod::CloneUser),
+			"oldusername newusername",       "Clones a user");
+		AddCommand("AddServer",    static_cast<CModCommand::ModCmdFunc>(&CAdminMod::AddServer),
+			"[username] server",             "Adds a new IRC server for the given or current user");
+		AddCommand("Reconnect",    static_cast<CModCommand::ModCmdFunc>(&CAdminMod::ReconnectUser),
+			"username",                      "Cycles the user's IRC server connection");
+		AddCommand("Disconnect",   static_cast<CModCommand::ModCmdFunc>(&CAdminMod::DisconnectUser),
+			"username",                      "Disconnects the user from their IRC server");
+		AddCommand("LoadModule",   static_cast<CModCommand::ModCmdFunc>(&CAdminMod::LoadModuleForUser),
+			"username modulename",           "Loads a Module for a user");
+		AddCommand("UnLoadModule", static_cast<CModCommand::ModCmdFunc>(&CAdminMod::UnLoadModuleForUser),
+			"username modulename",           "Removes a Module of a user");
+		AddCommand("ListMods",     static_cast<CModCommand::ModCmdFunc>(&CAdminMod::ListModuleForUser),
+			"username",                      "Get the list of modules for a user");
+		AddCommand("ListCTCPs",    static_cast<CModCommand::ModCmdFunc>(&CAdminMod::ListCTCP),
+			"username",                      "List the configured CTCP replies");
+		AddCommand("AddCTCP",      static_cast<CModCommand::ModCmdFunc>(&CAdminMod::AddCTCP),
+			"username ctcp [reply]",         "Configure a new CTCP reply");
+		AddCommand("DelCTCP",      static_cast<CModCommand::ModCmdFunc>(&CAdminMod::DelCTCP),
+			"username ctcp",                 "Remove a CTCP reply");
 	}
 
 	virtual ~CAdminMod() {}
-
-	virtual void OnModCommand(const CString& sLine) {
-		if (!m_pUser)
-			return;
-
-		const CString cmd = sLine.Token(0).AsLower();
-		function_map::iterator it = fnmap_.find(cmd);
-		if (it != fnmap_.end())
-			(this->*it->second)(sLine);
-		else
-			PutModule("Unknown command");
-	}
 };
 
-MODULEDEFS(CAdminMod, "Dynamic configuration of users/settings through irc")
+MODULEDEFS(CAdminMod, "Dynamic configuration of users/settings through IRC")
