@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -38,6 +38,7 @@ public:
 		Ping();
 		m_bIsAway = false;
 		m_bBootError = false;
+		m_saveMessages = true;
 		SetAwayTime(300);
 		AddTimer(new CAwayJob(this, 60, 0, "AwayJob", "Checks for idle and saves messages every 1 minute"));
 	}
@@ -50,29 +51,38 @@ public:
 	virtual bool OnLoad(const CString& sArgs, CString& sMessage)
 	{
 		CString sMyArgs = sArgs;
-		if (sMyArgs.Token(0) == "-notimer")
+		size_t uIndex = 0;
+		if (sMyArgs.Token(0) == "-nostore")
+		{
+			uIndex++;
+			m_saveMessages = false;
+		}
+		if (sMyArgs.Token(uIndex) == "-notimer")
 		{
 			SetAwayTime(0);
-			sMyArgs = sMyArgs.Token(1, true);
-		} else if (sMyArgs.Token(0) == "-timer")
+			sMyArgs = sMyArgs.Token(uIndex + 1, true);
+		} else if (sMyArgs.Token(uIndex) == "-timer")
 		{
-			SetAwayTime(sMyArgs.Token(1).ToInt());
-			sMyArgs = sMyArgs.Token(2, true);
+			SetAwayTime(sMyArgs.Token(uIndex + 1).ToInt());
+			sMyArgs = sMyArgs.Token(uIndex + 2, true);
 		}
-		if (!sMyArgs.empty())
+		if (m_saveMessages)
 		{
-			m_sPassword = CBlowfish::MD5(sMyArgs);
-		} else {
-			sMessage = "This module needs as an argument a keyphrase used for encryption";
-			return false;
-		}
+			if (!sMyArgs.empty())
+			{
+				m_sPassword = CBlowfish::MD5(sMyArgs);
+			} else {
+				sMessage = "This module needs as an argument a keyphrase used for encryption";
+				return false;
+			}
 
-		if (!BootStrap())
-		{
-			sMessage = "Failed to decrypt your saved messages - "
-				"Did you give the right encryption key as an argument to this module?";
-			m_bBootError = true;
-			return false;
+			if (!BootStrap())
+			{
+				sMessage = "Failed to decrypt your saved messages - "
+					"Did you give the right encryption key as an argument to this module?";
+				m_bBootError = true;
+				return false;
+			}
 		}
 
 		return true;
@@ -152,7 +162,7 @@ public:
 			if (sCommand.Token(1) != "-quiet")
 			{
 				sReason = sCommand.Token(1, true);
-				PutModNotice("You have been marked as away", "away");
+				PutModNotice("You have been marked as away");
 			}
 			else
 				sReason = sCommand.Token(2, true);
@@ -161,48 +171,48 @@ public:
 		else if (sCmdName == "back")
 		{
 			if ((m_vMessages.empty()) && (sCommand.Token(1) != "-quiet"))
-				PutModNotice("Welcome Back!", "away");
+				PutModNotice("Welcome Back!");
 			Back();
 		}
 		else if (sCmdName == "messages")
 		{
 			for (u_int a = 0; a < m_vMessages.size(); a++)
-				PutModule(m_vMessages[a], "away");
+				PutModule(m_vMessages[a]);
 		}
 		else if (sCmdName == "delete")
 		{
 			CString sWhich = sCommand.Token(1);
 			if (sWhich == "all")
 			{
-				PutModNotice("Deleted " + CString(m_vMessages.size()) + " Messages.", "away");
+				PutModNotice("Deleted " + CString(m_vMessages.size()) + " Messages.");
 				for (u_int a = 0; a < m_vMessages.size(); a++)
 					m_vMessages.erase(m_vMessages.begin() + a--);
 
 			}
 			else if (sWhich.empty())
 			{
-				PutModNotice("USAGE: delete <num|all>", "away");
+				PutModNotice("USAGE: delete <num|all>");
 				return;
 			} else
 			{
 				u_int iNum = sWhich.ToUInt();
 				if (iNum >= m_vMessages.size())
 				{
-					PutModNotice("Illegal Message # Requested", "away");
+					PutModNotice("Illegal Message # Requested");
 					return;
 				}
 				else
 				{
 					m_vMessages.erase(m_vMessages.begin() + iNum);
-					PutModNotice("Message Erased.", "away");
+					PutModNotice("Message Erased.");
 				}
 				SaveBufferToDisk();
 			}
 		}
-		else if (sCmdName == "save")
+		else if (sCmdName == "save" && m_saveMessages)
 		{
 			SaveBufferToDisk();
-			PutModNotice("Messages saved to disk.", "away");
+			PutModNotice("Messages saved to disk.");
 		}
 		else if (sCmdName == "ping")
 		{
@@ -227,7 +237,7 @@ public:
 				if ((sTime.empty()) || (sWhom.empty()) || (sMessage.empty()))
 				{
 					// illegal format
-					PutModule("Corrupt message! [" + m_vMessages[a] + "]", "away");
+					PutModule("Corrupt message! [" + m_vMessages[a] + "]");
 					m_vMessages.erase(m_vMessages.begin() + a--);
 					continue;
 				}
@@ -238,7 +248,7 @@ public:
 				size_t iCount = strftime(szFormat, 64, "%F %T", &t);
 				if (iCount <= 0)
 				{
-					PutModule("Corrupt time stamp! [" + m_vMessages[a] + "]", "away");
+					PutModule("Corrupt time stamp! [" + m_vMessages[a] + "]");
 					m_vMessages.erase(m_vMessages.begin() + a--);
 					continue;
 				}
@@ -250,11 +260,11 @@ public:
 			}
 			for (map< CString, vector< CString> >::iterator it = msvOutput.begin(); it != msvOutput.end(); ++it)
 			{
-				PutModule(it->first, "away");
+				PutModule(it->first);
 				for (u_int a = 0; a < it->second.size(); a++)
 					PutModule(it->second[a]);
 			}
-			PutModule("#--- End Messages", "away");
+			PutModule("#--- End Messages");
 		} else if (sCmdName == "enabletimer")
 		{
 			SetAwayTime(300);
@@ -279,7 +289,7 @@ public:
 			PutModule("Current timer setting: " + CString(GetAwayTime()) + " seconds");
 		} else
 		{
-			PutModule("Commands: away [-quiet], back [-quiet], delete <num|all>, ping, show, save, enabletimer, disabletimer, settimer <secs>, timer", "away");
+			PutModule("Commands: away [-quiet], back [-quiet], delete <num|all>, ping, show, save, enabletimer, disabletimer, settimer <secs>, timer");
 		}
 	}
 
@@ -323,13 +333,13 @@ public:
 		{
 			if (bUsePrivMessage)
 			{
-				PutModule("Welcome Back!", "away");
-				PutModule("You have " + CString(m_vMessages.size()) + " messages!", "away");
+				PutModule("Welcome Back!");
+				PutModule("You have " + CString(m_vMessages.size()) + " messages!");
 			}
 			else
 			{
-				PutModNotice("Welcome Back!", "away");
-				PutModNotice("You have " + CString(m_vMessages.size()) + " messages!", "away");
+				PutModNotice("Welcome Back!");
+				PutModNotice("You have " + CString(m_vMessages.size()) + " messages!");
 			}
 		}
 		m_sReason = "";
@@ -390,7 +400,7 @@ private:
 			CBlowfish c(m_sPassword, BF_DECRYPT);
 			sBuffer = c.Crypt(sFile);
 
-			if (sBuffer.substr(0, strlen(CRYPT_VERIFICATION_TOKEN)) != CRYPT_VERIFICATION_TOKEN)
+			if (sBuffer.Left(strlen(CRYPT_VERIFICATION_TOKEN)) != CRYPT_VERIFICATION_TOKEN)
 			{
 				// failed to decode :(
 				PutModule("Unable to decode Encrypted messages");
@@ -410,7 +420,10 @@ private:
 
 	void AddMessage(const CString & sText)
 	{
-		m_vMessages.push_back(sText);
+		if (m_saveMessages)
+		{
+			m_vMessages.push_back(sText);
+		}
 	}
 
 	time_t          m_iLastSentData;
@@ -418,6 +431,7 @@ private:
 	time_t          m_iAutoAway;
 	vector<CString> m_vMessages;
 	CString         m_sReason;
+	bool            m_saveMessages;
 };
 
 
