@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -96,6 +96,39 @@ public:
 		}
 	}
 
+	bool OnEmbeddedWebRequest(CWebSock& WebSock, const CString& sPageName, CTemplate& Tmpl) {
+		if (sPageName == "webadmin/user" && WebSock.GetSession()->IsAdmin()) {
+			CString sAction = Tmpl["WebadminAction"];
+			if (sAction == "display") {
+				Tmpl["Blocked"] = CString(IsBlocked(Tmpl["Username"]));
+				Tmpl["Self"] = CString(Tmpl["Username"].Equals(WebSock.GetSession()->GetUser()->GetUserName()));
+				return true;
+			}
+			if (sAction == "change" && WebSock.GetParam("embed_blockuser_presented").ToBool()) {
+				if (Tmpl["Username"].Equals(WebSock.GetSession()->GetUser()->GetUserName()) &&
+						WebSock.GetParam("embed_blockuser_block").ToBool()) {
+					WebSock.GetSession()->AddError("You can't block yourself");
+				} else if (WebSock.GetParam("embed_blockuser_block").ToBool()) {
+					if (!WebSock.GetParam("embed_blockuser_old").ToBool()) {
+						if (Block(Tmpl["Username"])) {
+							WebSock.GetSession()->AddSuccess("Blocked [" + Tmpl["Username"] + "]");
+						} else {
+							WebSock.GetSession()->AddError("Couldn't block [" + Tmpl["Username"] + "]");
+						}
+					}
+				} else  if (WebSock.GetParam("embed_blockuser_old").ToBool()){
+					if (DelNV(Tmpl["Username"])) {
+						WebSock.GetSession()->AddSuccess("Unblocked [" + Tmpl["Username"] + "]");
+					} else {
+						WebSock.GetSession()->AddError("User [" + Tmpl["Username"] + "is not blocked");
+					}
+				}
+				return true;
+			}
+		}
+		return false;
+	}
+
 private:
 	bool IsBlocked(const CString& sUser) {
 		MCString::iterator it;
