@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -47,8 +47,8 @@ public:
 		AddBuffer(Channel, Nick.GetNickMask() + " joined");
 	}
 
-	virtual void OnPart(const CNick& Nick, CChan& Channel) {
-		AddBuffer(Channel, Nick.GetNickMask() + " parted");
+	virtual void OnPart(const CNick& Nick, CChan& Channel, const CString& sMessage) {
+		AddBuffer(Channel, Nick.GetNickMask() + " parted with message: [" + sMessage + "]");
 	}
 
 	virtual void OnNick(const CNick& OldNick, const CString& sNewNick, const vector<CChan*>& vChans) {
