@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -22,6 +22,7 @@ static const struct {
 } vRouteReplies[] = {
 	{"WHO", {
 		{"352", false},
+		{"354", false}, // e.g. Quaknet uses this for WHO #chan %n
 		{"403", true}, // No such chan
 		{"315", true},
 		{NULL, true}
@@ -112,6 +113,21 @@ static const struct {
 		{"266", true},
 		{NULL, true},
 	}},
+	// This is just a list of all possible /mode replies stuffed together.
+	// Since there should never be more than one of these going on, this
+	// should work fine and makes the code simpler.
+	{"MODE", {
+		// MODE I
+		{"346", false},
+		{"347", true},
+		// MODE b
+		{"367", false},
+		{"368", true},
+		// MODE e
+		{"348", false},
+		{"349", true},
+		{NULL, true},
+	 }},
 	// END (last item!)
 	{NULL, {{NULL, true}}}
 };
@@ -180,16 +196,17 @@ public:
 		if (m_pClient == m_pDoing) {
 			// The replies which aren't received yet will be
 			// broadcasted to everyone, but at least nothing breaks
+			RemTimer("RouteTimeout");
 			m_pDoing = NULL;
 			m_pReplies = NULL;
 		}
 
 		it = m_vsPending.find(m_pClient);
 
-		if (it == m_vsPending.end())
-			return;
+		if (it != m_vsPending.end())
+			m_vsPending.erase(it);
 
-		m_vsPending.erase(it);
+		SendRequest();
 	}
 
 	virtual EModRet OnRaw(CString& sLine)
@@ -231,14 +248,47 @@ public:
 	virtual EModRet OnUserRaw(CString& sLine)
 	{
 		CString sCmd = sLine.Token(0).AsUpper();
-		size_t i = 0;
 
 		if (!m_pUser->GetIRCSock())
 			return CONTINUE;
 
-		while (vRouteReplies[i].szRequest != NULL) {
-			if (vRouteReplies[i].szRequest == sCmd) {
+		if (sCmd.Equals("MODE")) {
+			// Check if this is a mode request that needs to be handled
 
+			// If there are arguments to a mode change,
+			// we must not route it.
+			if (!sLine.Token(3, true).empty())
+				return CONTINUE;
+
+			// Grab the mode change parameter
+			CString sMode = sLine.Token(2);
+
+			// If this is a channel mode request, znc core replies to it
+			if (sMode.empty())
+				return CONTINUE;
+
+			// Check if this is a mode change or a specific
+			// mode request (the later needs to be routed).
+			sMode.TrimPrefix("+");
+			if (sMode.length() != 1)
+				return CONTINUE;
+
+			// Now just check if it's one of the supported modes
+			switch (sMode[0]) {
+			case 'I':
+			case 'b':
+			case 'e':
+				break;
+			default:
+				return CONTINUE;
+			}
+
+			// Ok, this looks like we should route it.
+			// Fall through to the next loop
+		}
+
+		for (size_t i = 0; vRouteReplies[i].szRequest != NULL; i++) {
+			if (vRouteReplies[i].szRequest == sCmd) {
 				struct queued_req req = {
 					sLine, vRouteReplies[i].vReplies
 				};
@@ -247,7 +297,6 @@ public:
 
 				return HALTCORE;
 			}
-			i++;
 		}
 
 		return CONTINUE;
