@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2011  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -26,6 +26,11 @@ namespace
 	};
 }
 
+static inline CString FormatBindError() {
+	CString sError = (errno == 0 ? CString("unknown error, check the host name") : CString(strerror(errno)));
+	return "Unable to bind [" + sError + "]";
+}
+
 CZNC::CZNC() {
 	if (!InitCsocket()) {
 		CUtils::PrintError("Could not initialize Csocket!");
@@ -39,6 +44,7 @@ CZNC::CZNC() {
 	SetISpoofFormat(""); // Set ISpoofFormat to default
 	m_uBytesRead = 0;
 	m_uBytesWritten = 0;
+	m_uiMaxBufferSize = 500;
 	m_pConnectUserTimer = NULL;
 	m_eConfigState = ECONFIG_NOTHING;
 	m_TimeStarted = time(NULL);
@@ -86,11 +92,11 @@ CString CZNC::GetVersion() {
 
 CString CZNC::GetTag(bool bIncludeVersion) {
 	if (!bIncludeVersion) {
-		return "ZNC - http://znc.sourceforge.net";
+		return "ZNC - http://znc.in";
 	}
 
 	char szBuf[128];
-	snprintf(szBuf, sizeof(szBuf), "ZNC %1.3f"VERSION_EXTRA" - http://znc.sourceforge.net", VERSION);
+	snprintf(szBuf, sizeof(szBuf), "ZNC %1.3f"VERSION_EXTRA" - http://znc.in", VERSION);
 	// If snprintf overflows (which I doubt), we want to be on the safe side
 	szBuf[sizeof(szBuf) - 1] = '\0';
 
@@ -103,24 +109,14 @@ CString CZNC::GetUptime() const {
 }
 
 bool CZNC::OnBoot() {
-	if (!GetModules().OnBoot()) {
-		return false;
-	}
-
-	for (map<CString,CUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
-		if (!it->second->GetModules().OnBoot()) {
-			return false;
-		}
-	}
+	ALLMODULECALL(OnBoot(), return false);
 
 	return true;
 }
 
 bool CZNC::ConnectUser(CUser *pUser) {
 	CString sSockName = "IRC::" + pUser->GetUserName();
-	// Don't use pUser->GetIRCSock(), as that only returns something if the
-	// CIRCSock is already connected, not when it's still connecting!
-	CIRCSock* pIRCSock = (CIRCSock*) m_Manager.FindSockByName(sSockName);
+	CIRCSock* pIRCSock = pUser->GetIRCSock();
 
 	if (m_pISpoofLockFile != NULL) {
 		return false;
@@ -132,9 +128,6 @@ bool CZNC::ConnectUser(CUser *pUser) {
 	if (pIRCSock || !pUser->HasServers())
 		return false;
 
-	if (pUser->ConnectPaused())
-		return false;
-
 	CServer* pServer = pUser->GetNextServer();
 
 	if (!pServer)
@@ -144,15 +137,15 @@ bool CZNC::ConnectUser(CUser *pUser) {
 		return false;
 
 	if (!WriteISpoof(pUser)) {
-		DEBUG("ISpoof could not be written");
-		pUser->PutStatus("ISpoof could not be written, retrying...");
+		DEBUG("ISpoof [" + m_sISpoofFile + "] could not be written");
+		pUser->PutStatus("ISpoof [" + m_sISpoofFile + "] could not be written, retrying...");
 		return true;
 	}
 
 	m_sConnectThrottle.AddItem(pServer->GetName());
 
-	DEBUG("User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetName() << " " << pServer->GetPort() << "] ...");
-	pUser->PutStatus("Attempting to connect to [" + pServer->GetName() + " " + CString(pServer->GetPort()) + "] ...");
+	DEBUG("User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetString(false) << "] ...");
+	pUser->PutStatus("Attempting to connect to [" + pServer->GetString(false) + "] ...");
 
 	pIRCSock = new CIRCSock(pUser);
 	pIRCSock->SetPass(pServer->GetPass());
@@ -172,7 +165,7 @@ bool CZNC::ConnectUser(CUser *pUser) {
 		return false;
 	);
 
-	if (!m_Manager.Connect(pServer->GetName(), pServer->GetPort(), sSockName, 120, bSSL, pUser->GetVHost(), pIRCSock)) {
+	if (!m_Manager.Connect(pServer->GetName(), pServer->GetPort(), sSockName, 120, bSSL, pUser->GetBindHost(), pIRCSock)) {
 		ReleaseISpoof();
 		pUser->PutStatus("Unable to connect. (Bad host?)");
 	}
@@ -199,9 +192,7 @@ bool CZNC::HandleUserDeletion()
 		}
 		m_msUsers.erase(pUser->GetUserName());
 
-		// Don't use pUser->GetIRCSock(), as that only returns something if the
-		// CIRCSock is already connected, not when it's still connecting!
-		CIRCSock* pIRCSock = (CIRCSock*) m_Manager.FindSockByName("IRC::" + pUser->GetUserName());
+		CIRCSock* pIRCSock = pUser->GetIRCSock();
 
 		if (pIRCSock) {
 			m_Manager.DelSockByAddr(pIRCSock);
@@ -239,7 +230,7 @@ void CZNC::Loop() {
 			SetConfigState(ECONFIG_NOTHING);
 
 			if (WriteConfig()) {
-				Broadcast("Writing the config suceeded", true);
+				Broadcast("Writing the config succeeded", true);
 			} else {
 				Broadcast("Writing the config file failed", true);
 			}
@@ -264,9 +255,10 @@ bool CZNC::WriteISpoof(CUser* pUser) {
 	if (m_pISpoofLockFile != NULL)
 		return false;
 
-	if (!m_sISpoofFile.empty()) {
+	if (!GetISpoofFile().empty()) {
 		m_pISpoofLockFile = new CFile;
-		if (!m_pISpoofLockFile->TryExLock(m_sISpoofFile, O_RDWR | O_CREAT)) {
+		if (!m_pISpoofLockFile->TryExLock(GetISpoofFile(), O_RDWR | O_CREAT)) {
+			DEBUG("Couldn't open and lock ISpoofFile: " << strerror(errno));
 			delete m_pISpoofLockFile;
 			m_pISpoofLockFile = NULL;
 			return false;
@@ -278,6 +270,7 @@ bool CZNC::WriteISpoof(CUser* pUser) {
 		m_sOrigISpoof = buf;
 
 		if (!m_pISpoofLockFile->Seek(0) || !m_pISpoofLockFile->Truncate()) {
+			DEBUG("Couldn't truncate the ISpoofFile: " << strerror(errno));
 			delete m_pISpoofLockFile;
 			m_pISpoofLockFile = NULL;
 			return false;
@@ -290,6 +283,7 @@ bool CZNC::WriteISpoof(CUser* pUser) {
 		if (sData == m_sISpoofFormat) {
 			sData.Replace("%", pUser->GetIdent());
 		}
+		DEBUG("Writing [" + sData + "] to ISpoofFile [" + m_pISpoofLockFile->GetLongName() + "]");
 		m_pISpoofLockFile->Write(sData + "\n");
 	}
 	return true;
@@ -299,14 +293,15 @@ void CZNC::ReleaseISpoof() {
 	if (m_pISpoofLockFile == NULL)
 		return;
 
-	if (!m_sISpoofFile.empty()) {
-		if (m_pISpoofLockFile->Seek(0) && m_pISpoofLockFile->Truncate()) {
-			m_pISpoofLockFile->Write(m_sOrigISpoof);
-		}
-
-		m_sOrigISpoof = "";
+	if (m_pISpoofLockFile->Seek(0) && m_pISpoofLockFile->Truncate()) {
+		DEBUG("Writing [" + m_sOrigISpoof + "] to ISpoofFile [" + m_pISpoofLockFile->GetLongName() + "]");
+		m_pISpoofLockFile->Write(m_sOrigISpoof);
+	} else {
+		DEBUG("Error while restoring ISpoof: " << strerror(errno));
 	}
 
+	m_sOrigISpoof = "";
+
 	delete m_pISpoofLockFile;
 	m_pISpoofLockFile = NULL;
 }
@@ -408,16 +403,6 @@ void CZNC::DeleteUsers() {
 	DisableConnectUser();
 }
 
-CUser* CZNC::GetUser(const CString& sUser) {
-	// Todo: make this case insensitive
-	map<CString,CUser*>::iterator it = m_msUsers.find(sUser);
-	return (it == m_msUsers.end()) ? NULL : it->second;
-}
-
-Csock* CZNC::FindSockByName(const CString& sSockName) {
-	return m_Manager.FindSockByName(sSockName);
-}
-
 bool CZNC::IsHostAllowed(const CString& sHostMask) const {
 	for (map<CString,CUser*>::const_iterator a = m_msUsers.begin(); a != m_msUsers.end(); ++a) {
 		if (a->second->IsHostAllowed(sHostMask)) {
@@ -471,6 +456,8 @@ void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
 	} else {
 		m_sZNCPath = sDataDir;
 	}
+
+	m_sSSLCertFile = m_sZNCPath + "/znc.pem";
 }
 
 CString CZNC::GetConfPath(bool bAllowMkDir) const {
@@ -542,11 +529,13 @@ bool CZNC::WriteConfig() {
 		return false;
 	}
 
-	if (GetModules().OnWriteConfig(m_LockFile)) {
-		return false;
-	}
+	m_LockFile.Write(MakeConfigHeader() + "\n");
+
+	GLOBALMODULECALL(OnWriteConfig(m_LockFile), NULL, NULL, return false);
 
 	m_LockFile.Write("AnonIPLimit  = " + CString(m_uiAnonIPLimit) + "\n");
+	m_LockFile.Write("MaxBufferSize= " + CString(m_uiMaxBufferSize) + "\n");
+	m_LockFile.Write("SSLCertFile  = " + CString(m_sSSLCertFile) + "\n");
 
 	for (size_t l = 0; l < m_vpListeners.size(); l++) {
 		CListener* pListener = m_vpListeners[l];
@@ -605,8 +594,8 @@ bool CZNC::WriteConfig() {
 		m_LockFile.Write("Motd         = " + m_vsMotd[m].FirstLine() + "\n");
 	}
 
-	for (unsigned int v = 0; v < m_vsVHosts.size(); v++) {
-		m_LockFile.Write("VHost        = " + m_vsVHosts[v].FirstLine() + "\n");
+	for (unsigned int v = 0; v < m_vsBindHosts.size(); v++) {
+		m_LockFile.Write("BindHost     = " + m_vsBindHosts[v].FirstLine() + "\n");
 	}
 
 	CGlobalModules& Mods = GetModules();
@@ -650,46 +639,77 @@ bool CZNC::WriteConfig() {
 	return true;
 }
 
+CString CZNC::MakeConfigHeader() {
+	return
+		"// WARNING\n"
+		"//\n"
+		"// Do NOT edit this file while ZNC is running!\n"
+		"// Use webadmin or *admin instead.\n"
+		"//\n"
+		"// Buf if you feel risky, you might want to read help on /znc saveconfig and /znc rehash.\n"
+		"// Also check http://en.znc.in/wiki/Configuration\n";
+}
+
 bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 	CString sAnswer, sUser;
 	VCString vsLines;
+	
+	vsLines.push_back(MakeConfigHeader());
 
 	m_sConfigFile = ExpandConfigPath(sConfigFile);
 	CUtils::PrintMessage("Building new config");
 
 	CUtils::PrintMessage("");
-	CUtils::PrintMessage("First lets start with some global settings...");
+	CUtils::PrintMessage("First let's start with some global settings...");
 	CUtils::PrintMessage("");
 
 	// Listen
-	unsigned int uListenPort = 0;
-	while (!CUtils::GetNumInput("What port would you like ZNC to listen on?", uListenPort, 1, 65535)) ;
-
-	CString sSSL;
-#ifdef HAVE_LIBSSL
-	if (CUtils::GetBoolInput("Would you like ZNC to listen using SSL?", false)) {
-		sSSL = "+";
-
-		CString sPemFile = GetPemLocation();
-		if (!CFile::Exists(sPemFile)) {
-			CUtils::PrintError("Unable to locate pem file: [" + sPemFile + "]");
-			if (CUtils::GetBoolInput("Would you like to create a new pem file now?",
-						true)) {
-				WritePemFile();
-			}
-		}
-	}
-#endif
-
-	CString s6 = "4";
-#ifdef HAVE_IPV6
-	if (CUtils::GetBoolInput("Would you like ZNC to listen using ipv6?", false)) {
-		s6 = " ";
-	}
-#endif
-
+	CString s6;
 	CString sListenHost;
-	CUtils::GetInput("Listen Host", sListenHost, "", "Blank for all ips");
+	CString sSSL;
+	unsigned int uListenPort = 0;
+	bool bSuccess;
+
+	do {
+		bSuccess = true;
+		while (!CUtils::GetNumInput("What port would you like ZNC to listen on?", uListenPort, 1, 65535)) ;
+
+#ifdef HAVE_LIBSSL
+		if (CUtils::GetBoolInput("Would you like ZNC to listen using SSL?", !sSSL.empty())) {
+			sSSL = "+";
+
+			CString sPemFile = GetPemLocation();
+			if (!CFile::Exists(sPemFile)) {
+				CUtils::PrintError("Unable to locate pem file: [" + sPemFile + "]");
+				if (CUtils::GetBoolInput("Would you like to create a new pem file now?",
+							true)) {
+					WritePemFile();
+				}
+			}
+		} else
+			sSSL = "";
+#endif
+
+#ifdef HAVE_IPV6
+		if (CUtils::GetBoolInput("Would you like ZNC to listen using ipv6?", s6 == " ")) {
+			s6 = " ";
+		} else {
+			s6 = "4";
+		}
+#endif
+
+		CUtils::GetInput("Listen Host", sListenHost, sListenHost, "Blank for all ips");
+
+		CUtils::PrintAction("Verifying the listener");
+		CListener* pListener = new CListener(uListenPort, sListenHost, !sSSL.empty(),
+				s6.empty() ? ADDR_IPV4ONLY : ADDR_ALL, CListener::ACCEPT_ALL);
+		if (!pListener->Listen()) {
+			CUtils::PrintStatus(false, FormatBindError());
+			bSuccess = false;
+		} else
+			CUtils::PrintStatus(true);
+		delete pListener;
+	} while (!bSuccess);
 
 	if (!sListenHost.empty()) {
 		sListenHost += " ";
@@ -707,48 +727,47 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 		CUtils::PrintMessage("-- Global Modules --");
 		CUtils::PrintMessage("");
 
-		if (CUtils::GetBoolInput("Do you want to load any global modules?")) {
-			CTable Table;
-			Table.AddColumn("Name");
-			Table.AddColumn("Description");
-			set<CModInfo>::iterator it;
+		CTable Table;
+		Table.AddColumn("Name");
+		Table.AddColumn("Description");
+		set<CModInfo>::iterator it;
 
-			for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); ++it) {
-				const CModInfo& Info = *it;
-				Table.AddRow();
-				Table.SetCell("Name", Info.GetName());
-				Table.SetCell("Description", Info.GetDescription().Ellipsize(128));
-			}
+		for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); ++it) {
+			const CModInfo& Info = *it;
+			Table.AddRow();
+			Table.SetCell("Name", Info.GetName());
+			Table.SetCell("Description", Info.GetDescription().Ellipsize(128));
+		}
 
-			unsigned int uTableIdx = 0; CString sLine;
-			while (Table.GetLine(uTableIdx++, sLine)) {
-				CUtils::PrintMessage(sLine);
-			}
+		unsigned int uTableIdx = 0; CString sLine;
+		while (Table.GetLine(uTableIdx++, sLine)) {
+			CUtils::PrintMessage(sLine);
+		}
 
-			if (uNrOtherGlobalMods > 0) {
-				CUtils::PrintMessage("And " + CString(uNrOtherGlobalMods) + " other (uncommon) modules. You can enable those later.");
-			}
+		if (uNrOtherGlobalMods > 0) {
+			CUtils::PrintMessage("And " + CString(uNrOtherGlobalMods) + " other (uncommon) modules. You can enable those later.");
+		}
 
-			CUtils::PrintMessage("");
+		CUtils::PrintMessage("");
 
-			for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); ++it) {
-				const CModInfo& Info = *it;
-				CString sName = Info.GetName();
+		for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); ++it) {
+			const CModInfo& Info = *it;
+			CString sName = Info.GetName();
 
-				if (CUtils::StdoutIsTTY()) {
-					if (CUtils::GetBoolInput("Load global module <\033[1m" + sName + "\033[22m>?", false))
-						vsLines.push_back("LoadModule = " + sName);
-				} else {
-					if (CUtils::GetBoolInput("Load global module <" + sName + ">?", false))
-						vsLines.push_back("LoadModule = " + sName);
-				}
+			if (CUtils::StdoutIsTTY()) {
+				if (CUtils::GetBoolInput("Load global module <\033[1m" + sName + "\033[22m>?", false))
+					vsLines.push_back("LoadModule = " + sName);
+			} else {
+				if (CUtils::GetBoolInput("Load global module <" + sName + ">?", false))
+					vsLines.push_back("LoadModule = " + sName);
 			}
 		}
 	}
 
 	// User
 	CUtils::PrintMessage("");
-	CUtils::PrintMessage("Now we need to setup a user...");
+	CUtils::PrintMessage("Now we need to set up a user...");
+	CUtils::PrintMessage("ZNC needs one user per IRC network.");
 	CUtils::PrintMessage("");
 
 	bool bFirstUser = true;
@@ -781,9 +800,9 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 		vsLines.push_back("\tIdent      = " + sAnswer);
 		CUtils::GetInput("Real Name", sAnswer, "Got ZNC?");
 		vsLines.push_back("\tRealName   = " + sAnswer);
-		CUtils::GetInput("VHost", sAnswer, "", "optional");
+		CUtils::GetInput("Bind Host", sAnswer, "", "optional");
 		if (!sAnswer.empty()) {
-			vsLines.push_back("\tVHost      = " + sAnswer);
+			vsLines.push_back("\tBindHost   = " + sAnswer);
 		}
 		// todo: Possibly add motd
 
@@ -814,41 +833,39 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 			CUtils::PrintMessage("-- User Modules --");
 			CUtils::PrintMessage("");
 
-			if (CUtils::GetBoolInput("Do you want to automatically load any user modules for this user?")) {
-				CTable Table;
-				Table.AddColumn("Name");
-				Table.AddColumn("Description");
-				set<CModInfo>::iterator it;
+			CTable Table;
+			Table.AddColumn("Name");
+			Table.AddColumn("Description");
+			set<CModInfo>::iterator it;
 
-				for (it = ssUserMods.begin(); it != ssUserMods.end(); ++it) {
-					const CModInfo& Info = *it;
-					Table.AddRow();
-					Table.SetCell("Name", Info.GetName());
-					Table.SetCell("Description", Info.GetDescription().Ellipsize(128));
-				}
+			for (it = ssUserMods.begin(); it != ssUserMods.end(); ++it) {
+				const CModInfo& Info = *it;
+				Table.AddRow();
+				Table.SetCell("Name", Info.GetName());
+				Table.SetCell("Description", Info.GetDescription().Ellipsize(128));
+			}
 
-				unsigned int uTableIdx = 0; CString sLine;
-				while (Table.GetLine(uTableIdx++, sLine)) {
-					CUtils::PrintMessage(sLine);
-				}
+			unsigned int uTableIdx = 0; CString sLine;
+			while (Table.GetLine(uTableIdx++, sLine)) {
+				CUtils::PrintMessage(sLine);
+			}
 
-				if (uNrOtherUserMods > 0) {
-					CUtils::PrintMessage("And " + CString(uNrOtherUserMods) + " other (uncommon) modules. You can enable those later.");
-				}
+			if (uNrOtherUserMods > 0) {
+				CUtils::PrintMessage("And " + CString(uNrOtherUserMods) + " other (uncommon) modules. You can enable those later.");
+			}
 
-				CUtils::PrintMessage("");
+			CUtils::PrintMessage("");
 
-				for (it = ssUserMods.begin(); it != ssUserMods.end(); ++it) {
-					const CModInfo& Info = *it;
-					CString sName = Info.GetName();
+			for (it = ssUserMods.begin(); it != ssUserMods.end(); ++it) {
+				const CModInfo& Info = *it;
+				CString sName = Info.GetName();
 
-					if (CUtils::StdoutIsTTY()) {
-						if (CUtils::GetBoolInput("Load module <\033[1m" + sName + "\033[22m>?", false))
-							vsLines.push_back("\tLoadModule = " + sName);
-					} else {
-						if (CUtils::GetBoolInput("Load module <" + sName + ">?", false))
-							vsLines.push_back("\tLoadModule = " + sName);
-					}
+				if (CUtils::StdoutIsTTY()) {
+					if (CUtils::GetBoolInput("Load module <\033[1m" + sName + "\033[22m>?", false))
+						vsLines.push_back("\tLoadModule = " + sName);
+				} else {
+					if (CUtils::GetBoolInput("Load module <" + sName + ">?", false))
+						vsLines.push_back("\tLoadModule = " + sName);
 				}
 			}
 		}
@@ -856,6 +873,8 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 		vsLines.push_back("");
 		CUtils::PrintMessage("");
 		CUtils::PrintMessage("-- IRC Servers --");
+		CUtils::PrintMessage("Only add servers from the same IRC network.");
+		CUtils::PrintMessage("If a server from the list can't be reached, another server will be used.");
 		CUtils::PrintMessage("");
 
 		do {
@@ -868,13 +887,13 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 			CUtils::GetInput("[" + sHost + "] Password (probably empty)", sPass);
 
 #ifdef HAVE_LIBSSL
-			bSSL = CUtils::GetBoolInput("Does this server use SSL? (probably no)", false);
+			bSSL = CUtils::GetBoolInput("Does this server use SSL?", false);
 #endif
 
 			vsLines.push_back("\tServer     = " + sHost + ((bSSL) ? " +" : " ") + CString(uServerPort) + " " + sPass);
 
 			CUtils::PrintMessage("");
-		} while (CUtils::GetBoolInput("Would you like to add another server?", false));
+		} while (CUtils::GetBoolInput("Would you like to add another server for this IRC network?", false));
 
 		vsLines.push_back("");
 		CUtils::PrintMessage("");
@@ -898,7 +917,7 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 
 		CUtils::PrintMessage("");
 		bFirstUser = false;
-	} while (CUtils::GetBoolInput("Would you like to setup another user?", false));
+	} while (CUtils::GetBoolInput("Would you like to set up another user (e.g. for connecting to another network)?", false));
 	// !User
 
 	CFile File;
@@ -961,17 +980,20 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 		cout << endl << "----------------------------------------------------------------------------" << endl << endl;
 	}
 
+	const CString sProtocol(sSSL.empty() ? "http" : "https");
 	CUtils::PrintMessage("");
-	CUtils::PrintMessage("To connect to this znc you need to connect to it as your irc server", true);
+	CUtils::PrintMessage("To connect to this ZNC you need to connect to it as your IRC server", true);
 	CUtils::PrintMessage("using the port that you supplied.  You have to supply your login info", true);
-	CUtils::PrintMessage("as the irc server password like so... user:pass.", true);
+	CUtils::PrintMessage("as the IRC server password like this: user:pass.", true);
 	CUtils::PrintMessage("");
 	CUtils::PrintMessage("Try something like this in your IRC client...", true);
-	CUtils::PrintMessage("/server <znc_server_ip> " + CString(uListenPort) + " " + sUser + ":<pass>", true);
+	CUtils::PrintMessage("/server <znc_server_ip> " + sSSL + CString(uListenPort) + " " + sUser + ":<pass>", true);
+	CUtils::PrintMessage("And this in your browser...", true);
+	CUtils::PrintMessage(sProtocol + "://<znc_server_ip>:" + CString(uListenPort) + "/", true);
 	CUtils::PrintMessage("");
 
 	m_LockFile.UnLock();
-	return bFileOpen && CUtils::GetBoolInput("Launch znc now?", true);
+	return bFileOpen && CUtils::GetBoolInput("Launch ZNC now?", true);
 }
 
 size_t CZNC::FilterUncommonModules(set<CModInfo>& ssModules) {
@@ -1005,11 +1027,7 @@ bool CZNC::ParseConfig(const CString& sConfig)
 
 bool CZNC::RehashConfig(CString& sError)
 {
-	GetModules().OnPreRehash();
-	for (map<CString, CUser*>::iterator itb = m_msUsers.begin();
-			itb != m_msUsers.end(); ++itb) {
-		itb->second->GetModules().OnPreRehash();
-	}
+	ALLMODULECALL(OnPreRehash(), NOTHING);
 
 	// This clears m_msDelUsers
 	HandleUserDeletion();
@@ -1019,11 +1037,7 @@ bool CZNC::RehashConfig(CString& sError)
 	m_msUsers.clear();
 
 	if (DoRehash(sError)) {
-		GetModules().OnPostRehash();
-		for (map<CString, CUser*>::iterator it = m_msUsers.begin();
-				it != m_msUsers.end(); ++it) {
-			it->second->GetModules().OnPostRehash();
-		}
+		ALLMODULECALL(OnPostRehash(), NOTHING);
 
 		return true;
 	}
@@ -1047,7 +1061,7 @@ bool CZNC::DoRehash(CString& sError)
 	if (!CFile::Exists(m_sConfigFile)) {
 		sError = "No such file";
 		CUtils::PrintStatus(false, sError);
-		CUtils::PrintMessage("Restart znc with the --makeconf option if you wish to create this config.");
+		CUtils::PrintMessage("Restart ZNC with the --makeconf option if you wish to create this config.");
 		return false;
 	}
 
@@ -1061,7 +1075,9 @@ bool CZNC::DoRehash(CString& sError)
 	if (m_LockFile.IsOpen())
 		m_LockFile.Close();
 
-	if (!m_LockFile.Open(m_sConfigFile)) {
+	// need to open the config file Read/Write for fcntl()
+	// exclusive locking to work properly!
+	if (!m_LockFile.Open(m_sConfigFile, O_RDWR)) {
 		sError = "Can not open config file";
 		CUtils::PrintStatus(false, sError);
 		return false;
@@ -1084,7 +1100,7 @@ bool CZNC::DoRehash(CString& sError)
 
 	CUtils::PrintStatus(true);
 
-	m_vsVHosts.clear();
+	m_vsBindHosts.clear();
 	m_vsMotd.clear();
 
 	// Delete all listeners
@@ -1106,8 +1122,9 @@ bool CZNC::DoRehash(CString& sError)
 	while (File.ReadLine(sLine)) {
 		uLineNum++;
 
-		// Remove all leading / trailing spaces and line endings
-		sLine.Trim();
+		// Remove all leading spaces and trailing line endings
+		sLine.TrimLeft();
+		sLine.TrimRight("\r\n");
 
 		if ((sLine.empty()) || (sLine[0] == '#') || (sLine.Left(2) == "//")) {
 			continue;
@@ -1251,14 +1268,21 @@ bool CZNC::DoRehash(CString& sError)
 		// If we have a regular line, figure out where it goes
 		CString sName = sLine.Token(0, false, "=");
 		CString sValue = sLine.Token(1, true, "=");
+
+		// Only remove the first space, people might want
+		// leading spaces (e.g. in the MOTD).
+		if (sValue.Left(1) == " ")
+			sValue.LeftChomp();
+
+		// We don't have any names with spaces, trim all
+		// leading/trailing spaces.
 		sName.Trim();
-		sValue.Trim();
 
 		if ((!sName.empty()) && (!sValue.empty())) {
 			if (pUser) {
 				if (pChan) {
 					if (sName.Equals("Buffer")) {
-						pChan->SetBufferCount(sValue.ToUInt());
+						pChan->SetBufferCount(sValue.ToUInt(), true);
 						continue;
 					} else if (sName.Equals("KeepBuffer")) {
 						pChan->SetKeepBuffer(sValue.Equals("true"));
@@ -1280,7 +1304,7 @@ bool CZNC::DoRehash(CString& sError)
 					}
 				} else {
 					if (sName.Equals("Buffer")) {
-						pUser->SetBufferCount(sValue.ToUInt());
+						pUser->SetBufferCount(sValue.ToUInt(), true);
 						continue;
 					} else if (sName.Equals("KeepBuffer")) {
 						pUser->SetKeepBuffer(sValue.Equals("true"));
@@ -1352,8 +1376,8 @@ bool CZNC::DoRehash(CString& sError)
 					} else if (sName.Equals("Admin")) {
 						pUser->SetAdmin(sValue.Equals("true"));
 						continue;
-					} else if (sName.Equals("DenySetVHost")) {
-						pUser->SetDenySetVHost(sValue.Equals("true"));
+					} else if (sName.Equals("DenySetBindHost") || sName.Equals("DenySetVHost")) {
+						pUser->SetDenySetBindHost(sValue.Equals("true"));
 						continue;
 					} else if (sName.Equals("StatusPrefix")) {
 						if (!pUser->SetStatusPrefix(sValue)) {
@@ -1376,11 +1400,11 @@ bool CZNC::DoRehash(CString& sError)
 					} else if (sName.Equals("ChanModes")) {
 						pUser->SetDefaultChanModes(sValue);
 						continue;
-					} else if (sName.Equals("VHost")) {
-						pUser->SetVHost(sValue);
+					} else if (sName.Equals("BindHost") || sName.Equals("VHost")) {
+						pUser->SetBindHost(sValue);
 						continue;
-					} else if (sName.Equals("DCCVHost")) {
-						pUser->SetDCCVHost(sValue);
+					} else if (sName.Equals("DCCBindHost") || sName.Equals("DCCVHost")) {
+						pUser->SetDCCBindHost(sValue);
 						continue;
 					} else if (sName.Equals("Allow")) {
 						pUser->AddAllowedHost(sValue);
@@ -1561,8 +1585,7 @@ bool CZNC::DoRehash(CString& sError)
 					CListener* pListener = new CListener(uPort, sBindHost, bSSL, eAddr, eAccept);
 
 					if (!pListener->Listen()) {
-						sError = (errno == 0 ? CString("unknown error, check the host name") : CString(strerror(errno)));
-						sError = "Unable to bind [" + sError + "]";
+						sError = FormatBindError();
 						CUtils::PrintStatus(false, sError);
 						delete pListener;
 						return false;
@@ -1582,23 +1605,56 @@ bool CZNC::DoRehash(CString& sError)
 						CUtils::PrintError(sError);
 						return false;
 					}
+					CString sModRet;
+					CModule *pOldMod;
+
+					pOldMod = GetModules().FindModule(sModName);
+					if (!pOldMod) {
+						CUtils::PrintAction("Loading Global Module [" + sModName + "]");
+
+						bool bModRet = GetModules().LoadModule(sModName, sArgs, NULL, sModRet);
+
+						// If the module was loaded, sModRet contains
+						// "Loaded Module [name] ..." and we strip away this beginning.
+						if (bModRet)
+							sModRet = sModRet.Token(1, true, sModName + "] ");
+
+						CUtils::PrintStatus(bModRet, sModRet);
+						if (!bModRet) {
+							sError = sModRet;
+							return false;
+						}
+					} else if (pOldMod->GetArgs() != sArgs) {
+						CUtils::PrintAction("Reloading Global Module [" + sModName + "]");
+
+						bool bModRet = GetModules().ReloadModule(sModName, sArgs, NULL, sModRet);
+
+						// If the module was loaded, sModRet contains
+						// "Loaded Module [name] ..." and we strip away this beginning.
+						if (bModRet)
+							sModRet = sModRet.Token(1, true, sModName + "] ");
+
+						CUtils::PrintStatus(bModRet, sModRet);
+						if (!bModRet) {
+							sError = sModRet;
+							return false;
+						}
+					} else
+						CUtils::PrintMessage("Module [" + sModName + "] already loaded.");
+
 					msModules[sModName] = sArgs;
 					continue;
 				} else if (sName.Equals("ISpoofFormat")) {
 					m_sISpoofFormat = sValue;
 					continue;
 				} else if (sName.Equals("ISpoofFile")) {
-					if (sValue.Left(2) == "~/") {
-						sValue.LeftChomp(2);
-						sValue = GetHomePath() + "/" + sValue;
-					}
 					m_sISpoofFile = sValue;
 					continue;
 				} else if (sName.Equals("MOTD")) {
 					AddMotd(sValue);
 					continue;
-				} else if (sName.Equals("VHost")) {
-					AddVHost(sValue);
+				} else if (sName.Equals("BindHost") || sName.Equals("VHost")) {
+					AddBindHost(sValue);
 					continue;
 				} else if (sName.Equals("PidFile")) {
 					m_sPidFile = sValue;
@@ -1618,6 +1674,12 @@ bool CZNC::DoRehash(CString& sError)
 				} else if (sName.Equals("AnonIPLimit")) {
 					m_uiAnonIPLimit = sValue.ToUInt();
 					continue;
+				} else if (sName.Equals("MaxBufferSize")) {
+					m_uiMaxBufferSize = sValue.ToUInt();
+					continue;
+				} else if (sName.Equals("SSLCertFile")) {
+					m_sSSLCertFile = sValue;
+					continue;
 				}
 			}
 
@@ -1640,49 +1702,7 @@ bool CZNC::DoRehash(CString& sError)
 		}
 	}
 
-	// First step: Load and reload new modules or modules with new arguments
-	for (MCString::iterator it = msModules.begin(); it != msModules.end(); ++it) {
-		CString sModName = it->first;
-		CString sArgs = it->second;
-		CString sModRet;
-		CModule *pOldMod;
-
-		pOldMod = GetModules().FindModule(sModName);
-		if (!pOldMod) {
-			CUtils::PrintAction("Loading Global Module [" + sModName + "]");
-
-			bool bModRet = GetModules().LoadModule(sModName, sArgs, NULL, sModRet);
-
-			// If the module was loaded, sModRet contains
-			// "Loaded Module [name] ..." and we strip away this beginning.
-			if (bModRet)
-				sModRet = sModRet.Token(1, true, sModName + "] ");
-
-			CUtils::PrintStatus(bModRet, sModRet);
-			if (!bModRet) {
-				sError = sModRet;
-				return false;
-			}
-		} else if (pOldMod->GetArgs() != sArgs) {
-			CUtils::PrintAction("Reloading Global Module [" + sModName + "]");
-
-			bool bModRet = GetModules().ReloadModule(sModName, sArgs, NULL, sModRet);
-
-			// If the module was loaded, sModRet contains
-			// "Loaded Module [name] ..." and we strip away this beginning.
-			if (bModRet)
-				sModRet = sModRet.Token(1, true, sModName + "] ");
-
-			CUtils::PrintStatus(bModRet, sModRet);
-			if (!bModRet) {
-				sError = sModRet;
-				return false;
-			}
-		} else
-			CUtils::PrintMessage("Module [" + sModName + "] already loaded.");
-	}
-
-	// Second step: Unload modules which are no longer in the config
+	// Unload modules which are no longer in the config
 	set<CString> ssUnload;
 	for (size_t i = 0; i < GetModules().size(); i++) {
 		CModule *pCurMod = GetModules()[i];
@@ -1707,7 +1727,7 @@ bool CZNC::DoRehash(CString& sError)
 		if (it->m_pUser) {
 			MODULECALL(OnConfigLine(it->m_sName, it->m_sValue, it->m_pUser, it->m_pChan), it->m_pUser, NULL, bHandled = true);
 		} else {
-			bHandled = GetModules().OnConfigLine(it->m_sName, it->m_sValue, it->m_pUser, it->m_pChan);
+			GLOBALMODULECALL(OnConfigLine(it->m_sName, it->m_sValue, it->m_pUser, it->m_pChan), it->m_pUser, NULL, bHandled = true);
 		}
 		if (!bHandled) {
 			CUtils::PrintMessage("unhandled global module config line [GM:" + it->m_sName + "] = [" + it->m_sValue + "]");
@@ -1715,13 +1735,17 @@ bool CZNC::DoRehash(CString& sError)
 	}
 
 	if (pChan) {
-		// TODO last <Chan> not closed
+		sError = "Last <Chan> section not properly closed. File truncated?";
+		CUtils::PrintError(sError);
 		delete pChan;
+		return false;
 	}
 
 	if (pUser) {
-		// TODO last <User> not closed
+		sError = "Last <User> section not properly closed. File truncated?";
+		CUtils::PrintError(sError);
 		delete pUser;
+		return false;
 	}
 
 	if (m_msUsers.empty()) {
@@ -1744,30 +1768,30 @@ bool CZNC::DoRehash(CString& sError)
 	return true;
 }
 
-void CZNC::ClearVHosts() {
-	m_vsVHosts.clear();
+void CZNC::ClearBindHosts() {
+	m_vsBindHosts.clear();
 }
 
-bool CZNC::AddVHost(const CString& sHost) {
+bool CZNC::AddBindHost(const CString& sHost) {
 	if (sHost.empty()) {
 		return false;
 	}
 
-	for (unsigned int a = 0; a < m_vsVHosts.size(); a++) {
-		if (m_vsVHosts[a].Equals(sHost)) {
+	for (unsigned int a = 0; a < m_vsBindHosts.size(); a++) {
+		if (m_vsBindHosts[a].Equals(sHost)) {
 			return false;
 		}
 	}
 
-	m_vsVHosts.push_back(sHost);
+	m_vsBindHosts.push_back(sHost);
 	return true;
 }
 
-bool CZNC::RemVHost(const CString& sHost) {
+bool CZNC::RemBindHost(const CString& sHost) {
 	VCString::iterator it;
-	for (it = m_vsVHosts.begin(); it != m_vsVHosts.end(); ++it) {
+	for (it = m_vsBindHosts.begin(); it != m_vsBindHosts.end(); ++it) {
 		if (sHost.Equals(*it)) {
-			m_vsVHosts.erase(it);
+			m_vsBindHosts.erase(it);
 			return true;
 		}
 	}
@@ -1840,11 +1864,11 @@ bool CZNC::AddUser(CUser* pUser, CString& sErrorRet) {
 				<< sErrorRet << "]");
 		return false;
 	}
-	if (GetModules().OnAddUser(*pUser, sErrorRet)) {
+	GLOBALMODULECALL(OnAddUser(*pUser, sErrorRet), pUser, NULL,
 		DEBUG("AddUser [" << pUser->GetUserName() << "] aborted by a module ["
 			<< sErrorRet << "]");
 		return false;
-	}
+	);
 	m_msUsers[pUser->GetUserName()] = pUser;
 	return true;
 }
@@ -1941,11 +1965,9 @@ CZNC::TrafficStatsMap CZNC::GetTrafficStats(TrafficStatsPair &Users,
 
 void CZNC::AuthUser(CSmartPtr<CAuthBase> AuthClass) {
 	// TODO unless the auth module calls it, CUser::IsHostAllowed() is not honoured
-	if (GetModules().OnLoginAttempt(AuthClass)) {
-		return;
-	}
+	GLOBALMODULECALL(OnLoginAttempt(AuthClass), NULL, NULL, return);
 
-	CUser* pUser = GetUser(AuthClass->GetUsername());
+	CUser* pUser = FindUser(AuthClass->GetUsername());
 
 	if (!pUser || !pUser->CheckPass(AuthClass->GetPassword())) {
 		AuthClass->RefuseLogin("Invalid Password");
@@ -2042,6 +2064,13 @@ private:
 	size_t m_uiPosNextUser;
 };
 
+void CZNC::SetConnectDelay(unsigned int i) {
+	if (m_uiConnectDelay != i && m_pConnectUserTimer != NULL) {
+		m_pConnectUserTimer->Start(i);
+	}
+	m_uiConnectDelay = i;
+}
+
 void CZNC::EnableConnectUser() {
 	if (m_pConnectUserTimer != NULL)
 		return;
@@ -2063,3 +2092,7 @@ void CZNC::LeakConnectUser(CConnectUserTimer *pTimer) {
 	if (m_pConnectUserTimer == pTimer)
 		m_pConnectUserTimer = NULL;
 }
+
+bool CZNC::WaitForChildLock() {
+	return m_LockFile.ExLock();
+}
